package com.deepclone.lw.beans.admin;


import java.net.InetAddress;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.admin.adata.AdminOverview;
import com.deepclone.lw.interfaces.admin.AdminDAO;
import com.deepclone.lw.interfaces.admin.AdminDAOException;
import com.deepclone.lw.sqld.admin.AdminConnection;
import com.deepclone.lw.sqld.admin.AdminRecord;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.StoredProc;



public class AdminDAOBean
		implements AdminDAO
{
	private static final String sGetAdminById = "SELECT * FROM admin.admins_view WHERE administrator_id = ?";
	private static final String sGetAdminByAddress = "SELECT * FROM admin.admins_view WHERE address = ?";
	private static final String sGetAdminByName = "SELECT * FROM admin.admins_view WHERE lower( name ) = ?";
	private static final String sListAdministrators = "SELECT * FROM admin.admins_view ORDER BY active DESC , name";

	private JdbcTemplate dTemplate;

	private final RowMapper< AdminRecord > mAdminRecord;
	private final RowMapper< AdminOverview > mAdminOverview;

	private StoredProc fCreateAdmin;
	private StoredProc fLogConnection;
	private StoredProc fLogDisconnection;
	private StoredProc fSetPassword;
	private StoredProc fResetPassword;
	private StoredProc fSetPrivileges;


	public AdminDAOBean( )
	{
		this.mAdminRecord = new RowMapper< AdminRecord >( ) {
			@Override
			public AdminRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AdminRecord admin = new AdminRecord( );
				admin.setId( rs.getInt( "administrator_id" ) );
				admin.setName( rs.getString( "name" ) );
				admin.setPrivileges( rs.getInt( "privileges" ) );
				admin.setActive( rs.getBoolean( "active" ) );
				admin.setAccount( (Integer) rs.getObject( "account_id" ) );
				admin.setAddress( rs.getString( "address" ) );
				admin.setpMd5( rs.getString( "pass_md5" ) );
				admin.setpSha1( rs.getString( "pass_sha1" ) );
				admin.setPassChangeRequired( (Boolean) rs.getObject( "pass_change_required" ) );
				return admin;
			}
		};
		this.mAdminOverview = new RowMapper< AdminOverview >( ) {
			@Override
			public AdminOverview mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AdminOverview ov = new AdminOverview( );
				ov.setId( rs.getInt( "admin_id" ) );
				ov.setNewMessages( rs.getLong( "new_messages" ) );
				ov.setPendingNames( rs.getLong( "pending_names" ) );
				ov.setPendingBans( rs.getLong( "pending_bans" ) );
				ov.setPendingBugs( rs.getLong( "pending_bugs" ) );
				ov.setOpenBugs( rs.getLong( "open_bugs" ) );
				ov.setUpdatedBugs( rs.getLong( "updated_bugs" ) );
				return ov;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fCreateAdmin = new StoredProc( dataSource , "admin" , "create_admin" );
		this.fCreateAdmin.addParameter( "address" , Types.VARCHAR );
		this.fCreateAdmin.addParameter( "admin_name" , Types.VARCHAR );
		this.fCreateAdmin.addParameter( "privileges" , Types.INTEGER );
		this.fCreateAdmin.addOutput( "err_code" , Types.INTEGER );
		this.fCreateAdmin.addOutput( "admin_id" , Types.INTEGER );

		this.fLogConnection = new StoredProc( dataSource , "admin" , "log_connection" );
		this.fLogConnection.addParameter( "admin_id" , Types.INTEGER );
		this.fLogConnection.addParameter( "result" , "admin_connection_result" );
		this.fLogConnection.addParameter( "ip_address" , Types.VARCHAR );

		this.fLogDisconnection = new StoredProc( dataSource , "admin" , "log_disconnection" );
		this.fLogDisconnection.addParameter( "admin_id" , Types.INTEGER );

		this.fSetPassword = new StoredProc( dataSource , "admin" , "set_password" );
		this.fSetPassword.addParameter( "admin_id" , Types.INTEGER );
		this.fSetPassword.addParameter( "pass_sha1" , Types.VARCHAR );
		this.fSetPassword.addParameter( "pass_md5" , Types.VARCHAR );
		this.fSetPassword.addOutput( "success" , Types.BOOLEAN );

		this.fResetPassword = new StoredProc( dataSource , "admin" , "reset_password" );
		this.fResetPassword.addParameter( "admin_id" , Types.INTEGER );
		this.fResetPassword.addParameter( "superuser_id" , Types.INTEGER );
		this.fResetPassword.addOutput( "success" , Types.BOOLEAN );

		this.fSetPrivileges = new StoredProc( dataSource , "admin" , "set_privileges" );
		this.fSetPrivileges.addParameter( "admin_id" , Types.INTEGER );
		this.fSetPrivileges.addParameter( "superuser_id" , Types.INTEGER );
		this.fSetPrivileges.addParameter( "privileges" , Types.INTEGER );
		this.fSetPrivileges.addOutput( "success" , Types.BOOLEAN );
	}


	@Override
	public AdminRecord getAdmin( int id )
	{
		try {
			return this.dTemplate.queryForObject( sGetAdminById , this.mAdminRecord , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public AdminRecord getAdmin( EmailAddress address )
	{
		try {
			return this.dTemplate.queryForObject( sGetAdminByAddress , this.mAdminRecord , address.getAddress( ) );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public AdminRecord getAdmin( String name )
	{
		try {
			return this.dTemplate.queryForObject( sGetAdminByName , this.mAdminRecord , name.toLowerCase( ) );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public int createAdmin( String address , String name , int privileges )
			throws AdminDAOException
	{
		Map< String , Object > result = this.fCreateAdmin.execute( address , name , privileges );
		int errCode = (Integer) result.get( "err_code" );
		if ( errCode != 0 ) {
			throw new AdminDAOException( errCode );
		}
		return (Integer) result.get( "admin_id" );
	}


	@Override
	public void logConnectionAttempt( int id , AdminConnection status , InetAddress address )
	{
		this.fLogConnection.execute( id , status.toString( ) , address.getHostAddress( ) );
	}


	@Override
	public void logDisconnection( int id )
	{
		this.fLogDisconnection.execute( id );
	}


	@Override
	public boolean setPassword( int id , String sha1 , String md5 )
	{
		return (Boolean) this.fSetPassword.execute( id , sha1 , md5 ).get( "success" );
	}


	@Override
	public List< AdminRecord > listAdministrators( )
	{
		return this.dTemplate.query( sListAdministrators , this.mAdminRecord );
	}


	@Override
	public boolean resetPassword( int identifier , int superuser )
	{
		return (Boolean) this.fResetPassword.execute( identifier , superuser ).get( "success" );
	}


	@Override
	public boolean setPrivileges( int identifier , int superuser , int privileges )
	{
		return (Boolean) this.fSetPrivileges.execute( identifier , superuser , privileges ).get( "success" );
	}


	@Override
	public AdminOverview getOverview( int id )
	{
		String sql = "SELECT * FROM admin.overview WHERE admin_id = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.mAdminOverview , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public List< AdminOverview > getOverviews( )
	{
		String sql = "SELECT * FROM admin.overview";
		return this.dTemplate.query( sql , this.mAdminOverview );
	}


	@Override
	public Timestamp isRecapTime( )
	{
		String sql = "SELECT * FROM admin.is_recap_time( )";
		RowMapper< Timestamp > mapper = new RowMapper< Timestamp >( ) {
			@Override
			public Timestamp mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return rs.getTimestamp( 1 );
			}
		};
		return this.dTemplate.queryForObject( sql , mapper );
	}

}
