package com.deepclone.lw.beans.bt;


import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.bt.data.*;
import com.deepclone.lw.cmd.player.bt.*;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.interfaces.bt.*;
import com.deepclone.lw.interfaces.game.EmpireManagement;



@Transactional
public class PlayerBugsBean
		implements PlayerBugs
{
	private EmpireManagement empireMgr;
	private BugsDAO bugsDao;
	private EmpireSummary summary;


	@Autowired( required = true )
	public void setBugsDao( BugsDAO bugsDao )
	{
		this.bugsDao = bugsDao;
	}


	@Autowired( required = true )
	public void setEmpireMgr( EmpireManagement empireMgr )
	{
		this.empireMgr = empireMgr;
	}


	@Autowired( required = true )
	public void setSummary( EmpireSummary summary )
	{
		this.summary = summary;
	}


	@Override
	public ListBugsResponse getBugs( int empireId , BugStatus status , boolean ownOnly , long first , int count )
	{
		long nBugs = this.bugsDao.countReports( empireId , status , ownOnly );
		List< BugReport > bugs = this.bugsDao.getReports( empireId , status , ownOnly , first , count );
		GamePageData page = this.empireMgr.getGeneralInformation( empireId );
		return new ListBugsResponse( page , status , ownOnly , first , count , nBugs , bugs );
	}


	@Override
	public ReportBugResponse postReport( int empireId , String title , String desc )
	{
		long bugId = this.bugsDao.postReport( empireId , title , desc , this.summary.getSummary( empireId ) );
		return new ReportBugResponse( this.empireMgr.getGeneralInformation( empireId ) , bugId );
	}


	@Override
	public ViewBugResponse getReport( int empireId , long bugId )
	{
		GamePageData page = this.empireMgr.getGeneralInformation( empireId );

		BugReport report = this.bugsDao.getReport( empireId , bugId );
		if ( report == null ) {
			return new ViewBugResponse( page );
		}

		List< BugEvent > events = this.bugsDao.getEvents( bugId );
		Iterator< BugEvent > it = events.iterator( );
		BugStatus status = BugStatus.PENDING;
		boolean pub = false;
		while ( it.hasNext( ) ) {
			BugEvent event = it.next( );
			BugEventType type = event.getType( );
			if ( type == BugEventType.STATUS ) {
				if ( event.getStatus( ) == status ) {
					it.remove( );
				} else {
					status = event.getStatus( );
				}
			} else if ( type == BugEventType.VISIBILITY ) {
				if ( event.getVisible( ) == pub ) {
					it.remove( );
				} else {
					pub = event.getVisible( );
				}
			} else if ( type == BugEventType.COMMENT && !event.getVisible( ) ) {
				BugSubmitter submitter = event.getSubmitter( );
				if ( submitter.isAdmin( ) || submitter.getUserId( ) == null
						|| !page.getEmpire( ).equals( submitter.getName( ) ) ) {
					it.remove( );
				}
			}
		}
		return new ViewBugResponse( page , report , events );
	}


	@Override
	public PostCommentResponse postComment( int empireId , long reportId , String comment )
	{
		this.bugsDao.postComment( empireId , reportId , comment );
		return new PostCommentResponse( this.empireMgr.getGeneralInformation( empireId ) , true );
	}

}
