package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.AllianceInformation;
import com.deepclone.lw.beans.bt.es.data.DebugInformation;



/**
 * Top-level empire summary row mapper
 * 
 * <p>
 * This class maps an entry from the database's main empire summary view,
 * <code>bugs.dump_main_view</code>, into a top-level {@link DebugInformation} instance.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class DebugInformationMapper
		implements RowMapper< DebugInformation >
{
	/**
	 * Map a row from <code>bugs.dump_main_view</code> into a {@link DebugInformation} instance
	 * 
	 * <p>
	 * Converts the row into a {@link DebugInformation} instance containing system information,
	 * account information and empire information. If the row contains alliance information, add an
	 * {@link AllianceInformation} record to the empire information.
	 */
	@Override
	public DebugInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		DebugInformation di = new DebugInformation( );

		di.getSystem( ).setNextTick( rs.getLong( "next_tick" ) );
		di.getSystem( ).setCurrentTick( (Long) rs.getObject( "current_tick" ) );

		di.getAccount( ).setId( rs.getInt( "account_id" ) );
		di.getAccount( ).setAddress( rs.getString( "account_address" ) );
		di.getAccount( ).setGameCredits( rs.getInt( "game_credits" ) );
		di.getAccount( ).setStatus( rs.getString( "account_status" ) );
		di.getAccount( ).setLanguage( rs.getString( "account_language" ) );

		di.getEmpire( ).setId( rs.getInt( "empire_id" ) );
		di.getEmpire( ).setName( rs.getString( "empire_name" ) );
		di.getEmpire( ).setCash( rs.getDouble( "cash" ) );

		String allianceTag = rs.getString( "alliance_tag" );
		if ( allianceTag != null ) {
			AllianceInformation alliance = new AllianceInformation( );
			alliance.setId( rs.getInt( "alliance_id" ) );
			alliance.setTag( allianceTag );
			alliance.setPending( rs.getBoolean( "alliance_pending" ) );
			di.getEmpire( ).setAlliance( alliance );
		}

		return di;
	}
}