package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.FleetInformation;
import com.deepclone.lw.beans.bt.es.data.MovementInformation;



/**
 * Fleet information row mapper
 * 
 * <p>
 * This class is responsible for mapping rows from <code>bugs.dump_fleets_view</code> into
 * {@link FleetInformation} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class FleetInformationMapper
		implements RowMapper< FleetInformation >
{

	/**
	 * Map a row from <code>bugs.dump_fleets_view</code>
	 * 
	 * <p>
	 * Generate a {@link FleetInformation} instance from the row's contents. If the row indicates
	 * that the fleet is moving, add a {@link MovementInformation} instance to the fleet information
	 * record.
	 */
	@Override
	public FleetInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		FleetInformation fi = new FleetInformation( );
		fi.setId( rs.getLong( "fleet_id" ) );
		fi.setName( rs.getString( "fleet_name" ) );
		fi.setStatus( rs.getString( "status" ) );
		fi.setAttacking( rs.getBoolean( "attacking" ) );
		fi.setLocationId( rs.getInt( "location_id" ) );
		fi.setLocationName( rs.getString( "location_name" ) );

		Integer sourceId = (Integer) rs.getObject( "source_id" );
		if ( sourceId != null ) {
			MovementInformation mi = new MovementInformation( );
			mi.setSourceId( sourceId );
			mi.setSourceName( rs.getString( "source_name" ) );
			mi.setTimeLeft( rs.getInt( "time_left" ) );
			mi.setStateTimeLeft( rs.getInt( "state_time_left" ) );
			mi.setNearId( (Integer) rs.getObject( "ref_point_id" ) );
			mi.setNearName( rs.getString( "ref_point_name" ) );
			mi.setOutwards( (Boolean) rs.getObject( "outwards" ) );
			mi.setPastRefPoint( (Boolean) rs.getObject( "past_ref_point" ) );
			mi.setStartX( (Float) rs.getObject( "start_x" ) );
			mi.setStartY( (Float) rs.getObject( "start_y" ) );
			fi.setMovement( mi );
		}

		return fi;
	}
}