package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.PlanetInformation;



/**
 * Top-level planet information row mapper
 * 
 * <p>
 * This class maps rows from <code>bugs.dump_planets_view</code> into {@link PlanetInformation}
 * instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class PlanetInformationMapper
		implements RowMapper< PlanetInformation >
{

	/**
	 * Map a row from <code>bugs.dump_planets_view</code>
	 * 
	 * <p>
	 * Create a new {@link PlanetInformation} instance from the row's contents. The instance's
	 * civilian and military construction queues, while left mostly uninitialised, will have their
	 * money and work accumulators set.
	 */
	@Override
	public PlanetInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		PlanetInformation pi = new PlanetInformation( );
		pi.setId( rs.getInt( "planet_id" ) );
		pi.setPopulation( rs.getDouble( "population" ) );
		pi.setCurrentHappiness( rs.getDouble( "current_happiness" ) );
		pi.setTargetHappiness( rs.getDouble( "target_happiness" ) );
		pi.getCivilianQueue( ).setAccMoney( rs.getDouble( "civ_money" ) );
		pi.getCivilianQueue( ).setAccWork( rs.getDouble( "civ_work" ) );
		pi.getMilitaryQueue( ).setAccMoney( rs.getDouble( "mil_money" ) );
		pi.getMilitaryQueue( ).setAccWork( rs.getDouble( "mil_work" ) );
		return pi;
	}
}