package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.QueueItemInformation;



/**
 * Generic row mapper for construction queue items
 * 
 * <p>
 * This class is responsible for mapping rows from <code>bugs.dump_queues_view</code> into
 * {@link QueueItemInformation} instances. These instances will be added to the right queue later.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class QueueItemInformationMapper
		implements RowMapper< QueueItemInformation >
{

	/**
	 * Map a row from <code>bugs.dump_queues_view</code>
	 * 
	 * <p>
	 * Create a {@link QueueItemInformation} instance from the row's contents.
	 */
	@Override
	public QueueItemInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		QueueItemInformation qii = new QueueItemInformation( );
		qii.setPlanetId( rs.getInt( "planet_id" ) );
		qii.setMilitary( rs.getBoolean( "military" ) );
		qii.setId( rs.getInt( "item_id" ) );
		qii.setName( rs.getString( "item_name" ) );
		qii.setDestroy( rs.getBoolean( "destroy" ) );
		qii.setAmount( rs.getInt( "amount" ) );
		return qii;
	}
}