package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.ShipsInformation;



/**
 * Fleet ships information row mapper
 * 
 * <p>
 * This class is responsible for creating {@link ShipsInformation} instances from the contents of
 * <code>bugs.dump_ships_view</code>.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class ShipsInformationMapper
		implements RowMapper< ShipsInformation >
{

	/**
	 * Map a row from <code>bugs.dump_ships_view</code>
	 * 
	 * <p>
	 * Generate a {@link ShipsInformation} instance and initialise its fields based on the row's
	 * contents.
	 */
	@Override
	public ShipsInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		ShipsInformation si = new ShipsInformation( );
		si.setFleetId( rs.getLong( "fleet_id" ) );
		si.setId( rs.getInt( "ship_id" ) );
		si.setName( rs.getString( "ship_name" ) );
		si.setAmount( rs.getInt( "amount" ) );
		si.setDamage( rs.getDouble( "damage" ) );
		return si;
	}
}