package com.deepclone.lw.beans.bt.es.data;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Empire information record for XML dumps
 * 
 * <p>
 * This class regroups all "main" empire information in XML dumps. This includes the empire's name
 * and identifier, details about the alliance, and the list of resources.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
public class EmpireInformation
		implements Serializable
{

	/** The empire's numeric identifier */
	@XStreamAsAttribute
	private Integer id;

	/** The empire's name */
	@XStreamAsAttribute
	private String name;

	/** The empire's cash */
	@XStreamAsAttribute
	private Double cash;

	/**
	 * The alliance the empire belongs to or has requested membership of (or <code>null</code> if
	 * the empire is neither in an alliance nor requesting to join one)
	 */
	private AllianceInformation alliance;

	/** The empire's resources */
	private final ArrayList< EmpireResourceInformation > resources = new ArrayList< EmpireResourceInformation >( );


	/** @return the empire's numeric identifier */
	public Integer getId( )
	{
		return this.id;
	}


	/**
	 * Set the empire's numeric identifier
	 * 
	 * @param id
	 *            the empire's numeric identifier
	 */
	public void setId( int id )
	{
		this.id = id;
	}


	/** @return the empire's name */
	public String getName( )
	{
		return this.name;
	}


	/**
	 * Set the empire's name
	 * 
	 * @param name
	 *            the empire's name
	 */
	public void setName( String name )
	{
		this.name = name;
	}


	/** @return the amount of cash possessed by the empire */
	public Double getCash( )
	{
		return this.cash;
	}


	/**
	 * Set the amount of cash possessed by the empire
	 * 
	 * @param cash
	 *            the amount of cash possessed by the empire
	 */
	public void setCash( double cash )
	{
		this.cash = cash;
	}


	/**
	 * @return the alliance the empire belongs to or has requested membership of, or
	 *         <code>null</code> if the empire is neither in an alliance nor requesting to join one
	 */
	public AllianceInformation getAlliance( )
	{
		return this.alliance;
	}


	/**
	 * Set the information about the alliance
	 * 
	 * @param alliance
	 *            the information about the alliance
	 */
	public void setAlliance( AllianceInformation alliance )
	{
		this.alliance = alliance;
	}


	/**
	 * Access the list of resources
	 * 
	 * @return the list of resources, even if none was set in the file the instance was loaded from.
	 */
	public List< EmpireResourceInformation > getResources( )
	{
		if ( this.resources == null ) {
			return Collections.emptyList( );
		}
		return this.resources;
	}

}
