package com.deepclone.lw.beans.bt.es.data;


import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Research line information
 * 
 * <p>
 * This class defines data stored in the dumps for B6M1 research lines. It is no longer actively
 * used as of B6M2, but is kept around so the XML files can still be used if necessary.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@XStreamAlias( "research-line" )
public class ResearchLineInformation
		extends ResearchInformation
{

	private static final long serialVersionUID = 1L;

	/** Identifier of the technology line */
	@XStreamAsAttribute
	@XStreamAlias( "line" )
	private int id;

	/** Current level for that line */
	@XStreamAsAttribute
	@XStreamAlias( "level" )
	private int currentLevel;

	/** Name of the last level */
	@XStreamAsAttribute
	@XStreamAlias( "name" )
	private String levelName;

	/** Accumulated points towards the next level */
	@XStreamAsAttribute
	@XStreamAlias( "accumulated-points" )
	private double accumulated;


	/**
	 * Gets the identifier of the technology line.
	 * 
	 * @return the identifier of the technology line
	 */
	public int getId( )
	{
		return this.id;
	}


	/**
	 * Sets the identifier of the technology line.
	 * 
	 * @param id
	 *            the new identifier of the technology line
	 */
	public void setId( int id )
	{
		this.id = id;
	}


	/**
	 * Gets the current level for that line.
	 * 
	 * @return the current level for that line
	 */
	public int getCurrentLevel( )
	{
		return this.currentLevel;
	}


	/**
	 * Sets the current level for that line.
	 * 
	 * @param currentLevel
	 *            the new level for that line
	 */
	public void setCurrentLevel( int currentLevel )
	{
		this.currentLevel = currentLevel;
	}


	/**
	 * Gets the name of the last level.
	 * 
	 * @return the name of the last level
	 */
	public String getLevelName( )
	{
		return this.levelName;
	}


	/**
	 * Sets the name of the last level.
	 * 
	 * @param levelName
	 *            the new name of the last level
	 */
	public void setLevelName( String levelName )
	{
		this.levelName = levelName;
	}


	/**
	 * Gets the accumulated points towards the next level.
	 * 
	 * @return the accumulated points towards the next level
	 */
	public double getAccumulated( )
	{
		return this.accumulated;
	}


	/**
	 * Sets the accumulated points towards the next level.
	 * 
	 * @param accumulated
	 *            the new accumulated points towards the next level
	 */
	public void setAccumulated( double accumulated )
	{
		this.accumulated = accumulated;
	}

}
