package com.deepclone.lw.beans.bt.es.data;


import java.io.Serializable;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Resource delta information
 * 
 * <p>
 * This class represents records which contain a resource's delta - that is, the income and upkeep
 * for some type of resource.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "resource-delta" )
public class ResourceDeltaInformation
		implements Serializable
{

	/** Identifier of the resource */
	@XStreamAlias( "id" )
	@XStreamAsAttribute
	private String resource;

	/** Income for that type of resource */
	@XStreamAsAttribute
	private Double income;

	/** Upkeep for that type of resource */
	@XStreamAsAttribute
	private Double upkeep;


	/** @return the resource's identifier */
	public String getResource( )
	{
		return this.resource;
	}


	/**
	 * Set the resource's identifier
	 * 
	 * @param resource
	 *            the resource's identifier
	 * 
	 * @throws InvalidDumpContentsException
	 *             if the specified resource type is <code>null</code>
	 */
	public void setResource( String resource )
			throws InvalidDumpContentsException
	{
		if ( resource == null ) {
			throw new InvalidDumpContentsException( this.getClass( ) , "resource" );
		}
		this.resource = resource;
	}


	/**
	 * @return the income for that type of resource, or <code>null</code> if no income information
	 *         is available
	 */
	public Double getIncome( )
	{
		return this.income;
	}


	/**
	 * Set the income for that type of resource
	 * 
	 * @param income
	 *            the income for that type of resource
	 */
	public void setIncome( double income )
	{
		this.income = income;
	}


	/**
	 * @return the upkeep for that type of resource, or <code>null</code> if no upkeep information
	 *         is available
	 */
	public Double getUpkeep( )
	{
		return this.upkeep;
	}


	/**
	 * Set the upkeep for that type of resource
	 * 
	 * @param upkeep
	 *            the upkeep for that type of resource
	 */
	public void setUpkeep( double upkeep )
	{
		this.upkeep = upkeep;
	}

}
