package com.deepclone.lw.beans.eventlog;


import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;



/**
 * Log clean-up task.
 * 
 * <p>
 * This class implements the log clean-up task; it is simultaneously a constants user (as it needs
 * to keep informed about <em>log.maxAge.*</em>), a runnable (as it needs to be registered as a
 * ticker task) and a transaction callback (to perform the actual clean-up).
 * 
 * @author tseeker
 */
class LogCleanerTask
		implements Runnable
{

	/** Transaction manager interface */
	private final TransactionTemplate tTemplate;
	private JdbcTemplate dTemplate;



	/**
	 * @param sessionFactory
	 *            the Hibernate session factory
	 * @param transactionManager
	 *            the transaction manager
	 */
	LogCleanerTask( DataSource dataSource , PlatformTransactionManager transactionManager )
	{
		this.tTemplate = new TransactionTemplate( transactionManager );
		this.dTemplate = new JdbcTemplate( dataSource );
	}


	/**
	 * Executes the clean-up transaction if the constants have been set.
	 */
	@Override
	synchronized public void run( )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				dTemplate.execute( "SELECT sys.clean_logs( )" );
			}
		} );
	}
}
