package com.deepclone.lw.beans.i18n;


import javax.sql.DataSource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.interfaces.i18n.I18NAdministration;
import com.deepclone.lw.interfaces.i18n.I18NManager;



/**
 * The I18N manager bean creates the {@link I18NData} instance on initialisation, which it then
 * shares with translator beans and administrative session instances.
 * 
 * @author tseeker
 */
public class I18NManagerBean
		implements I18NManager , InitializingBean
{

	/** Transaction manager interface */
	private TransactionTemplate tTemplate;

	/** Database interface */
	private DataSource dataSource;

	/** Data store shared amongst the translator beans and administrative session instances */
	private I18NData data;


	/**
	 * Sets the transaction manager interface (DI)
	 * 
	 * @param manager
	 *            the transaction manager
	 */
	@Autowired( required = true )
	public void setTransactionManager( DataSourceTransactionManager manager )
	{
		this.tTemplate = new TransactionTemplate( manager );
	}


	/**
	 * Sets the JBDC interface (DI)
	 * 
	 * @param dSource
	 *            the data source
	 */
	@Autowired( required = true )
	public void setDataSource( DataSource dSource )
	{
		this.dataSource = dSource;
	}


	/** Creates the shared {@link I18NData} instance on initialisation */
	@Override
	public void afterPropertiesSet( )
	{
		this.data = new I18NData( this.dataSource , this.tTemplate );
	}


	/** @return the shared {@link I18NData} instance */
	I18NData getData( )
	{
		return this.data;
	}


	/* Documentation in I18NManager interface */
	@Override
	public I18NAdministration getAdminSession( int administrator )
	{
		return new I18NAdministrationImpl( this.data , administrator );
	}


	/* Documentation in I18NManager interface */
	@Override
	public void reload( )
	{
		this.data.writeLock( ).lock( );
		try {
			this.data.loadAll( );
		} finally {
			this.data.writeLock( ).unlock( );
		}
	}

}
