package com.deepclone.lw.beans.i18n;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.HashSet;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;

import com.deepclone.lw.sqld.i18n.Translation;




/**
 * The I18N data loader transaction reads all string definitions and language definitions from the
 * database, creating {@link LanguageStore} instances for each language.
 * 
 * @author tseeker
 */
class LoaderTransaction
		extends TransactionCallbackWithoutResult
{

	/** Database interface */
	private final JdbcTemplate dTemplate;

	/** String definition map being initialised */
	private final HashSet< String > strings;

	/** Language definition map being initialised */
	private final HashMap< String , LanguageStore > languages;


	/** Copies the required references */
	LoaderTransaction( JdbcTemplate dTemplate , HashSet< String > strings ,
			HashMap< String , LanguageStore > languages )
	{
		this.dTemplate = dTemplate;
		this.strings = strings;
		this.languages = languages;
	}


	/**
	 * The transaction loads all string definitions, storing them in the string definition map, then
	 * loads all languages, creating a {@link LanguageStore} for each instance.
	 */
	@Override
	protected void doInTransactionWithoutResult( TransactionStatus status )
	{
		String sql;
		RowMapper< Translation > mapper = new RowMapper< Translation >( ) {

			@Override
			public Translation mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				Translation t = new Translation( );
				t.setLanguageId( rs.getString( "language_id" ) );
				t.setLanguageName( rs.getString( "language_name" ) );
				t.setStringId( rs.getString( "string_id" ) );
				t.setTranslation( rs.getString( "translation" ) );
				return t;
			}

		};

		sql = "SELECT language_id , language_name , string_id , translation FROM defs.translations_view";
		for ( Translation trans : this.dTemplate.query( sql , mapper ) ) {
			this.strings.add( trans.getStringId( ) );

			LanguageStore ls = this.languages.get( trans.getLanguageId( ) );
			if ( ls == null ) {
				ls = new LanguageStore( trans.getLanguageId( ) , trans.getLanguageName( ) );
				this.languages.put( trans.getLanguageId( ) , ls );
			}
			ls.addTranslation( trans.getStringId( ) , trans.getTranslation( ) );
		}
	}

}
