package com.deepclone.lw.beans.mailer;


import com.deepclone.lw.interfaces.mailer.NotSentException;



/**
 * Mail queue handler interface
 * 
 * This interface is normally implemented by the mail queue task. However, it is required in order
 * to test the MailData implementation separately.
 * 
 * @author tseeker
 */
interface MailQueueHandler
{

	/**
	 * Stops the queue-handling task.
	 * 
	 * This method inserts a terminator into the queue, then waits for the queue to be
	 * de-initialised (which indicates that the main loop ended).
	 */
	public void stop( );


	/**
	 * Adds a message to the queue.
	 * 
	 * @param mailTo
	 *            the address of the email's recipient.
	 * @param message
	 *            the contents of the email
	 */
	public void queueMessage( String mailTo , String message );


	/**
	 * Sends a message.
	 * 
	 * @param mailTo
	 *            address of the email's recipient
	 * @param message
	 *            contents of the message
	 * @throws NotSentException
	 *             if the mail could not be sent for some reason
	 */
	public void sendMessage( String mailTo , String message )
			throws NotSentException;

}