package com.deepclone.lw.beans.naming;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.naming.GetNamesResponse;
import com.deepclone.lw.cmd.admin.naming.NameType;
import com.deepclone.lw.cmd.admin.naming.NamesSummaryResponse;
import com.deepclone.lw.cmd.admin.naming.NamesSummaryResponse.Entry;
import com.deepclone.lw.interfaces.naming.NamesManager;
import com.deepclone.lw.interfaces.naming.NamingDAO;



@Transactional
public class NamesManagerBean
		implements NamesManager
{

	private NamingDAO namingDao;


	@Autowired( required = true )
	public void setNamingDao( NamingDAO namingDao )
	{
		this.namingDao = namingDao;
	}


	@Override
	public NamesSummaryResponse getSummary( Administrator admin )
	{
		List< Entry > entries = new LinkedList< Entry >( );
		for ( Map.Entry< NameType , Long > entry : this.namingDao.countNames( ).entrySet( ) ) {
			entries.add( new Entry( entry.getKey( ) , entry.getValue( ) ) );
		}
		return new NamesSummaryResponse( admin , entries );
	}


	@Override
	public GetNamesResponse getNames( Administrator admin , NameType type )
	{
		return new GetNamesResponse( admin , type , this.namingDao.getNames( type ) );
	}


	@Override
	public void validateMapNames( Administrator admin , int[] ids )
	{
		for ( int id : ids ) {
			this.namingDao.validateMapName( admin.getId( ) , id );
		}
	}


	@Override
	public void allowMapNameChanges( Administrator admin , int[] ids )
	{
		for ( int id : ids ) {
			this.namingDao.allowMapNameChange( admin.getId( ) , id );
		}
	}


	@Override
	public void rejectMapNames( Administrator admin , int[] ids , boolean ban )
	{
		for ( int id : ids ) {
			this.namingDao.rejectMapName( admin.getId( ) , id , ban );
		}
	}


	@Override
	public void rejectEmpireNames( Administrator admin , int[] ids , boolean ban )
	{
		for ( int id : ids ) {
			this.namingDao.rejectEmpireName( admin.getId( ) , id , ban );
		}
	}


	@Override
	public void rejectAllianceNames( Administrator admin , int[] ids )
	{
		for ( int id : ids ) {
			this.namingDao.rejectAllianceName( admin.getId( ) , id );
		}
	}

}
