package com.deepclone.lw.beans.game.resources;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.player.gdata.AbstractResourceRecord;



/**
 * Base class for resource information row mappers
 * 
 * <p>
 * This class can be used to map resource information rows (for either planets or empires). It
 * provides a method which maps the resource information's descriptive fields, and implements
 * Spring's {@link RowMapper} for the correct type.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 * @param <RTYPE>
 *            a resource information record based on {@link AbstractResourceRecord}.
 */
abstract class AbstractResourceMapper< RTYPE extends AbstractResourceRecord >
		implements RowMapper< RTYPE >
{

	/**
	 * Extract a resource information row's descriptive fields
	 * 
	 * <p>
	 * This method will extract the descriptive fields from a resource information row and store
	 * them in a resource information instance. It extracts:
	 * <ul>
	 * <li>the resource's text identifier,
	 * <li>the resource's internationalised name,
	 * <li>the resource's internationalised description,
	 * <li>the resource's internationalised category name, if there is one.
	 * </ul>
	 * 
	 * @param resource
	 *            the resource information record
	 * @param rs
	 *            the result set with the correct row selected
	 * 
	 * @throws SQLException
	 *             if a SQLException is encountered getting column values
	 */
	protected final void getResourceDescription( RTYPE resource , ResultSet rs )
			throws SQLException
	{
		resource.setIdentifier( rs.getString( "resource_identifier" ) );
		resource.setTitle( rs.getString( "resource_name" ) );
		resource.setDescription( rs.getString( "resource_description" ) );
		resource.setCategory( rs.getString( "resource_category" ) );

	}

}
