package com.deepclone.lw.beans.game.resources;


import java.sql.ResultSet;
import java.sql.SQLException;

import com.deepclone.lw.cmd.player.gdata.empire.EmpireResourceRecord;



/**
 * Row mapper for empire resources
 * 
 * <p>
 * This class is responsible for converting empire resource information rows into instances of the
 * corresponding class, {@link EmpireResourceRecord}.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class EmpireResourceMapper
		extends AbstractResourceMapper< EmpireResourceRecord >
{

	/**
	 * Map a row from <code>emp.resources_view</code>
	 * 
	 * <p>
	 * This method extracts the resource's description and the fields specific to empire resource
	 * information from the row, returning the information as an {@link EmpireResourceRecord}
	 * instance.
	 */
	@Override
	public EmpireResourceRecord mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		EmpireResourceRecord resource = new EmpireResourceRecord( );
		this.getResourceDescription( resource , rs );
		this.getEmpireFields( resource , rs );
		return resource;
	}


	/**
	 * Read empire-specific resource information from a row
	 * 
	 * <p>
	 * This method extracts the stockpiled quantity as well as income and upkeep from the row. If a
	 * mining setting is set, it extracts it as well.
	 * 
	 * @param resource
	 *            the empire resource record
	 * @param rs
	 *            the result set with the correct row selected
	 * 
	 * @throws SQLException
	 *             if a SQLException is encountered getting column values
	 */
	private void getEmpireFields( EmpireResourceRecord resource , ResultSet rs )
			throws SQLException
	{
		resource.setStockpiled( rs.getLong( "empres_possessed" ) );
		resource.setPlanetUpkeep( rs.getLong( "planets_upkeep" ) );
		resource.setIncome( rs.getLong( "planets_income" ) );
		resource.setFleetUpkeep( rs.getLong( "fleets_upkeep" ) );

		int miningPriority = rs.getInt( "empmset_weight" );
		if ( !rs.wasNull( ) ) {
			resource.setMiningPriority( miningPriority );
		}
	}

}
