package com.deepclone.lw.beans.empire;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.alliances.AllianceStatusResponse;
import com.deepclone.lw.cmd.player.alliances.CreateAllianceResponse;
import com.deepclone.lw.cmd.player.alliances.JoinAllianceResponse;
import com.deepclone.lw.cmd.player.alliances.ViewAllianceResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceCreationStatus;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceData;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceLeaderData;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceMemberData;
import com.deepclone.lw.cmd.player.gdata.alliance.PublicAllianceInformation;
import com.deepclone.lw.interfaces.game.AllianceDAO;
import com.deepclone.lw.interfaces.game.AllianceManagement;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.sqld.game.AllianceMembership;



@Transactional
public class AllianceManagementBean
		implements AllianceManagement
{

	private EmpireManagement empireManagement;
	private AllianceDAO allianceDao;


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement manager )
	{
		this.empireManagement = manager;
	}


	@Autowired( required = true )
	public void setAllianceDao( AllianceDAO allianceDao )
	{
		this.allianceDao = allianceDao;
	}


	private AllianceData getAllianceData( int empireId )
	{
		PublicAllianceInformation pub = null;
		AllianceMemberData member = null;
		AllianceLeaderData leader = null;

		AllianceMembership m = this.allianceDao.getAlliance( empireId );
		if ( m != null ) {
			int allianceId = m.getAllianceId( );
			pub = this.allianceDao.getPublicInformation( allianceId );
			if ( !m.isPending( ) ) {
				member = this.allianceDao.getMemberData( allianceId );
				if ( empireId == pub.getLeaderId( ) ) {
					leader = this.allianceDao.getLeaderData( allianceId );
				}
			}
		}

		return new AllianceData( pub , member , leader );
	}


	@Override
	public AllianceStatusResponse getView( int empireId )
	{
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		AllianceData alliance = this.getAllianceData( empireId );
		return new AllianceStatusResponse( page , alliance );
	}


	@Override
	public AllianceStatusResponse getInformation( int empireId , String tag )
	{
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		AllianceData alliance = this.getAllianceData( empireId );

		if ( "".equals( tag ) ) {
			return new AllianceStatusResponse( page , alliance );
		}

		Integer allianceId = this.allianceDao.findAlliance( tag );
		PublicAllianceInformation pub;
		if ( allianceId == null ) {
			pub = null;
		} else {
			pub = this.allianceDao.getPublicInformation( allianceId );
		}

		return new ViewAllianceResponse( page , alliance , tag , pub );
	}


	@Override
	public AllianceStatusResponse create( int empireId , String tag , String name )
	{
		Integer aId = this.allianceDao.createAlliance( empireId , tag , name );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		AllianceData alliance = this.getAllianceData( empireId );

		if ( aId == null ) {
			AllianceCreationStatus acs = new AllianceCreationStatus( tag , ObjectNameError.UNAVAILABLE , name , null );
			return new CreateAllianceResponse( page , alliance , acs );
		}

		return new AllianceStatusResponse( page , alliance );
	}


	@Override
	public AllianceStatusResponse requestJoin( int empireId , String tag )
	{
		Integer aId = this.allianceDao.findAlliance( tag );
		boolean joined;
		if ( aId == null ) {
			joined = false;
		} else {
			joined = this.allianceDao.requestJoin( empireId , aId );
		}

		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		AllianceData alliance = this.getAllianceData( empireId );

		return joined
				? new AllianceStatusResponse( page , alliance )
				: new JoinAllianceResponse( page , alliance , tag );
	}


	@Override
	public AllianceStatusResponse cancelJoin( int empireId )
	{
		this.allianceDao.cancelJoin( empireId );
		return this.getView( empireId );
	}


	@Override
	public AllianceStatusResponse leave( int empireId )
	{
		this.allianceDao.leave( empireId );
		return this.getView( empireId );
	}


	@Override
	public AllianceStatusResponse transferLeadership( int empireId , int toMember )
	{
		this.allianceDao.transferLeadership( empireId , toMember );
		return this.getView( empireId );
	}


	@Override
	public AllianceStatusResponse manageRequests( int empireId , boolean accept , int[] members )
	{
		this.allianceDao.manageRequests( empireId , accept , members );
		return this.getView( empireId );
	}


	@Override
	public AllianceStatusResponse kick( int empireId , int[] members )
	{
		this.allianceDao.kick( empireId , members );
		return this.getView( empireId );
	}

}
