package com.deepclone.lw.beans.fleets;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.PlanetRelationType;
import com.deepclone.lw.cmd.player.gdata.ShortBattleView;
import com.deepclone.lw.cmd.player.gdata.fleets.*;
import com.deepclone.lw.interfaces.game.FleetsDAO;
import com.deepclone.lw.sqld.game.RawFleetOwner;
import com.deepclone.lw.sqld.game.RawFleetShip;
import com.deepclone.lw.sqld.game.RawStaticFleet;
import com.deepclone.lw.utils.StoredProc;



public class FleetsDAOBean
		implements FleetsDAO
{
	private static String sqlLocations = "SELECT * FROM fleets.locations_view WHERE empire = ?";
	private static String sqlOwners = "SELECT * FROM fleets.owners_view WHERE empire = ? ORDER BY relation";
	private static String sqlStaticFleets = "SELECT * FROM fleets.static_fleets WHERE empire = ? ORDER BY power DESC, name NULLS LAST, id DESC";
	private static String sqlMovingFleets = "SELECT * FROM fleets.moving_fleets WHERE empire = ? ORDER BY time_left + penalty, name NULLS LAST, id DESC";
	private static String sqlShips = "SELECT * FROM fleets.ships_view WHERE empire = ?";
	private static String sqlShortStatic = "SELECT * FROM fleets.short_static_fleets WHERE empire = ? AND id IN ( SELECT unnest AS id FROM unnest( ?::BIGINT[] ) )";
	private static String sqlShortMoving = "SELECT * FROM fleets.moving_fleets WHERE empire = ? AND id IN ( SELECT unnest AS id FROM unnest( ?::BIGINT[] ) )";
	private static String sqlSingleShort = "SELECT * FROM fleets.short_static_fleets WHERE empire = ? AND id = ?";
	private static String sqlSingleMoving = "SELECT * FROM fleets.moving_fleets WHERE empire = ? AND id = ?";
	private static String sqlFleetShips = "SELECT * FROM fleets.ships_view WHERE empire = ? AND id = ?";

	private JdbcTemplate dTemplate;

	private RowMapper< FleetLocation > mLocations;
	private RowMapper< RawFleetOwner > mOwners;
	private RowMapper< RawStaticFleet > mStaticFleets;
	private RowMapper< MovingFleet > mMovingFleets;
	private RowMapper< RawFleetShip > mShips;
	private RowMapper< ShortFleetView > mShortStatic;
	private RowMapper< ShortFleetView > mShortMoving;

	private StoredProc fMoveFleets;
	private StoredProc fSetMode;
	private StoredProc fRenameFleets;
	private StoredProc fMergeFleets;
	private StoredProc fDisbandFleets;
	private StoredProc fInitSplit;
	private StoredProc fSetSplitShips;
	private StoredProc fExecuteSplit;


	public FleetsDAOBean( )
	{
		this.mLocations = new RowMapper< FleetLocation >( ) {
			@Override
			public FleetLocation mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				FleetLocation location = new FleetLocation( rs.getInt( "location" ) , rs.getString( "name" ) );
				location.setOwn( rs.getBoolean( "is_own" ) );
				location.setAttacking( rs.getBoolean( "attacking" ) );
				location.setX( rs.getInt( "x" ) );
				location.setY( rs.getInt( "y" ) );
				location.setOrbit( rs.getInt( "orbit" ) );
				location.setTag( rs.getString( "tag" ) );
				location.setPicture( rs.getInt( "picture" ) );
				location.setPopulation( rs.getLong( "population" ) );
				location.setDefence( rs.getLong( "defence" ) );
				if ( rs.getObject( "battle" ) == null ) {
					location.setBattle( null );
					location.setOnVacation( rs.getBoolean( "on_vacation" ) );
				} else {
					ShortBattleView battle = new ShortBattleView( );
					battle.setId( rs.getLong( "battle" ) );
					battle.setFriendly( rs.getLong( "friendly_power" ) );
					battle.setHostile( rs.getLong( "hostile_power" ) );
					location.setBattle( battle );
					location.setOnVacation( false );
				}
				return location;
			}
		};

		this.mOwners = new RowMapper< RawFleetOwner >( ) {
			@Override
			public RawFleetOwner mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				RawFleetOwner info = new RawFleetOwner( );
				info.setLocation( rs.getInt( "location" ) );
				info.setId( rs.getInt( "id" ) );
				info.setName( rs.getString( "name" ) );
				info.setRelation( rs.getString( "relation" ) );
				return info;
			}
		};

		this.mStaticFleets = new RowMapper< RawStaticFleet >( ) {
			@Override
			public RawStaticFleet mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				RawStaticFleet info = new RawStaticFleet( );
				info.setLocation( rs.getInt( "location" ) );
				info.setOwner( rs.getInt( "owner" ) );
				info.setId( rs.getLong( "id" ) );
				info.setName( rs.getString( "name" ) );
				info.setStatus( rs.getString( "status" ) );
				info.setPenalty( rs.getInt( "penalty" ) );
				info.setPower( rs.getLong( "power" ) );
				info.setFlightTime( rs.getInt( "flight_time" ) );
				return info;
			}
		};

		this.mMovingFleets = new RowMapper< MovingFleet >( ) {
			@Override
			public MovingFleet mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				MovingFleet fleet = new MovingFleet( rs.getLong( "id" ) , rs.getString( "name" ) );
				fleet.setAttacking( rs.getBoolean( "attacking" ) );
				fleet.setStatus( FleetStatus.valueOf( rs.getString( "status" ) ) );
				fleet.setPenalty( rs.getInt( "penalty" ) );
				fleet.setFlightTime( rs.getInt( "flight_time" ) );
				fleet.setPower( rs.getLong( "power" ) );
				fleet.setTimeLeft( rs.getInt( "time_left" ) );
				fleet.setSource( new NameIdPair( rs.getInt( "from_id" ) , rs.getString( "from_name" ) ) );
				fleet.setDestination( new NameIdPair( rs.getInt( "to_id" ) , rs.getString( "to_name" ) ) );
				fleet.setCurrentX( rs.getDouble( "cx" ) );
				fleet.setCurrentY( rs.getDouble( "cy" ) );
				if ( rs.getString( "nearest_name" ) != null ) {
					fleet.setNearest( new NameIdPair( rs.getInt( "nearest_id" ) , rs.getString( "nearest_name" ) ) );
				}
				return fleet;
			}
		};

		this.mShips = new RowMapper< RawFleetShip >( ) {
			@Override
			public RawFleetShip mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				RawFleetShip ship = new RawFleetShip( );
				ship.setFleet( rs.getLong( "id" ) );
				ship.setShipType( rs.getInt( "ship_id" ) );
				ship.setAmount( rs.getInt( "amount" ) );
				ship.setPower( rs.getInt( "power" ) );
				ship.setName( rs.getString( "name" ) );
				return ship;
			}
		};

		this.mShortStatic = new RowMapper< ShortFleetView >( ) {
			@Override
			public ShortFleetView mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ShortFleetView sfv = new ShortFleetView( rs.getLong( "id" ) , rs.getString( "name" ) );
				sfv.setFlightTime( rs.getInt( "flight_time" ) );
				sfv.setStatus( FleetStatus.valueOf( rs.getString( "status" ) ) );
				sfv.setPenalty( rs.getInt( "penalty" ) );
				sfv.setPower( rs.getLong( "power" ) );
				sfv.setAttack( rs.getBoolean( "attacking" ) );
				sfv.setAtPlanet( true );
				sfv.setX( rs.getDouble( "x" ) );
				sfv.setY( rs.getDouble( "y" ) );
				sfv.setNearest( new NameIdPair( rs.getInt( "location_id" ) , rs.getString( "location_name" ) ) );
				return sfv;
			}
		};

		this.mShortMoving = new RowMapper< ShortFleetView >( ) {
			@Override
			public ShortFleetView mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ShortFleetView sfv = new ShortFleetView( rs.getLong( "id" ) , rs.getString( "name" ) );
				sfv.setFlightTime( rs.getInt( "flight_time" ) );
				sfv.setStatus( FleetStatus.valueOf( rs.getString( "status" ) ) );
				sfv.setPenalty( rs.getInt( "penalty" ) );
				sfv.setPower( rs.getLong( "power" ) );
				sfv.setAttack( rs.getBoolean( "attacking" ) );
				sfv.setAtPlanet( false );
				sfv.setX( rs.getDouble( "cx" ) );
				sfv.setY( rs.getDouble( "cy" ) );
				if ( rs.getString( "nearest_name" ) != null ) {
					sfv.setNearest( new NameIdPair( rs.getInt( "nearest_id" ) , rs.getString( "nearest_name" ) ) );
				}
				return sfv;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fMoveFleets = new StoredProc( dataSource , "fleets" , "move_fleets" );
		this.fMoveFleets.addParameter( "empire_id" , Types.INTEGER );
		this.fMoveFleets.addParameter( "fleet_ids" , "BIGINT[]" );
		this.fMoveFleets.addParameter( "destination" , Types.VARCHAR );
		this.fMoveFleets.addOutput( "success" , Types.BOOLEAN );

		this.fSetMode = new StoredProc( dataSource , "fleets" , "set_mode" );
		this.fSetMode.addParameter( "empire_id" , Types.INTEGER );
		this.fSetMode.addParameter( "fleet_ids" , "BIGINT[]" );
		this.fSetMode.addParameter( "attack" , Types.BOOLEAN );

		this.fRenameFleets = new StoredProc( dataSource , "fleets" , "rename_fleets" );
		this.fRenameFleets.addParameter( "empire_id" , Types.INTEGER );
		this.fRenameFleets.addParameter( "fleet_ids" , "BIGINT[]" );
		this.fRenameFleets.addParameter( "new_name" , Types.VARCHAR );

		this.fMergeFleets = new StoredProc( dataSource , "fleets" , "merge" );
		this.fMergeFleets.addParameter( "empire_id" , Types.INTEGER );
		this.fMergeFleets.addParameter( "fleet_ids" , "BIGINT[]" );

		this.fDisbandFleets = new StoredProc( dataSource , "fleets" , "disband" );
		this.fDisbandFleets.addParameter( "empire_id" , Types.INTEGER );
		this.fDisbandFleets.addParameter( "fleet_ids" , "BIGINT[]" );

		this.fInitSplit = new StoredProc( dataSource , "fleets" , "init_split" );
		this.fInitSplit.addParameter( "fleet_id" , Types.BIGINT );
		this.fInitSplit.addParameter( "n_fleets" , Types.INTEGER );
		this.fInitSplit.addParameter( "name" , Types.VARCHAR );
		this.fInitSplit.addOutput( "success" , Types.BOOLEAN );

		this.fSetSplitShips = new StoredProc( dataSource , "fleets" , "set_split_ships" );
		this.fSetSplitShips.addParameter( "ships_type" , Types.INTEGER );
		this.fSetSplitShips.addParameter( "split_amount" , Types.INTEGER );

		this.fExecuteSplit = new StoredProc( dataSource , "fleets" , "execute_split" );
		this.fExecuteSplit.addParameter( "simulate" , Types.BOOLEAN );
		this.fExecuteSplit.addOutput( "success" , Types.BOOLEAN );
	}


	private String getSQLArray( long[] fleets )
	{
		StringBuilder fArray = new StringBuilder( ).append( "{" );
		for ( int i = 0 ; i < fleets.length ; i++ ) {
			fArray.append( fleets[ i ] );
			if ( i != fleets.length - 1 ) {
				fArray.append( "," );
			}
		}
		fArray.append( "}" );
		return fArray.toString( );
	}


	@Override
	public FleetsView getFleets( int empireId )
	{
		FleetsView result = new FleetsView( );

		// Get visible locations
		Map< Integer , FleetLocation > locations = new HashMap< Integer , FleetLocation >( );
		Map< Integer , Map< Integer , FleetOwner >> owners = new HashMap< Integer , Map< Integer , FleetOwner > >( );
		for ( FleetLocation l : this.dTemplate.query( FleetsDAOBean.sqlLocations , this.mLocations , empireId ) ) {
			locations.put( (int) l.getId( ) , l );
			owners.put( (int) l.getId( ) , new HashMap< Integer , FleetOwner >( ) );
			result.addLocation( l );
		}

		// Get visible fleet owners
		for ( RawFleetOwner rfo : this.dTemplate.query( FleetsDAOBean.sqlOwners , this.mOwners , empireId ) ) {
			FleetOwner owner;
			int ownerId = rfo.getId( );
			owner = new FleetOwner( ownerId , rfo.getName( ) , PlanetRelationType.valueOf( rfo.getRelation( ) ) );

			int locationId = rfo.getLocation( );
			FleetLocation location = locations.get( locationId );
			Map< Integer , FleetOwner > lOwners = owners.get( locationId );
			location.addFleetOwner( owner );
			lOwners.put( ownerId , owner );
		}

		// Get static fleets
		Map< Long , StaticFleet > fleets = new HashMap< Long , StaticFleet >( );
		for ( RawStaticFleet rsf : this.dTemplate.query( FleetsDAOBean.sqlStaticFleets , this.mStaticFleets , empireId ) ) {
			int ownerId = rsf.getOwner( );
			int locationId = rsf.getLocation( );
			long fleetId = rsf.getId( );

			StaticFleet fleet = new StaticFleet( fleetId , rsf.getName( ) );
			fleet.setFlightTime( rsf.getFlightTime( ) );
			fleet.setStatus( FleetStatus.valueOf( rsf.getStatus( ) ) );
			fleet.setPenalty( rsf.getPenalty( ) );
			fleet.setPower( rsf.getPower( ) );

			fleets.put( fleetId , fleet );
			owners.get( locationId ).get( ownerId ).addFleet( fleet );
		}

		// Get moving fleets
		for ( MovingFleet f : this.dTemplate.query( FleetsDAOBean.sqlMovingFleets , this.mMovingFleets , empireId ) ) {
			result.addMovingFleet( f );
			fleets.put( f.getId( ) , f );
		}

		// Fetch fleet ships
		for ( RawFleetShip rfs : this.dTemplate.query( FleetsDAOBean.sqlShips , this.mShips , empireId ) ) {
			FleetShips ships = new FleetShips( );
			ships.setAmount( rfs.getAmount( ) );
			ships.setPower( rfs.getPower( ) );
			ships.setName( rfs.getName( ) );
			fleets.get( rfs.getFleet( ) ).addShips( ships );
		}

		return result;
	}


	@Override
	public List< ShortFleetView > getShortFleets( int empireId , long[] fleets , boolean needsAvailable )
	{
		String fArray = this.getSQLArray( fleets );

		List< ShortFleetView > result = new LinkedList< ShortFleetView >( );
		for ( ShortFleetView sfv : this.dTemplate.query( FleetsDAOBean.sqlShortStatic , this.mShortStatic , empireId ,
				fArray ) ) {
			if ( needsAvailable && sfv.getStatus( ) != FleetStatus.AVAILABLE ) {
				continue;
			}
			result.add( sfv );
		}
		for ( ShortFleetView sfv : this.dTemplate.query( FleetsDAOBean.sqlShortMoving , this.mShortMoving , empireId ,
				fArray ) ) {
			if ( needsAvailable && sfv.getStatus( ) != FleetStatus.AVAILABLE ) {
				continue;
			}
			result.add( sfv );
		}

		return result;
	}


	@Override
	public boolean move( int empireId , long[] fleets , String destination )
	{
		return (Boolean) this.fMoveFleets.execute( empireId , this.getSQLArray( fleets ) , destination )
				.get( "success" );
	}


	@Override
	public void setMode( int empireId , long[] fleets , boolean attack )
	{
		this.fSetMode.execute( empireId , this.getSQLArray( fleets ) , attack );
	}


	@Override
	public void rename( int empireId , long[] fleets , String name )
	{
		this.fRenameFleets.execute( empireId , this.getSQLArray( fleets ) , name );
	}


	@Override
	public void merge( int empireId , long[] fleets )
	{
		this.fMergeFleets.execute( empireId , this.getSQLArray( fleets ) );
	}


	@Override
	public void disband( int empireId , long[] fleets )
	{
		this.fDisbandFleets.execute( empireId , this.getSQLArray( fleets ) );
	}


	@Override
	public ShortFleetView getShortFleet( int empireId , long fleetId )
	{
		try {
			return this.dTemplate.queryForObject( sqlSingleShort , mShortStatic , empireId , fleetId );
		} catch ( EmptyResultDataAccessException e ) {
			// EMPTY
		}

		try {
			return this.dTemplate.queryForObject( sqlSingleMoving , mShortMoving , empireId , fleetId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public List< SplitShips > getShips( int empireId , long fleetId )
	{
		List< SplitShips > result = new LinkedList< SplitShips >( );
		for ( RawFleetShip rfs : this.dTemplate.query( sqlFleetShips , mShips , empireId , fleetId ) ) {
			SplitShips ships = new SplitShips( );
			ships.setId( rfs.getShipType( ) );
			ships.setType( rfs.getShipType( ) );
			ships.setAmount( rfs.getAmount( ) );
			ships.setPower( rfs.getPower( ) );
			ships.setName( rfs.getName( ) );
			result.add( ships );
		}
		return result;
	}


	@Override
	public boolean initSplit( long fleetId , int nFleets , String name )
	{
		return (Boolean) this.fInitSplit.execute( fleetId , nFleets , name ).get( "success" );
	}


	@Override
	public void setSplitShips( int shipType , int amount )
	{
		this.fSetSplitShips.execute( shipType , amount );
	}


	@Override
	public boolean executeSplit( boolean simulate )
	{
		return (Boolean) this.fExecuteSplit.execute( simulate ).get( "success" );
	}

}
