package com.deepclone.lw.beans.map;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.player.gdata.MapSize;
import com.deepclone.lw.cmd.player.gdata.PlanetRelationType;
import com.deepclone.lw.cmd.player.gdata.map.MapPlanetData;
import com.deepclone.lw.cmd.player.gdata.map.MapSystemData;
import com.deepclone.lw.interfaces.acm.UsersDAO;
import com.deepclone.lw.interfaces.game.MapViewParameters;
import com.deepclone.lw.interfaces.game.MapViewer;
import com.deepclone.lw.interfaces.game.UniverseDAO;
import com.deepclone.lw.interfaces.prefs.AccountPreferences;
import com.deepclone.lw.interfaces.prefs.PreferencesDAO;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.sqld.game.MapData;
import com.deepclone.lw.utils.EmailAddress;



@Transactional
public class MapViewerBean
		implements MapViewer
{

	private UsersDAO usersDao;
	private PreferencesDAO preferencesDao;
	private UniverseDAO universeDao;


	@Autowired( required = true )
	public void setUsersDao( UsersDAO usersDao )
	{
		this.usersDao = usersDao;
	}


	@Autowired( required = true )
	public void setPreferencesDao( PreferencesDAO preferencesDao )
	{
		this.preferencesDao = preferencesDao;
	}


	@Autowired( required = true )
	public void setUniverseDao( UniverseDAO universeDao )
	{
		this.universeDao = universeDao;
	}


	@Override
	public MapViewParameters getDefaults( EmailAddress address )
	{
		Account accnt = this.usersDao.getAccount( address );
		AccountPreferences prefs = this.preferencesDao.getPreferences( accnt );
		int x = (Integer) prefs.getPreference( "mapX" , Integer.class );
		int y = (Integer) prefs.getPreference( "mapY" , Integer.class );
		MapSize size = (MapSize) prefs.getPreference( "mapSize" , MapSize.class );
		return new MapViewParameters( x , y , size );
	}


	@Override
	public MapSystemData[][] getMapView( int empireId , MapViewParameters parameters )
	{
		int size = 3 + ( 2 * parameters.size.ordinal( ) );
		int minX = parameters.x - ( size >> 1 );
		int maxX = parameters.x + ( size >> 1 );
		int minY = parameters.y - ( size >> 1 );
		int maxY = parameters.y + ( size >> 1 );

		MapSystemData[][] results = new MapSystemData[ size ][ size ];
		for ( MapData entry : this.universeDao.getMap( empireId , minX , minY , maxX , maxY ) ) {
			int x = entry.getX( ) - minX;
			int y = maxY - entry.getY( );
			if ( entry.getOrbit( ) == 1 ) {
				results[ y ][ x ] = new MapSystemData( );
			}

			PlanetRelationType relation;
			String rValue = entry.getDisplay( );
			relation = ( rValue == null ) ? null : PlanetRelationType.valueOf( rValue );

			MapPlanetData planet;
			planet = new MapPlanetData( entry.getId( ) , entry.getPicture( ) , entry.getName( ) , entry.getTag( ) ,
					relation );
			results[ y ][ x ].setPlanet( entry.getOrbit( ) , planet );
		}

		return results;
	}

}
