package com.deepclone.lw.beans.map;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import com.deepclone.lw.cmd.player.gdata.planets.BuildableBuildingData;
import com.deepclone.lw.cmd.player.gdata.planets.BuildableShipData;
import com.deepclone.lw.cmd.player.gdata.planets.BuildingData;
import com.deepclone.lw.cmd.player.gdata.planets.QueueData;
import com.deepclone.lw.cmd.player.gdata.planets.QueueItemData;
import com.deepclone.lw.interfaces.game.PlanetDAO;
import com.deepclone.lw.sqld.game.PlanetData.AccessType;
import com.deepclone.lw.sqld.game.PlanetData.Basic;
import com.deepclone.lw.sqld.game.PlanetData.Orbital;
import com.deepclone.lw.sqld.game.PlanetData.Owner;
import com.deepclone.lw.utils.StoredProc;



public class PlanetDAOBean
		implements PlanetDAO
{

	private JdbcTemplate dTemplate;

	private RowMapper< Basic > mPlanetBasics;
	private RowMapper< Orbital > mPlanetOrbital;
	private RowMapper< Owner > mPlanetOwner;
	private RowMapper< BuildingData > mOwnerBuildings;
	private RowMapper< BuildingData > mOrbitalBuildings;
	private RowMapper< QueueItemData > mQueueItem;
	private RowMapper< BuildableBuildingData > mAvailableBuilding;
	private RowMapper< BuildableShipData > mAvailableShip;

	private StoredProc fFlushBuildQueue;
	private StoredProc fFlushMilitaryQueue;
	private StoredProc fAddToMilitaryQueue;
	private StoredProc fConstructBuildings;
	private StoredProc fDestroyBuildings;
	private StoredProc fAbandon;
	private StoredProc fCancelAbandon;


	public PlanetDAOBean( )
	{
		this.mPlanetBasics = new RowMapper< Basic >( ) {
			@Override
			public Basic mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				if ( rs.getString( "access" ) == null ) {
					return null;
				}

				Basic info = new Basic( );
				info.setAccess( AccessType.valueOf( rs.getString( "access" ) ) );
				info.setX( rs.getInt( "x" ) );
				info.setY( rs.getInt( "y" ) );
				info.setOrbit( rs.getInt( "orbit" ) );
				info.setPicture( rs.getInt( "picture" ) );
				info.setName( rs.getString( "name" ) );
				info.setTag( rs.getString( "tag" ) );
				return info;
			}
		};

		this.mPlanetOrbital = new RowMapper< Orbital >( ) {
			@Override
			public Orbital mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				Orbital info = new Orbital( );
				info.setPopulation( rs.getLong( "population" ) );
				info.setDefence( rs.getLong( "defence" ) );
				info.setOwnPower( rs.getLong( "own_power" ) );
				info.setFriendlyPower( rs.getLong( "friendly_power" ) );
				info.setHostilePower( rs.getLong( "hostile_power" ) );
				info.setBattle( (Long) rs.getObject( "battle_id" ) );
				return info;
			}
		};

		this.mPlanetOwner = new RowMapper< Owner >( ) {
			@Override
			public Owner mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				Owner info = new Owner( );
				info.setHappiness( rs.getInt( "happiness" ) );
				info.sethChange( rs.getInt( "h_change" ) );
				info.setIncome( rs.getLong( "income" ) );
				info.setUpkeep( rs.getLong( "upkeep" ) );
				info.setRenamePossible( rs.getBoolean( "can_rename" ) );
				info.setAbandonPossible( rs.getBoolean( "can_abandon" ) );
				info.setAbandonTime( rs.getInt( "abandon_time" ) );
				info.setOwnSettings( rs.getBoolean( "specific_mining_settings" ) );
				return info;
			}
		};

		this.mOrbitalBuildings = new RowMapper< BuildingData >( ) {
			@Override
			public BuildingData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BuildingData info = new BuildingData( );
				info.setAmount( rs.getInt( "amount" ) );
				info.setName( rs.getString( "name" ) );
				info.setDescription( rs.getString( "description" ) );
				return info;
			}
		};

		this.mOwnerBuildings = new RowMapper< BuildingData >( ) {
			@Override
			public BuildingData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BuildingData info = new BuildingData( );
				info.setId( rs.getInt( "id" ) );
				info.setAmount( rs.getInt( "amount" ) );
				info.setName( rs.getString( "name" ) );
				info.setDescription( rs.getString( "description" ) );
				info.setJobs( rs.getInt( "jobs" ) );
				info.setUpkeep( rs.getLong( "upkeep" ) );
				info.setProduces( rs.getString( "p_type" ) );
				info.setOutput( rs.getLong( "p_value" ) );
				return info;
			}
		};

		this.mQueueItem = new RowMapper< QueueItemData >( ) {
			@Override
			public QueueItemData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				QueueItemData item = new QueueItemData( );
				item.setName( rs.getString( "name" ) );
				item.setDescription( rs.getString( "description" ) );
				item.setAmount( rs.getInt( "amount" ) );
				item.setDestroy( rs.getBoolean( "destroy" ) );
				item.setInvested( rs.getLong( "investment" ) );
				item.setTimeLeft( (Long) rs.getObject( "time_left" ) );
				return item;
			}
		};

		this.mAvailableBuilding = new RowMapper< BuildableBuildingData >( ) {
			@Override
			public BuildableBuildingData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BuildableBuildingData info = new BuildableBuildingData( );
				info.setId( rs.getInt( "id" ) );
				info.setName( rs.getString( "name" ) );
				info.setDescription( rs.getString( "description" ) );
				info.setCost( rs.getInt( "cost" ) );
				info.setTime( (Long) rs.getObject( "time_to_build" ) );
				info.setUpkeep( rs.getInt( "upkeep" ) );
				info.setWorkers( rs.getInt( "workers" ) );
				info.setProdType( rs.getString( "p_type" ) );
				info.setOutput( rs.getInt( "p_value" ) );
				return info;
			}
		};

		this.mAvailableShip = new RowMapper< BuildableShipData >( ) {
			@Override
			public BuildableShipData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BuildableShipData info = new BuildableShipData( );
				info.setId( rs.getInt( "id" ) );
				info.setName( rs.getString( "name" ) );
				info.setDescription( rs.getString( "description" ) );
				info.setCost( rs.getInt( "cost" ) );
				info.setTime( (Long) rs.getObject( "time_to_build" ) );
				info.setUpkeep( rs.getInt( "upkeep" ) );
				info.setFlightTime( rs.getInt( "flight_time" ) );
				info.setPower( rs.getInt( "power" ) );
				return info;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fFlushBuildQueue = new StoredProc( dataSource , "verse" , "flush_build_queue" );
		this.fFlushBuildQueue.addParameter( "planet_id" , Types.INTEGER );

		this.fFlushMilitaryQueue = new StoredProc( dataSource , "verse" , "flush_military_queue" );
		this.fFlushMilitaryQueue.addParameter( "planet_id" , Types.INTEGER );

		this.fAddToMilitaryQueue = new StoredProc( dataSource , "verse" , "add_military_item" );
		this.fAddToMilitaryQueue.addParameter( "planet_id" , Types.INTEGER );
		this.fAddToMilitaryQueue.addParameter( "ship_id" , Types.INTEGER );
		this.fAddToMilitaryQueue.addParameter( "amount" , Types.INTEGER );

		this.fConstructBuildings = new StoredProc( dataSource , "verse" , "construct_buildings" );
		this.fConstructBuildings.addParameter( "planet_id" , Types.INTEGER );
		this.fConstructBuildings.addParameter( "building_id" , Types.INTEGER );
		this.fConstructBuildings.addParameter( "amount" , Types.INTEGER );

		this.fDestroyBuildings = new StoredProc( dataSource , "verse" , "destroy_buildings" );
		this.fDestroyBuildings.addParameter( "planet_id" , Types.INTEGER );
		this.fDestroyBuildings.addParameter( "building_id" , Types.INTEGER );
		this.fDestroyBuildings.addParameter( "amount" , Types.INTEGER );
		this.fDestroyBuildings.addOutput( "success" , Types.BOOLEAN );

		this.fAbandon = new StoredProc( dataSource , "verse" , "abandon" );
		this.fAbandon.addParameter( "planet_id" , Types.INTEGER );
		this.fAbandon.addOutput( "tta" , Types.INTEGER );

		this.fCancelAbandon = new StoredProc( dataSource , "verse" , "cancel_abandon" );
		this.fCancelAbandon.addParameter( "planet_id" , Types.INTEGER );
	}


	@Override
	public Basic getBasicInformation( int empireId , int planetId )
	{
		String sql = "SELECT * FROM verse.get_planet_basics( ? , ? )";
		return this.dTemplate.queryForObject( sql , this.mPlanetBasics , empireId , planetId );
	}


	@Override
	public Orbital getOrbitalInformation( int empireId , int planetId )
	{
		String sql = "SELECT * FROM verse.get_orbital_view( ? , ? )";
		return this.dTemplate.queryForObject( sql , this.mPlanetOrbital , empireId , planetId );
	}


	@Override
	public Owner getOwnerInformation( int empireId , int planetId )
	{
		String sql = "SELECT * FROM verse.get_owner_view( ? , ? )";
		return this.dTemplate.queryForObject( sql , this.mPlanetOwner , empireId , planetId );
	}


	@Override
	public List< BuildingData > getBuildings( int empireId , int planetId , boolean isOwner )
	{
		RowMapper< BuildingData > mapper = isOwner ? this.mOwnerBuildings : this.mOrbitalBuildings;
		String sql = "SELECT * FROM verse.get_buildings_view( ? , ? )";
		return this.dTemplate.query( sql , mapper , empireId , planetId );
	}


	@Override
	public QueueData getConstructionQueue( int planetId )
	{
		String sql = "SELECT * FROM verse.get_build_queue( ? )";
		List< QueueItemData > items = this.dTemplate.query( sql , this.mQueueItem , planetId );
		return new QueueData( ( items.size( ) < 5 ) , items );
	}


	@Override
	public QueueData getMilitaryQueue( int planetId )
	{
		String sql = "SELECT * FROM verse.get_military_queue( ? )";
		List< QueueItemData > items = this.dTemplate.query( sql , this.mQueueItem , planetId );
		return new QueueData( ( items.size( ) < 5 ) , items );
	}


	@Override
	public List< BuildableBuildingData > getAvailableBuildings( int planetId )
	{
		String sql = "SELECT * FROM verse.get_available_buildings( ? )";
		return this.dTemplate.query( sql , this.mAvailableBuilding , planetId );
	}


	@Override
	public List< BuildableShipData > getAvailableShips( int planetId )
	{
		String sql = "SELECT * FROM verse.get_available_ships( ? )";
		return this.dTemplate.query( sql , this.mAvailableShip , planetId );
	}


	@Override
	public void flushQueue( int planetId , boolean military )
	{
		StoredProc proc = military ? this.fFlushMilitaryQueue : this.fFlushBuildQueue;
		proc.execute( planetId );
	}


	@Override
	public void addToMilitaryQueue( int planetId , int sType , int amount )
	{
		this.fAddToMilitaryQueue.execute( planetId , sType , amount );
	}


	@Override
	public void constructBuildings( int planetId , int bType , int amount )
	{
		this.fConstructBuildings.execute( planetId , bType , amount );
	}


	@Override
	public boolean destroyBuildings( int planetId , int bType , int amount )
	{
		return (Boolean) this.fDestroyBuildings.execute( planetId , bType , amount ).get( "success" );
	}


	@Override
	public Integer abandon( int planetId )
	{
		int v;
		v = (Integer) this.fAbandon.execute( planetId ).get( "tta" );
		return v == 0 ? null : v;
	}


	@Override
	public void cancelAbandon( int planetId )
	{
		this.fCancelAbandon.execute( planetId );
	}
}
