package com.deepclone.lw.beans.msgs;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.admin.adata.AdministratorBasics;
import com.deepclone.lw.cmd.admin.naming.Name;
import com.deepclone.lw.cmd.admin.naming.NameType;
import com.deepclone.lw.cmd.msgdata.Message;
import com.deepclone.lw.cmd.msgdata.MessageListEntry;
import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.alliance.PublicAllianceInformation;
import com.deepclone.lw.cmd.player.msgs.ComposeMessageResponse;
import com.deepclone.lw.cmd.player.msgs.GetMessagesResponse;
import com.deepclone.lw.cmd.player.msgs.ListTargetsResponse;
import com.deepclone.lw.cmd.player.msgs.ReadMessageResponse;
import com.deepclone.lw.interfaces.acm.UsersDAO;
import com.deepclone.lw.interfaces.admin.AdminDAO;
import com.deepclone.lw.interfaces.game.AllianceDAO;
import com.deepclone.lw.interfaces.game.EmpireDAO;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.msg.MessageBoxDAO;
import com.deepclone.lw.interfaces.msg.MessageContentCache;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatRegistry;
import com.deepclone.lw.interfaces.msg.EmpireMessages;
import com.deepclone.lw.interfaces.naming.NamingDAO;
import com.deepclone.lw.sqld.admin.AdminRecord;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Transactional
public class EmpireMessagesBean
		implements EmpireMessages
{

	private EmpireManagement empireManager;
	private MessageBoxDAO mboxDao;
	private MessageContentCache contentCache;
	private MessageFormatRegistry formats;
	private EmpireDAO empireDao;
	private AllianceDAO allianceDao;
	private UsersDAO usersDao;
	private AdminDAO adminDao;
	private NamingDAO namingDao;


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement empireManager )
	{
		this.empireManager = empireManager;
	}


	@Autowired( required = true )
	public void setMboxDao( MessageBoxDAO mboxDao )
	{
		this.mboxDao = mboxDao;
	}


	@Autowired( required = true )
	public void setContentCache( MessageContentCache contentCache )
	{
		this.contentCache = contentCache;
	}


	@Autowired( required = true )
	public void setFormats( MessageFormatRegistry formats )
	{
		this.formats = formats;
	}


	@Autowired( required = true )
	public void setEmpireDao( EmpireDAO empireDao )
	{
		this.empireDao = empireDao;
	}


	@Autowired( required = true )
	public void setAllianceDao( AllianceDAO allianceDao )
	{
		this.allianceDao = allianceDao;
	}


	@Autowired( required = true )
	public void setUsersDao( UsersDAO usersDao )
	{
		this.usersDao = usersDao;
	}


	@Autowired( required = true )
	public void setAdminDao( AdminDAO adminDao )
	{
		this.adminDao = adminDao;
	}


	@Autowired( required = true )
	public void setNamingDao( NamingDAO namingDao )
	{
		this.namingDao = namingDao;
	}


	private Message readEmpireMessage( int empireId , boolean inbox , long id )
	{
		int accountId = this.empireDao.getInformation( empireId ).getAccountId( );
		String lang = this.usersDao.getAccount( accountId ).getLanguage( );

		// Get list of messages
		List< InboxRecord > mbox = this.mboxDao.getList( false , empireId , inbox );
		Map< Long , MessageDataRecord > contents = this.contentCache.getContent( mbox );
		if ( !contents.containsKey( id ) ) {
			return null;
		}

		// Find next and previous messages
		Long previous = null;
		Long next = null;
		InboxRecord record = null;
		boolean found = false;
		for ( InboxRecord message : mbox ) {
			if ( !found ) {
				if ( message.getId( ) == id ) {
					found = true;
					record = message;
				} else {
					previous = message.getId( );
				}
			} else if ( next == null ) {
				next = message.getId( );
				break;
			}
		}

		MessageDataRecord content = contents.get( record.getId( ) );
		MessageExtractor formatter = this.formats.getFormatter( content ).getExtractor( record , content , lang );

		// Extract basic information
		Message message = new Message( );
		message.setId( id );
		message.setNext( next );
		message.setPrevious( previous );
		message.setTime( record.getReceived( ) );
		message.setSender( inbox ? formatter.getSender( ) : record.getSenderName( ) );
		message.setReceiver( record.getReceiverName( ) );
		message.setTitle( formatter.getSubject( ) );
		message.setContents( formatter.getContents( ) );
		message.setUnread( !record.isRead( ) );
		message.setGameTime( content.getTick( ) );

		// Handle message type
		if ( record.isInternal( ) ) {
			message.setType( MessageType.INTERNAL );
		} else {
			String type = inbox ? record.getSenderType( ) : record.getReceiverType( );
			if ( "ALL".equals( record.getReceiverType( ) ) ) {
				message.setType( MessageType.ALLIANCE );
				message.setReceiver( "[" + message.getReceiver( ) + "]" );
			} else if ( "EMP".equals( type ) ) {
				message.setType( MessageType.EMPIRE );
			} else if ( "ADM".equals( type ) ) {
				message.setType( MessageType.ADMINISTRATOR );
			} else {
				throw new RuntimeException( "unknown message type " + type );
			}
		}

		return message;
	}


	@Override
	public GetMessagesResponse getMessages( int empireId , boolean inbox )
	{
		int accountId = this.empireDao.getInformation( empireId ).getAccountId( );
		String lang = this.usersDao.getAccount( accountId ).getLanguage( );

		// Get list of messages
		List< InboxRecord > mbox = this.mboxDao.getList( false , empireId , inbox );
		Map< Long , MessageDataRecord > contents = this.contentCache.getContent( mbox );

		// Format messages and create list of entries
		List< MessageListEntry > results = new LinkedList< MessageListEntry >( );
		for ( InboxRecord mRecord : mbox ) {
			MessageDataRecord content = contents.get( mRecord.getId( ) );
			MessageExtractor formatter = this.formats.getFormatter( content ).getExtractor( mRecord , content , lang );

			MessageListEntry result = new MessageListEntry( );
			result.setId( mRecord.getId( ) );
			result.setTime( mRecord.getReceived( ) );
			result.setRead( mRecord.isRead( ) );
			result.setTitle( formatter.getSubject( ) );
			result.setSender( inbox ? formatter.getSender( ) : mRecord.getReceiverName( ) );
			if ( mRecord.isInternal( ) ) {
				result.setType( MessageType.INTERNAL );
			} else {
				String type = inbox ? mRecord.getSenderType( ) : mRecord.getReceiverType( );
				if ( "ADM".equals( type ) ) {
					result.setType( MessageType.ADMINISTRATOR );
				} else if ( "ALL".equals( mRecord.getReceiverType( ) ) ) {
					result.setType( MessageType.ALLIANCE );
					result.setSender( "[" + mRecord.getReceiverName( ) + "]" );
				} else if ( "EMP".equals( type ) ) {
					result.setType( MessageType.EMPIRE );
				} else {
					throw new RuntimeException( "unknown message type " + type );
				}
			}
			results.add( result );
		}

		return new GetMessagesResponse( this.empireManager.getGeneralInformation( empireId ) , results );
	}


	@Override
	public ReadMessageResponse getMessage( int empireId , boolean inbox , long id )
	{
		// Get list of messages
		Message message = this.readEmpireMessage( empireId , inbox , id );
		if ( message == null ) {
			return new ReadMessageResponse( this.empireManager.getGeneralInformation( empireId ) , inbox );
		}

		// Mark message as read
		if ( inbox && message.isUnread( ) ) {
			this.mboxDao.markRead( false , empireId , new long[] {
				id
			} );
		}

		return new ReadMessageResponse( this.empireManager.getGeneralInformation( empireId ) , inbox , message );
	}


	@Override
	public void deleteMessages( int empireId , boolean inbox , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.delete( false , empireId , inbox );
		} else {
			this.mboxDao.delete( false , empireId , inbox , selection );
		}
	}


	@Override
	public void markRead( int empireId , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.markRead( false , empireId );
		} else {
			this.mboxDao.markRead( false , empireId , selection );
		}
	}


	@Override
	public void markUnread( int empireId , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.markUnread( false , empireId );
		} else {
			this.mboxDao.markUnread( false , empireId , selection );
		}
	}


	@Override
	public ComposeMessageResponse prepareBlankMessage( int empireId )
	{
		ComposeMessageResponse response = new ComposeMessageResponse( this.empireManager
				.getGeneralInformation( empireId ) );
		response.setMessageType( MessageType.EMPIRE );
		response.setTarget( "" );
		response.setTitle( "" );
		response.setContents( "" );
		return response;
	}


	@Override
	public ComposeMessageResponse prepareMessageTo( int empireId , MessageType type , int id )
	{
		ComposeMessageResponse response = this.prepareBlankMessage( empireId );
		response.setMessageType( type );
		try {
			switch ( type ) {
				case EMPIRE:
					response.setTarget( this.empireDao.getInformation( id ).getName( ) );
					break;
				case ALLIANCE:
					response.setTarget( this.allianceDao.getPublicInformation( id ).getTag( ) );
					break;
				case ADMINISTRATOR:
					response.setTarget( this.adminDao.getAdmin( id ).getName( ) );
					break;
				default:
					throw new RuntimeException( "unsupported target type " + type );
			}
		} catch ( NullPointerException e ) {
			response.setTargetError( true );
		}
		return response;
	}


	@Override
	public ComposeMessageResponse sendMessage( int empireId , MessageType type , String target , String title ,
			String contents , boolean simulate )
	{
		int errCode = this.mboxDao.sendMessage( false , empireId , type , target , title , contents , simulate );
		if ( errCode == 0 && !simulate ) {
			return this.prepareBlankMessage( empireId );
		}

		ComposeMessageResponse response = new ComposeMessageResponse( this.empireManager
				.getGeneralInformation( empireId ) );
		return empireSendError( type , target , title , contents , errCode , response );
	}


	@Override
	public ComposeMessageResponse sendReply( int empireId , boolean inbox , long replyTo , MessageType type ,
			String target , String title , String contents , boolean simulate )
	{
		int errCode = this.mboxDao.sendMessage( false , empireId , type , target , title , contents , simulate );
		if ( errCode == 0 && !simulate ) {
			return this.prepareBlankMessage( empireId );
		}

		Message message = this.readEmpireMessage( empireId , inbox , replyTo );
		ComposeMessageResponse response;
		if ( message == null || message.getType( ) == MessageType.INTERNAL ) {
			response = new ComposeMessageResponse( this.empireManager.getGeneralInformation( empireId ) );
		} else {
			response = new ComposeMessageResponse( this.empireManager.getGeneralInformation( empireId ) , inbox ,
					message );
		}

		return empireSendError( type , target , title , contents , errCode , response );
	}


	private ComposeMessageResponse empireSendError( MessageType type , String target , String title , String contents ,
			int errCode , ComposeMessageResponse response )
	{
		switch ( errCode ) {
			case 1:
				response.setTargetError( true );
				break;
			case 2:
				response.setTimingError( true );
				break;
		}

		response.setMessageType( type );
		response.setTarget( target );
		response.setTitle( title );
		response.setContents( contents );

		return response;
	}


	@Override
	public ComposeMessageResponse prepareReply( int empireId , boolean inbox , long id )
	{
		Message message = this.readEmpireMessage( empireId , inbox , id );
		if ( message == null || message.getType( ) == MessageType.INTERNAL ) {
			return this.prepareBlankMessage( empireId );
		}

		ComposeMessageResponse response;
		response = new ComposeMessageResponse( this.empireManager.getGeneralInformation( empireId ) , inbox , message );

		response.setMessageType( message.getType( ) );
		if ( message.getType( ) == MessageType.ALLIANCE ) {
			String allTag = message.getReceiver( );
			response.setTarget( allTag.substring( 1 , allTag.length( ) - 1 ) );
		} else {
			response.setTarget( message.getSender( ) );
		}

		String title = message.getTitle( );
		if ( !title.startsWith( "Re: " ) ) {
			title = "Re: " + title;
			if ( title.length( ) > 64 ) {
				title = title.substring( 0 , 64 );
			}
		}
		response.setTitle( title );

		response.setContents( "" );
		return response;
	}


	@Override
	public ListTargetsResponse getTargets( int empireId )
	{
		// List all administrators
		List< AdministratorBasics > admins = new LinkedList< AdministratorBasics >( );
		for ( AdminRecord record : this.adminDao.listAdministrators( ) ) {
			if ( !record.isActive( ) ) {
				continue;
			}
			AdministratorBasics admin = new AdministratorBasics( );
			admin.setId( record.getId( ) );
			admin.setName( record.getName( ) );
			admin.setPrivileges( record.getPrivileges( ) );
			admins.add( admin );
		}

		// List active empires
		List< NameIdPair > empires = new LinkedList< NameIdPair >( );
		for ( Name n : this.namingDao.getNames( NameType.EMPIRE ) ) {
			if ( n.getExtra( ) == null || n.getId( ) == empireId ) {
				continue;
			}
			empires.add( new NameIdPair( n.getId( ) , n.getName( ) ) );
		}

		// List alliances
		List< PublicAllianceInformation > alliances = new LinkedList< PublicAllianceInformation >( );
		for ( Name n : this.namingDao.getNames( NameType.ALLIANCE ) ) {
			alliances.add( new PublicAllianceInformation( n.getId( ) , n.getName( ) , n.getExtra( ) , 0 , null , 0 ) );
		}

		return new ListTargetsResponse( this.empireManager.getGeneralInformation( empireId ) , empires , alliances ,
				admins );
	}

}
