package com.deepclone.lw.beans.msgs;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.cmd.player.msgs.MessageBoxAction;
import com.deepclone.lw.interfaces.msg.MessageBoxDAO;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.utils.StoredProc;



public class MessageBoxDAOBean
		implements MessageBoxDAO
{
	private static final int cacheTime = 6;

	private static class CacheKey
	{
		private final boolean admin;
		private final int id;
		private final boolean inbox;


		public CacheKey( boolean admin , int id , boolean inbox )
		{
			this.admin = admin;
			this.id = id;
			this.inbox = inbox;
		}


		@Override
		public int hashCode( )
		{
			final int prime = 31;
			int result = 1;
			result = prime * result + ( admin ? 1231 : 1237 );
			result = prime * result + id;
			result = prime * result + ( inbox ? 1231 : 1237 );
			return result;
		}


		@Override
		public boolean equals( Object obj )
		{
			if ( this == obj )
				return true;
			if ( obj == null )
				return false;
			if ( getClass( ) != obj.getClass( ) )
				return false;
			CacheKey other = (CacheKey) obj;
			if ( admin != other.admin )
				return false;
			if ( id != other.id )
				return false;
			if ( inbox != other.inbox )
				return false;
			return true;
		}

	}

	private static class CacheData
	{
		public int timeLeft = MessageBoxDAOBean.cacheTime;
		public long maxId = Long.MIN_VALUE;
		public List< InboxRecord > records = new LinkedList< InboxRecord >( );
		public Map< Long , MessageBoxAction > updates = new HashMap< Long , MessageBoxAction >( );
	}

	private final Map< CacheKey , CacheData > cache = new HashMap< CacheKey , CacheData >( );

	private JdbcTemplate dTemplate;
	private final RowMapper< InboxRecord > listMapper;

	private StoredProc fSendSpam;
	private StoredProc fEmpireSend;
	private StoredProc fEmpireMarkRead;
	private StoredProc fEmpireMarkReadAll;
	private StoredProc fEmpireDelete;
	private StoredProc fEmpireDeleteAll;
	private StoredProc fEmpireMarkUnread;
	private StoredProc fEmpireMarkUnreadAll;
	private StoredProc fAdminSend;
	private StoredProc fAdminMarkRead;
	private StoredProc fAdminMarkReadAll;
	private StoredProc fAdminDelete;
	private StoredProc fAdminDeleteAll;
	private StoredProc fAdminMarkUnread;
	private StoredProc fAdminMarkUnreadAll;


	public MessageBoxDAOBean( )
	{
		this.listMapper = new RowMapper< InboxRecord >( ) {
			@Override
			public InboxRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				InboxRecord iRec = new InboxRecord( );
				iRec.setId( rs.getLong( "id" ) );
				iRec.setRead( rs.getBoolean( "read" ) );
				iRec.setSenderType( rs.getString( "sender_type" ) );
				iRec.setSenderId( (Integer) rs.getObject( "sender_id" ) );
				iRec.setSenderName( rs.getString( "sender_name" ) );
				iRec.setReceiverType( rs.getString( "receiver_type" ) );
				iRec.setReceiverId( rs.getInt( "receiver_id" ) );
				iRec.setReceiverName( rs.getString( "receiver_name" ) );
				iRec.setInternal( rs.getBoolean( "internal_message" ) );
				iRec.setReceived( rs.getTimestamp( "r_time" ) );
				iRec.setContentId( rs.getLong( "content_id" ) );
				return iRec;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fSendSpam = new StoredProc( dataSource , "msgs" , "deliver_admin_spam" );
		this.fSendSpam.addParameter( "admin_id" , Types.INTEGER );
		this.fSendSpam.addParameter( "subject" , Types.VARCHAR );
		this.fSendSpam.addParameter( "contents" , "TEXT" );

		this.fEmpireSend = new StoredProc( dataSource , "msgs" , "send_message" );
		this.fEmpireSend.addParameter( "empire_id" , Types.INTEGER );
		this.fEmpireSend.addParameter( "to_type" , "receiver_type" );
		this.fEmpireSend.addParameter( "target" , Types.VARCHAR );
		this.fEmpireSend.addParameter( "subject" , Types.VARCHAR );
		this.fEmpireSend.addParameter( "contents" , "TEXT" );
		this.fEmpireSend.addParameter( "really_send" , Types.BOOLEAN );
		this.fEmpireSend.addOutput( "err_code" , Types.INTEGER );

		this.fEmpireMarkRead = new StoredProc( dataSource , "msgs" , "empire_mark_read" );
		this.fEmpireMarkRead.addParameter( "empire_id" , Types.INTEGER );
		this.fEmpireMarkRead.addParameter( "messages" , "BIGINT[]" );

		this.fEmpireMarkReadAll = new StoredProc( dataSource , "msgs" , "empire_mark_read" );
		this.fEmpireMarkReadAll.addParameter( "empire_id" , Types.INTEGER );

		this.fEmpireMarkUnread = new StoredProc( dataSource , "msgs" , "empire_mark_unread" );
		this.fEmpireMarkUnread.addParameter( "empire_id" , Types.INTEGER );
		this.fEmpireMarkUnread.addParameter( "messages" , "BIGINT[]" );

		this.fEmpireMarkUnreadAll = new StoredProc( dataSource , "msgs" , "empire_mark_unread" );
		this.fEmpireMarkUnreadAll.addParameter( "empire_id" , Types.INTEGER );

		this.fEmpireDelete = new StoredProc( dataSource , "msgs" , "empire_delete" );
		this.fEmpireDelete.addParameter( "empire_id" , Types.INTEGER );
		this.fEmpireDelete.addParameter( "in_inbox" , Types.BOOLEAN );
		this.fEmpireDelete.addParameter( "messages" , "BIGINT[]" );

		this.fEmpireDeleteAll = new StoredProc( dataSource , "msgs" , "empire_delete" );
		this.fEmpireDeleteAll.addParameter( "empire_id" , Types.INTEGER );
		this.fEmpireDeleteAll.addParameter( "in_inbox" , Types.BOOLEAN );

		this.fAdminSend = new StoredProc( dataSource , "msgs" , "send_admin_message" );
		this.fAdminSend.addParameter( "admin_id" , Types.INTEGER );
		this.fAdminSend.addParameter( "to_type" , "receiver_type" );
		this.fAdminSend.addParameter( "target" , Types.VARCHAR );
		this.fAdminSend.addParameter( "subject" , Types.VARCHAR );
		this.fAdminSend.addParameter( "contents" , "TEXT" );
		this.fAdminSend.addParameter( "really_send" , Types.BOOLEAN );
		this.fAdminSend.addOutput( "err_code" , Types.INTEGER );

		this.fAdminMarkRead = new StoredProc( dataSource , "msgs" , "admin_mark_read" );
		this.fAdminMarkRead.addParameter( "admin_id" , Types.INTEGER );
		this.fAdminMarkRead.addParameter( "messages" , "BIGINT[]" );

		this.fAdminMarkReadAll = new StoredProc( dataSource , "msgs" , "admin_mark_read" );
		this.fAdminMarkReadAll.addParameter( "admin_id" , Types.INTEGER );

		this.fAdminMarkUnread = new StoredProc( dataSource , "msgs" , "admin_mark_unread" );
		this.fAdminMarkUnread.addParameter( "admin_id" , Types.INTEGER );
		this.fAdminMarkUnread.addParameter( "messages" , "BIGINT[]" );

		this.fAdminMarkUnreadAll = new StoredProc( dataSource , "msgs" , "admin_mark_unread" );
		this.fAdminMarkUnreadAll.addParameter( "admin_id" , Types.INTEGER );

		this.fAdminDelete = new StoredProc( dataSource , "msgs" , "admin_delete" );
		this.fAdminDelete.addParameter( "admin_id" , Types.INTEGER );
		this.fAdminDelete.addParameter( "in_inbox" , Types.BOOLEAN );
		this.fAdminDelete.addParameter( "messages" , "BIGINT[]" );

		this.fAdminDeleteAll = new StoredProc( dataSource , "msgs" , "admin_delete" );
		this.fAdminDeleteAll.addParameter( "admin_id" , Types.INTEGER );
		this.fAdminDeleteAll.addParameter( "in_inbox" , Types.BOOLEAN );
	}


	private String getIdArray( long[] ids )
	{
		StringBuilder iArray = new StringBuilder( ).append( "{" );
		int i = 0;
		int s = ids.length;
		for ( Long id : ids ) {
			iArray.append( id );
			if ( i != s - 1 ) {
				iArray.append( "," );
			}
			i++;
		}
		iArray.append( "}" );
		return iArray.toString( );
	}


	@Override
	public int sendMessage( boolean admin , int empireId , MessageType type , String target , String title ,
			String contents , boolean simulate )
	{
		StoredProc send = admin ? this.fAdminSend : this.fEmpireSend;
		Map< String , Object > result;
		result = send.execute( empireId , type.toString( ).substring( 0 , 3 ) , target , title , contents , !simulate );
		return (Integer) result.get( "err_code" );
	}


	@Override
	public List< InboxRecord > getList( boolean admin , int id , boolean inbox )
	{
		CacheKey key = new CacheKey( admin , id , inbox );
		CacheData data;

		synchronized ( this.cache ) {
			data = this.cache.get( key );
			if ( data == null ) {
				data = new CacheData( );
				this.cache.put( key , data );
			} else if ( !data.updates.isEmpty( ) ) {
				this.applyUpdates( data );
			}
			this.getNewRecords( admin , id , inbox , data );
			data.timeLeft = MessageBoxDAOBean.cacheTime;
		}

		return data.records;
	}


	private void applyUpdates( CacheData data )
	{
		Iterator< InboxRecord > iterator = data.records.iterator( );
		while ( iterator.hasNext( ) ) {
			InboxRecord record = iterator.next( );
			MessageBoxAction update = data.updates.get( record.getId( ) );
			if ( update == null ) {
				continue;
			}

			switch ( update ) {
				case DELETE:
					iterator.remove( );
					break;
				case MARK_READ:
					record.setRead( true );
					break;
				case MARK_UNREAD:
					record.setRead( false );
					break;
			}
		}
		data.updates.clear( );
	}


	private void getNewRecords( boolean admin , int id , boolean inbox , CacheData data )
	{
		String tString = ( admin ? "admin" : "empire" );
		String bString = ( inbox ? "in" : "out" );
		String sql = "SELECT * FROM msgs." + tString + "_" + bString + "box WHERE " + tString + "_id = ? AND id > ?";
		List< InboxRecord > newRecords = this.dTemplate.query( sql , this.listMapper , id , data.maxId );
		if ( newRecords.isEmpty( ) ) {
			return;
		}

		data.records.addAll( 0 , newRecords );
		data.maxId = newRecords.get( 0 ).getId( );
	}


	@Override
	public void markRead( boolean admin , int userId , long[] ids )
	{
		synchronized ( this.cache ) {
			CacheData data = this.cache.get( new CacheKey( admin , userId , true ) );
			if ( data != null ) {
				for ( long id : ids ) {
					if ( data.updates.containsKey( id ) ) {
						continue;
					}
					data.updates.put( id , MessageBoxAction.MARK_READ );
				}
			}

			StoredProc dbMark = admin ? this.fAdminMarkRead : this.fEmpireMarkRead;
			dbMark.execute( userId , this.getIdArray( ids ) );
		}
	}


	@Override
	public void markRead( boolean admin , int userId )
	{
		synchronized ( this.cache ) {
			this.cache.remove( new CacheKey( admin , userId , true ) );

			StoredProc dbMark = admin ? this.fAdminMarkReadAll : this.fEmpireMarkReadAll;
			dbMark.execute( userId );
		}
	}


	@Override
	public void markUnread( boolean admin , int userId , long[] ids )
	{
		synchronized ( this.cache ) {
			CacheData data = this.cache.get( new CacheKey( admin , userId , true ) );
			if ( data != null ) {
				for ( long id : ids ) {
					if ( data.updates.containsKey( id ) ) {
						continue;
					}
					data.updates.put( id , MessageBoxAction.MARK_UNREAD );
				}
			}

			StoredProc dbMark = admin ? this.fAdminMarkUnread : this.fEmpireMarkUnread;
			dbMark.execute( userId , this.getIdArray( ids ) );
		}
	}


	@Override
	public void markUnread( boolean admin , int userId )
	{
		synchronized ( this.cache ) {
			this.cache.remove( new CacheKey( admin , userId , true ) );
			StoredProc dbMark = admin ? this.fAdminMarkUnreadAll : this.fEmpireMarkUnreadAll;
			dbMark.execute( userId );
		}
	}


	@Override
	public void delete( boolean admin , int userId , boolean inbox , long[] ids )
	{
		synchronized ( this.cache ) {
			CacheData data = this.cache.get( new CacheKey( admin , userId , inbox ) );
			if ( data != null ) {
				for ( long id : ids ) {
					data.updates.put( id , MessageBoxAction.DELETE );
				}
			}

			StoredProc dbDelete = admin ? this.fAdminDelete : this.fEmpireDelete;
			dbDelete.execute( userId , inbox , this.getIdArray( ids ) );
		}
	}


	@Override
	public void delete( boolean admin , int userId , boolean inbox )
	{
		synchronized ( this.cache ) {
			this.cache.remove( new CacheKey( admin , userId , inbox ) );

			StoredProc dbDelete = admin ? this.fAdminDeleteAll : this.fEmpireDeleteAll;
			dbDelete.execute( userId , inbox );
		}
	}


	@Override
	public void clearCache( )
	{
		List< CacheKey > toRemove = new LinkedList< CacheKey >( );

		synchronized ( this.cache ) {
			for ( Map.Entry< CacheKey , CacheData > entry : this.cache.entrySet( ) ) {
				CacheData data = entry.getValue( );
				data.timeLeft--;
				if ( data.timeLeft == 0 ) {
					toRemove.add( entry.getKey( ) );
				}
			}

			for ( CacheKey key : toRemove ) {
				this.cache.remove( key );
			}
		}
	}


	@Override
	public void sendSpam( int adminId , String title , String body )
	{
		this.fSendSpam.execute( adminId , title , body );
	}
}
