package com.deepclone.lw.beans.msgs;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.LinkedList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.interfaces.msg.NotificationsDAO;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.NotificationsRecord;
import com.deepclone.lw.utils.StoredProc;



public class NotificationsDAOBean
		implements NotificationsDAO
{
	private static final String sGetInstantNotifications = "SELECT * FROM msgs.get_mail_data() WHERE last_unmailed IS NOT NULL";
	private static final String sGetRecapNotifications = "SELECT * FROM msgs.get_mail_data() WHERE last_unrecaped IS NOT NULL";

	private JdbcTemplate dTemplate;

	private final RowMapper< NotificationsRecord > mNotifications;
	private RowMapper< InboxRecord > mList;
	private StoredProc fMarkInstantMessages;
	private StoredProc fMarkRecap;


	public NotificationsDAOBean( )
	{
		this.mNotifications = new RowMapper< NotificationsRecord >( ) {
			@Override
			public NotificationsRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				NotificationsRecord nr = new NotificationsRecord( );
				nr.setId( rs.getInt( "empire" ) );
				nr.setName( rs.getString( "empire_name" ) );
				nr.setLanguage( rs.getString( "language" ) );
				nr.setAddress( rs.getString( "address" ) );
				nr.setOnPrivate( rs.getString( "on_private" ) );
				nr.setOnInternal( rs.getString( "on_internal" ) );
				nr.setOnAlliance( rs.getString( "on_alliance" ) );
				nr.setOnAdmin( rs.getString( "on_admin" ) );
				nr.setLastUnmailed( (Long) rs.getObject( "last_unmailed" ) );
				nr.setLastUnrecaped( (Long) rs.getObject( "last_unrecaped" ) );
				nr.setCanNotify( rs.getBoolean( "can_notify" ) );
				return nr;
			}
		};
		this.mList = new RowMapper< InboxRecord >( ) {
			@Override
			public InboxRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				InboxRecord iRec = new InboxRecord( );
				iRec.setId( rs.getLong( "id" ) );
				iRec.setSenderType( rs.getString( "sender_type" ) );
				iRec.setSenderId( (Integer) rs.getObject( "sender_id" ) );
				iRec.setSenderName( rs.getString( "sender_name" ) );
				iRec.setReceiverType( rs.getString( "receiver_type" ) );
				iRec.setReceiverId( rs.getInt( "receiver_id" ) );
				iRec.setReceiverName( rs.getString( "receiver_name" ) );
				iRec.setInternal( rs.getBoolean( "internal_message" ) );
				iRec.setReceived( rs.getTimestamp( "r_time" ) );
				iRec.setContentId( rs.getLong( "content_id" ) );
				return iRec;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fMarkInstantMessages = new StoredProc( dataSource , "msgs" , "mark_instant_notifications" );
		this.fMarkInstantMessages.addParameter( "empire_id" , Types.INTEGER );
		this.fMarkInstantMessages.addParameter( "message_id" , Types.BIGINT );
		this.fMarkInstantMessages.addParameter( "sending" , Types.BOOLEAN );

		this.fMarkRecap = new StoredProc( dataSource , "msgs" , "mark_recaps" );
		this.fMarkRecap.addParameter( "empire_id" , Types.INTEGER );
		this.fMarkRecap.addParameter( "message_id" , Types.BIGINT );
	}


	@Override
	public boolean isRecapTime( )
	{
		String sql = "SELECT msgs.is_recap_time() AS recap";
		return (Boolean) this.dTemplate.queryForMap( sql ).get( "recap" );
	}


	@Override
	public List< NotificationsRecord > getNotificationRecords( boolean instant )
	{
		String sql = instant ? sGetInstantNotifications : sGetRecapNotifications;
		return this.dTemplate.query( sql , this.mNotifications );
	}


	@Override
	public List< InboxRecord > listMessages( int empireId , long maxId , boolean instant , boolean nPrivate ,
			boolean nInternal , boolean nAlliance , boolean nAdmin )
	{
		List< InboxRecord > result;
		if ( nPrivate || nInternal || nAlliance || nAdmin ) {
			String sql = "SELECT * FROM msgs.empire_" + ( instant ? "instant" : "recap" );
			sql += " WHERE empire_id = ? AND id <= ?";

			if ( ! ( nPrivate && nInternal && nAlliance && nAdmin ) ) {
				sql += " AND (";
				if ( nPrivate ) {
					sql += "( sender_type = 'EMP' AND receiver_type='EMP' )";
				}
				if ( nInternal ) {
					if ( nPrivate ) {
						sql += " OR ";
					}
					sql += "sender_type = 'INT'";
				}
				if ( nAlliance ) {
					if ( nPrivate || nInternal ) {
						sql += " OR ";
					}
					sql += "receiver_type = 'ALL'";
				}
				if ( nAdmin ) {
					if ( nPrivate || nInternal || nAlliance ) {
						sql += " OR ";
					}
					sql += "sender_type = 'ADM'";
				}
				sql += ")";
			}

			result = this.dTemplate.query( sql , this.mList , empireId , maxId );
		} else {
			result = new LinkedList< InboxRecord >( );
		}

		if ( instant ) {
			this.fMarkInstantMessages.execute( empireId , maxId , !result.isEmpty( ) );
		} else {
			this.fMarkRecap.execute( empireId , maxId );
		}

		return result;
	}


	@Override
	public void cleanupMessages( )
	{
		this.dTemplate.execute( "SELECT msgs.cleanup( )" );
	}
}
