package com.deepclone.lw.beans.msgs;


import org.springframework.transaction.PlatformTransactionManager;

import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.cmd.msgdata.Message;
import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.mailer.MailData;
import com.deepclone.lw.interfaces.mailer.Mailer;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.interfaces.msg.MessageContentCache;
import com.deepclone.lw.interfaces.msg.MessageFormatRegistry;
import com.deepclone.lw.interfaces.msg.NotificationsDAO;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.NotificationsRecord;



class NotificationsTask
		extends MailTaskBase
{

	public NotificationsTask( Logger logger , PlatformTransactionManager tManager , NotificationsDAO notificationsDao ,
			MessageContentCache contents , MessageFormatRegistry formats , Translator translator , Mailer mailer ,
			MessageCleanerBean cleaner )
	{
		super( logger , tManager , notificationsDao , contents , formats , translator , mailer , cleaner , true );
	}


	protected void notify( NotificationsRecord notification )
	{
		boolean nPrivate = "INSTANT".equals( notification.getOnPrivate( ) ) && notification.isCanNotify( );
		boolean nInternal = "INSTANT".equals( notification.getOnInternal( ) ) && notification.isCanNotify( );
		boolean nAlliance = "INSTANT".equals( notification.getOnAlliance( ) ) && notification.isCanNotify( );
		boolean nAdmin = "INSTANT".equals( notification.getOnAdmin( ) ) && notification.isCanNotify( );

		MessageData toSend = this.getMessagesToSend( notification.getId( ) , notification.getLastUnmailed( ) ,
				nPrivate , nInternal , nAlliance , nAdmin );
		if ( toSend.messages.isEmpty( ) ) {
			return;
		}

		this.logger.log( LogLevel.DEBUG , "Processing messages for empire " + notification.getName( ) );
		this.processMessages( notification , toSend );
	}


	private void processMessages( NotificationsRecord notification , MessageData toSend )
	{
		StringBuilder buffer = new StringBuilder( );
		String lang = notification.getLanguage( );

		String template = this.translate( "instantNotification" , lang );
		int empireId = notification.getId( );
		for ( InboxRecord record : toSend.messages ) {
			Message message = this.readMessage( empireId , lang , record , toSend.data.get( record.getId( ) ) , true );
			this.outputMessage( buffer , template , message );
		}

		MailData mailData;
		try {
			mailData = this.mailer.createMail( lang , "messageMail" , notification.getAddress( ) );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}

		mailData.setData( "empire" , notification.getName( ) );
		mailData.setData( "messages" , buffer.toString( ) );

		try {
			mailData.queue( );
		} catch ( MailerException e ) {
			throw new RuntimeException( e );
		}
	}


	private void outputMessage( StringBuilder buffer , String template , Message message )
	{
		String data = template.replace( "${from}" , message.getSender( ) );
		data = data.replace( "${to}" , message.getReceiver( ) );
		if ( message.getType( ) == MessageType.INTERNAL ) {
			data = data.replace( "${subject}" , message.getTitle( ) );
			data = data.replace( "${text}" , this.cleaner.removeInternals( message.getContents( ) ) );
		} else {
			data = data.replace( "${subject}" , this.cleaner.cleanMessage( message.getTitle( ) ) );
			data = data.replace( "${text}" , this.cleaner.cleanMessage( message.getContents( ) ) );
		}
		buffer.append( data );
	}

}
