package com.deepclone.lw.beans.msgs;


import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;

import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.cmd.msgdata.Message;
import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.mailer.MailData;
import com.deepclone.lw.interfaces.mailer.Mailer;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.interfaces.msg.MessageBoxDAO;
import com.deepclone.lw.interfaces.msg.MessageContentCache;
import com.deepclone.lw.interfaces.msg.MessageFormatRegistry;
import com.deepclone.lw.interfaces.msg.NotificationsDAO;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.NotificationsRecord;



class RecapitulationTask
		extends MailTaskBase
{

	private MessageBoxDAO mboxDao;


	public RecapitulationTask( Logger logger , PlatformTransactionManager tManager , NotificationsDAO notificationsDao ,
			MessageContentCache contents , MessageFormatRegistry formats , Translator translator , Mailer mailer ,
			MessageCleanerBean cleaner , MessageBoxDAO mbox )
	{
		super( logger , tManager , notificationsDao , contents , formats , translator , mailer , cleaner , false );
		this.mboxDao = mbox;
	}


	@Override
	public void run( )
	{
		if ( this.isTime( ) ) {
			super.run( );
		}

		this.cleanup( );
		this.contents.clearCache( );
		this.mboxDao.clearCache( );
	}


	private void cleanup( )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				notificationsDao.cleanupMessages( );
			}
		} );
	}


	private boolean isTime( )
	{
		return this.tTemplate.execute( new TransactionCallback< Boolean >( ) {
			@Override
			public Boolean doInTransaction( TransactionStatus status )
			{
				return notificationsDao.isRecapTime( );
			}
		} );
	}


	protected void notify( NotificationsRecord notification )
	{
		boolean nPrivate = "DAILY_RECAP".equals( notification.getOnPrivate( ) );
		boolean nInternal = "DAILY_RECAP".equals( notification.getOnInternal( ) );
		boolean nAlliance = "DAILY_RECAP".equals( notification.getOnAlliance( ) );
		boolean nAdmin = "DAILY_RECAP".equals( notification.getOnAdmin( ) );

		MessageData toSend = this.getMessagesToSend( notification.getId( ) , notification.getLastUnrecaped( ) ,
				nPrivate , nInternal , nAlliance , nAdmin );
		if ( toSend.messages.isEmpty( ) ) {
			return;
		}

		this.logger.log( LogLevel.DEBUG , "Processing messages for empire " + notification.getName( ) );
		this.processMessages( notification , toSend );
	}


	private void processMessages( NotificationsRecord notification , MessageData toSend )
	{
		String lang = notification.getLanguage( );

		Map< MessageType , List< Message > > messages = new HashMap< MessageType , List< Message > >( );
		for ( MessageType type : MessageType.values( ) ) {
			messages.put( type , new LinkedList< Message >( ) );
		}

		int empireId = notification.getId( );
		for ( InboxRecord record : toSend.messages ) {
			Message message = this.readMessage( empireId , lang , record , toSend.data.get( record.getId( ) ) , false );
			messages.get( message.getType( ) ).add( message );
		}

		StringBuilder body = new StringBuilder( );
		String template = this.translate( "recapsMessage" , lang );
		for ( MessageType type : MessageType.values( ) ) {
			List< Message > tMessages = messages.get( type );
			if ( tMessages.isEmpty( ) ) {
				continue;
			}

			StringBuilder typeList = new StringBuilder( );
			typeList.append( this.getTypeTitle( type , lang ) ).append( "\n\n" );
			for ( Message message : tMessages ) {
				typeList.append( this.outputMessage( template , message ) ).append( "\n" );
			}
			body.append( typeList ).append( "\n" );
		}

		MailData mailData;
		try {
			mailData = this.mailer.createMail( lang , "recapMail" , notification.getAddress( ) );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}

		mailData.setData( "empire" , notification.getName( ) );
		mailData.setData( "messages" , body.toString( ) );

		try {
			mailData.queue( );
		} catch ( MailerException e ) {
			throw new RuntimeException( e );
		}
	}


	private String getTypeTitle( MessageType type , String lang )
	{
		String tString = type.toString( );
		return this.translate( "recaps" + tString.substring( 0 , 1 ) + tString.substring( 1 ).toLowerCase( ) , lang );
	}


	private String outputMessage( String template , Message message )
	{
		String data = template.replace( "${from}" , message.getSender( ) );
		if ( message.getType( ) == MessageType.INTERNAL ) {
			data = data.replace( "${subject}" , message.getTitle( ) );
		} else {
			data = data.replace( "${subject}" , this.cleaner.cleanMessage( message.getTitle( ) ) );
		}
		SimpleDateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss Z" );
		data = data.replace( "${time}" , dateFormat.format( message.getTime( ) ) );
		return data;
	}

}
