package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;
import com.deepclone.lw.sqld.msgs.PlanetEventRecord;



@Component
public class AbandonMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class AbandonMessageExtractor
			extends PlanetMessageExtractor
	{
		private String mTitle;
		private String mBody;
		private String mSource;


		public AbandonMessageExtractor( PlanetEventRecord event , String mTitle , String mBody , String mSource )
		{
			super(event);
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			return this.mBody.replace( "${location}" , this.getLocation( ) );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle.replace( "${location}" , this.event.getLocationName( ) );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.PLANET , 5 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		PlanetEventRecord event = (PlanetEventRecord) contents;
		String mTitle , mBody , mSource;
		try {
			mTitle = this.translator.translate( language , "imtAbandonPlanet" );
			mBody = this.translator.translate( language , "imAbandonPlanet" );
			mSource = this.translator.translate( language , "imSenderEco" );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new AbandonMessageExtractor( event , mTitle , mBody , mSource );
	}

}
