package com.deepclone.lw.beans.msgs.fmt;


import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.sqld.msgs.FleetEventFleet;
import com.deepclone.lw.sqld.msgs.FleetEventRecord;



public class FleetMessageExtractor
		implements MessageExtractor
{
	private FleetEventRecord event;
	private Translator translator;
	private String language;
	private String eType;
	private int empireId;


	private String translate( String what )
	{
		try {
			return this.translator.translate( this.language , what );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
	}


	public FleetMessageExtractor( int empireId , FleetEventRecord event , Translator translator , String language )
	{
		this.empireId = empireId;
		this.event = event;
		this.translator = translator;
		this.language = language;
		switch ( this.event.getSubType( ) ) {
			case 0:
				this.eType = "Arrival";
				break;
			case 1:
				this.eType = "Departure";
				break;
			case 2:
				this.eType = "Switch";
				break;
			case 3:
				this.eType = "ELSwitch";
				break;
			default:
				throw new RuntimeException( "unsupported event sub-type " + this.event.getSubType( ) );
		}
	}


	@Override
	public String getSender( )
	{
		return this.translate( "imSenderWar" );
	}


	@Override
	public String getSubject( )
	{
		return this.translate( "imtFleet" + this.eType ).replace( "${location}" , this.event.getLocationName( ) );
	}


	@Override
	public String getContents( )
	{
		String body = this.translate( "imFleet" + this.eType );
		String fList;
		if ( this.event.getSubType( ) == 2 ) {
			fList = this.makeSwitchList( );
		} else if ( this.event.getSubType( ) == 0 ) {
			fList = this.makeArrivalList( );
		} else {
			fList = this.makeStandardList( );
		}
		String locLink = "{{planet:" + this.event.getLocationId( ) + " " + this.event.getLocationName( ) + "}} (";
		locLink += this.event.getX( ) + "," + this.event.getY( ) + ";" + this.event.getOrbit( ) + ")";
		return body.replace( "${location}" , locLink ) + fList;
	}


	private String makeStandardList( )
	{
		String namedOwn = this.translate( "imfOwnNamed" );
		String unnamedOwn = this.translate( "imfOwnUnnamed" );
		String namedFriendly = this.translate( "imfFriendlyNamed" );
		String unnamedFriendly = this.translate( "imfFriendlyUnnamed" );
		String namedHostile = this.translate( "imfHostileNamed" );
		String unnamedHostile = this.translate( "imfHostileUnnamed" );
		StringBuilder sBuilder = new StringBuilder( );

		for ( FleetEventFleet fleet : this.event.getFleets( ) ) {
			sBuilder.append( "\n* " );
			String named , unnamed;
			if ( fleet.getOwnerId( ) != null && fleet.getOwnerId( ) == this.empireId ) {
				named = namedOwn;
				unnamed = unnamedOwn;
			} else if ( fleet.getStatus( ) == true ) {
				named = namedHostile;
				unnamed = unnamedHostile;
			} else {
				named = namedFriendly;
				unnamed = unnamedFriendly;
			}
			sBuilder.append( this.fleetLine( fleet , named , unnamed ) );
		}

		return sBuilder.toString( );
	}


	private String makeArrivalList( )
	{
		String namedOwn = this.translate( "imfOwnNamed" );
		String unnamedOwn = this.translate( "imfOwnUnnamed" );
		String namedFriendly = this.translate( "imfFriendlyNamed" );
		String unnamedFriendly = this.translate( "imfFriendlyUnnamed" );
		String namedHostile = this.translate( "imfHostileNamed" );
		String unnamedHostile = this.translate( "imfHostileUnnamed" );
		String arrival = this.translate( "imfSource" );
		StringBuilder sBuilder = new StringBuilder( );

		for ( FleetEventFleet fleet : this.event.getFleets( ) ) {
			sBuilder.append( "\n* " );
			String named , unnamed;
			if ( fleet.getOwnerId( ) != null && fleet.getOwnerId( ) == this.empireId ) {
				named = namedOwn;
				unnamed = unnamedOwn;
			} else if ( fleet.getStatus( ) == true ) {
				named = namedHostile;
				unnamed = unnamedHostile;
			} else {
				named = namedFriendly;
				unnamed = unnamedFriendly;
			}
			sBuilder.append( this.fleetLine( fleet , named , unnamed ) );

			String source = "{{planet:" + fleet.getSourceId( ) + " " + fleet.getSourceName( ) + "}}";
			sBuilder.append( arrival.replace( "${source}" , source ) );
		}

		return sBuilder.toString( );
	}


	private String makeSwitchList( )
	{
		String named = this.translate( "imfNamed" );
		String unnamed = this.translate( "imfUnnamed" );
		String toAttack = this.translate( "imfSwitchAttack" );
		String toDefence = this.translate( "imfSwitchDefence" );
		StringBuilder sBuilder = new StringBuilder( );

		for ( FleetEventFleet fleet : this.event.getFleets( ) ) {
			sBuilder.append( "\n* " );
			sBuilder.append( this.fleetLine( fleet , named , unnamed ) );
			sBuilder.append( fleet.getStatus( ) ? toAttack : toDefence );
		}

		return sBuilder.toString( );
	}


	private String fleetLine( FleetEventFleet fleet , String named , String unnamed )
	{
		String tmpl;
		if ( fleet.getFleetName( ) == null ) {
			tmpl = unnamed;
		} else {
			tmpl = named.replace( "${fleet}" , fleet.getFleetName( ) );
		}
		tmpl = tmpl.replace( "${power}" , ( (Long) fleet.getFleetPower( ) ).toString( ) );

		String empLink;
		if ( fleet.getOwnerId( ) == null ) {
			empLink = fleet.getOwnerName( );
		} else {
			empLink = "{{empire:" + fleet.getOwnerId( ) + " " + fleet.getOwnerName( ) + "}}";
		}
		return tmpl.replace( "${owner}" , empLink );
	}

}
