package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;
import com.deepclone.lw.sqld.msgs.QueueEventLocation;
import com.deepclone.lw.sqld.msgs.QueueEventRecord;



@Component
public class QueueMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static abstract class QMEBase
			implements MessageExtractor
	{

		public String formatLocation( QueueEventLocation location )
		{
			return "{{planet:" + location.getLocationId( ) + " " + location.getLocationName( ) + "}} ("
					+ location.getX( ) + "," + location.getY( ) + ";" + location.getOrbit( ) + ")";
		}
	}

	private static class QueuesMessageExtractor
			extends QMEBase
	{
		private QueueEventRecord event;
		private String mTitle;
		private String mBody;
		private String mSource;


		public QueuesMessageExtractor( QueueEventRecord event , String mTitle , String mBody , String mSource )
		{
			this.event = event;
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			StringBuilder sb = new StringBuilder( ).append( this.mBody );
			for ( QueueEventLocation location : this.event.getLocations( ) ) {
				sb.append( "\n* " ).append( this.formatLocation( location ) );
			}
			return sb.toString( );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle;
		}

	}

	private static class QueueMessageExtractor
	extends QMEBase
	{
		private QueueEventRecord event;
		private String mTitle;
		private String mBody;
		private String mSource;


		public QueueMessageExtractor( QueueEventRecord event , String mTitle , String mBody , String mSource )
		{
			this.event = event;
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			QueueEventLocation location = this.event.getLocations( ).get( 0 );
			return this.mBody.replace( "${location}" , this.formatLocation( location ) );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			QueueEventLocation location = this.event.getLocations( ).get( 0 );
			return this.mTitle.replace( "${location}" , location.getLocationName( ) );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.QUEUE , 0 ) );
		fmts.add( new FormatType( EventType.QUEUE , 1 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		QueueEventRecord event = (QueueEventRecord) contents;
		boolean isCiv = ( event.getSubType( ) == 0 );
		boolean multi = ( event.getLocations( ).size( ) > 1 );
		String mTitle , mBody , mSource;
		try {
			mTitle = this.translator.translate( language , "imtEmpty" + ( isCiv ? "Civ" : "Mil" ) + "Queue"
					+ ( multi ? "s" : "" ) );
			mBody = this.translator.translate( language , "imEmpty" + ( isCiv ? "Civ" : "Mil" ) + "Queue"
					+ ( multi ? "s" : "" ) );
			mSource = this.translator.translate( language , "imSender" + ( isCiv ? "Eco" : "War" ) );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}

		if ( multi ) {
			return new QueuesMessageExtractor( event , mTitle , mBody , mSource );
		}
		return new QueueMessageExtractor( event , mTitle , mBody , mSource );
	}

}
