package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.EmpireEventRecord;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Component
public class TechMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class TechMessageExtractor
			implements MessageExtractor
	{
		private String mTitle;
		private String mBody;
		private String mSource;
		private String techName;


		public TechMessageExtractor( String mTitle , String mBody , String mSource ,
				String techName )
		{
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
			this.techName = techName;
		}


		@Override
		public String getContents( )
		{
			return this.mBody.replace( "${tech}" , this.techName );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle.replace( "${tech}" , this.techName );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.EMPIRE , 0 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		EmpireEventRecord event = (EmpireEventRecord) contents;
		String mTitle , mBody , mSource , techName;
		try {
			mTitle = this.translator.translate( language , "imtTechAvailable" );
			mBody = this.translator.translate( language , "imTechAvailable" );
			mSource = this.translator.translate( language , "imSenderSci" );
			techName = this.translator.translate( language , event.getTech( ) );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new TechMessageExtractor( mTitle , mBody , mSource , techName );
	}

}
