package com.deepclone.lw.beans.sys;


import java.util.Collection;
import java.util.Set;

import javax.sql.DataSource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.sys.ConstantDefinition;
import com.deepclone.lw.interfaces.sys.ConstantsAdministration;
import com.deepclone.lw.interfaces.sys.ConstantsManager;
import com.deepclone.lw.interfaces.sys.ConstantsUser;



/**
 * The constants manager bean initialises a {@link ConstantsData} instance, to which it delegates
 * all operations.
 * 
 * @author tseeker
 */
public class ConstantsManagerBean
		implements ConstantsManager , InitializingBean
{

	/** Transaction manager interface */
	private TransactionTemplate tTemplate;

	/** Logger bean */
	private Logger logger;

	/** Shared constants data instance */
	private ConstantsData data;

	private DataSource dataSource;


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dataSource = dataSource;
	}


	/**
	 * Sets the transaction manager interface (DI)
	 * 
	 * @param transactionManager
	 *            the transaction manager
	 */
	@Autowired( required = true )
	public void setTransactionManager( PlatformTransactionManager transactionManager )
	{
		this.tTemplate = new TransactionTemplate( transactionManager );
	}


	/**
	 * Sets the logger bean interface (DI)
	 * 
	 * @param logger
	 *            the logger bean
	 */
	@Autowired( required = true )
	public void setLogger( Logger logger )
	{
		this.logger = logger;
	}


	/** Creates the data storage and manipulation instance once the bean is ready */
	@Override
	public void afterPropertiesSet( )
	{
		this.data = new ConstantsData( this.dataSource , this.tTemplate , this.logger );
	}


	/* Documented in ConstantsManager interface */
	@Override
	public void registerConstants( Collection< ConstantDefinition > definitions )
	{
		this.data.registerConstants( definitions );
	}


	/* Documented in ConstantsManager interface */
	@Override
	public void registerUser( ConstantsUser user , Set< String > constants )
	{
		this.data.registerUser( user , constants );
	}


	/* Documented in ConstantsManager interface */
	@Override
	public void unregisterUser( ConstantsUser user )
	{
		this.data.unregisterUser( user );
	}


	/* Documented in ConstantsManager interface */
	@Override
	public ConstantsAdministration getAdminSession( int admin )
	{
		return new ConstantsAdministrationImpl( this.data , admin );
	}
}
