package com.deepclone.lw.beans.sys;


import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.sys.ConstantDefinition;
import com.deepclone.lw.interfaces.sys.ConstantsManager;



public class ConstantsRegistrarBean
{

	@Autowired( required = true )
	public void setConstantsManager( ConstantsManager cm )
	{
		List< ConstantDefinition > defs = new LinkedList< ConstantDefinition >( );
		String cDesc;

		// Default maximal age for log entries = 1 week
		double oneWeek = 60 * 60 * 24 * 7;

		// Initial values
		cDesc = "Initial cash for new empires.";
		defs.add( new ConstantDefinition( "game.initialCash" , "Initial values" , cDesc , 2000.0 , 0.0 , true ) );
		cDesc = "Initial credits for new accounts.";
		defs.add( new ConstantDefinition( "game.initialCredits" , "Initial values" , cDesc , 0.0 , 0.0 , true ) );

		// Misc. game-related values
		cDesc = "Game updates - batch size.";
		defs.add( new ConstantDefinition( "game.batchSize" , "Game (misc)" , cDesc , 20.0 , 1.0 , true ) );
		cDesc = "Event processing - batch size.";
		defs.add( new ConstantDefinition( "game.events.batchSize" , "Game (misc)" , cDesc , 100.0 , 1.0 , true ) );
		cDesc = "Population growth factor.";
		defs.add( new ConstantDefinition( "game.growthFactor" , "Game (misc)" , cDesc , 50.0 , 1.0 , true ) );
		cDesc = "Increase to the population growth factor caused by reanimation centres.";
		defs.add( new ConstantDefinition( "game.growthFactor.rCentre" , "Game (misc)" , cDesc , 10.0 , 1.0 , true ) );
		cDesc = "Time to abandon a planet (in ticks).";
		defs.add( new ConstantDefinition( "game.timeToAbandon" , "Game (misc)" , cDesc , 1440.0 , 1.0 , true ) );
		cDesc = "Fleet damage ratio per day when debt and upkeep are equal.";
		defs.add( new ConstantDefinition( "game.debt.fleet" , "Game (misc)" , cDesc , 0.3 , 0.001 , 0.999 ) );
		cDesc = "Buildings damage ratio per day when debt and upkeep are equal.";
		defs.add( new ConstantDefinition( "game.debt.buildings" , "Game (misc)" , cDesc , 0.1 , 0.001 , 0.999 ) );

		// Universe
		cDesc = "Ratio of free planets below which universe expansion is triggered.";
		defs.add( new ConstantDefinition( "game.universe.minFreeRatio" , "Universe" , cDesc , 0.1 , 0.02 , 0.98 ) );
		cDesc = "Amount of pictures available to generate planets.";
		defs.add( new ConstantDefinition( "game.universe.pictures" , "Universe" , cDesc , 200.0 , 1.0 , true ) );
		cDesc = "Initial population on first-generation planets.";
		defs.add( new ConstantDefinition( "game.universe.initialPopulation" , "Universe" , cDesc , 2000.0 , 1000.0 ,
				true ) );
		cDesc = "Initial universe size (offset relative to the centre).";
		defs.add( new ConstantDefinition( "game.universe.initialSize" , "Universe" , cDesc , 1.0 , 1.0 , true ) );

		// Natural resources
		cDesc = "Global recovery rate multiplier for natural resources. This should be kept close to 0.";
		defs.add( new ConstantDefinition( "game.resources.recovery" , "Natural resources" , cDesc , 0.01 , 0.00001 ,
				1.0 ) );
		cDesc = "Resource recovery dampener. Lower means less dampening";
		defs.add( new ConstantDefinition( "game.resources.recoveryDampening" , "Natural resources" , cDesc , 1.5 , 1.0 ,
				true ) );
		cDesc = "Resource weight base value. This value is taken to the Xth power to compute the actual "
				+ "weight when determining how mining work is distributed amongst a planet's resource providers.";
		defs.add( new ConstantDefinition( "game.resources.weightBase" , "Natural resources" , cDesc , 10.0 , 1.1 ,
				100.0 ) );
		cDesc = "Resources extracted per work unit, per (real) day, from a full provider with minimal difficulty.";
		defs.add( new ConstantDefinition( "game.resources.extraction" , "Natural resources" , cDesc , 10.0 , 1.0 , true ) );

		// Happiness
		String[] hcNames = {
				"noEmployment" , "employmentLimit" , "noDefence" , "defenceLimit" , "popPerDefencePoint" ,
				"idealEmpireSize" , "smallEmpire" , "eSizeLimit" , "strike" , "relativeChange" , "maxAbsoluteChange"
		};
		for ( int i = 0 ; i < hcNames.length ; i++ ) {
			hcNames[ i ] = "game.happiness." + hcNames[ i ];
		}
		String cat = "Happiness";
		cDesc = "Employment happiness level when there is no employment.";
		defs.add( new ConstantDefinition( hcNames[ 0 ] , cat , cDesc , 0.7 , 0.05 , 0.99 ) );
		cDesc = "Employment ratio beyond which things get tough.";
		defs.add( new ConstantDefinition( hcNames[ 1 ] , cat , cDesc , 2.0 , 1.1 , true ) );
		cDesc = "Defence happiness level when there is no defence.";
		defs.add( new ConstantDefinition( hcNames[ 2 ] , cat , cDesc , 0.5 , 0.05 , 0.99 ) );
		cDesc = "Defence ratio beyond which things get tough.";
		defs.add( new ConstantDefinition( hcNames[ 3 ] , cat , cDesc , 4.0 , 1.1 , true ) );
		cDesc = "Population covered by a single defence power point.";
		defs.add( new ConstantDefinition( hcNames[ 4 ] , cat , cDesc , 5.0 , 1.0 , true ) );
		cDesc = "Ideal empire size.";
		defs.add( new ConstantDefinition( hcNames[ 5 ] , cat , cDesc , 10.0 , 2.0 , true ) );
		cDesc = "Happiness level for 1-planet empires.";
		defs.add( new ConstantDefinition( hcNames[ 6 ] , cat , cDesc , 0.9 , 0.5 , true ) );
		cDesc = "Empire size limit (relative to the ideal size) beyond which things get tough.";
		defs.add( new ConstantDefinition( hcNames[ 7 ] , cat , cDesc , 2.0 , 1.1 , true ) );
		cDesc = "Happiness level below which strikes begin.";
		defs.add( new ConstantDefinition( hcNames[ 8 ] , cat , cDesc , 0.5 , 0.0 , 1.0 ) );
		cDesc = "Happiness change at each update, relative to the size of the population.";
		defs.add( new ConstantDefinition( hcNames[ 9 ] , cat , cDesc , 0.001 , 0.00001 , 0.99999 ) );
		cDesc = "Maximal population units for which happiness will change.";
		defs.add( new ConstantDefinition( hcNames[ 10 ] , cat , cDesc , 1000.0 , 1.0 , true ) );

		// Work and income
		String[] wcNames = {
				"population" , "factory" , "strikeEffect" , "wuPerPopUnit" , "destructionRecovery" , "destructionWork" ,
				"cancelRecovery"
		};
		for ( int i = 0 ; i < wcNames.length ; i++ ) {
			wcNames[ i ] = "game.work." + wcNames[ i ];
		}
		cat = "Work & income";
		cDesc = "Daily income per population unit.";
		defs.add( new ConstantDefinition( wcNames[ 0 ] , cat , cDesc , 0.2 , 0.01 , true ) );
		cDesc = "Daily income per factory production unit.";
		defs.add( new ConstantDefinition( wcNames[ 1 ] , cat , cDesc , 250.0 , 0.01 , true ) );
		cDesc = "Proportion of the base income that disappears if the whole population is on strike.";
		defs.add( new ConstantDefinition( wcNames[ 2 ] , cat , cDesc , 1.0 , 0.0 , 1.0 ) );
		cDesc = "Work units generated by each population unit.";
		defs.add( new ConstantDefinition( wcNames[ 3 ] , cat , cDesc , 0.01 , 0.001 , true ) );
		cDesc = "Proportion of a building's cost that is recovered when it is destroyed.";
		defs.add( new ConstantDefinition( wcNames[ 4 ] , cat , cDesc , 0.1 , 0.01 , 0.99 ) );
		cDesc = "Proportion of a building's construction work units required to destroy it";
		defs.add( new ConstantDefinition( wcNames[ 5 ] , cat , cDesc , 0.25 , 0.01 , 1.0 ) );
		cDesc = "Proportion of queue investments that is recovered when flushing the queue.";
		defs.add( new ConstantDefinition( wcNames[ 6 ] , cat , cDesc , 0.1 , 0.01 , 1.0 ) );

		// Research
		String[] rcNames = {
				"basePoints" , "visibility.points" , "visibility.ratio" , "weightBase" , "vacation"
		};
		for ( int i = 0 ; i < rcNames.length ; i++ ) {
			rcNames[ i ] = "game.research." + rcNames[ i ];
		}
		cat = "Research & technologies";
		cDesc = "Research points per population unit.";
		defs.add( new ConstantDefinition( rcNames[ 0 ] , cat , cDesc , 0.50 , 0.01 , true ) );
		cDesc = "Points above which a technology becomes visible.";
		defs.add( new ConstantDefinition( rcNames[ 1 ] , cat , cDesc , 2500.0 , 0.01 , true ) );
		cDesc = "Completion ratio above which a technology becomes visible.";
		defs.add( new ConstantDefinition( rcNames[ 2 ] , cat , cDesc , 0.10 , 0.01 , 0.99 ) );
		cDesc = "Technology weight base value. This value is taken to the Xth power (where X is a priority) "
				+ "to compute the actual weight when determining how research points are distributed between "
				+ "an empire's in-progress research.";
		defs.add( new ConstantDefinition( rcNames[ 3 ] , cat , cDesc , 10.0 , 1.0 , true ) );
		cDesc = "Research points multiplier used when the player is on vacation.";
		defs.add( new ConstantDefinition( rcNames[ 4 ] , cat , cDesc , 0.1 , 0.01 , 0.99 ) );

		// Vacation mode
		cDesc = "Initial vacation credits.";
		defs.add( new ConstantDefinition( "vacation.initial" , "Vacation mode" , cDesc , 4320.0 , 0.0 , true ) );
		cDesc = "Delay before vacation mode is activated (seconds).";
		defs.add( new ConstantDefinition( "vacation.delay" , "Vacation mode" , cDesc , 21600.0 , 3600.0 , true ) );
		cDesc = "Maximal vacation credits.";
		defs.add( new ConstantDefinition( "vacation.max" , "Vacation mode" , cDesc , 259200.0 , 10000.0 , true ) );
		cDesc = "Vacation cost (credits / minute).";
		defs.add( new ConstantDefinition( "vacation.cost" , "Vacation mode" , cDesc , 3.0 , 2.0 , 20.0 ) );
		cDesc = "Income/upkeep divider used when vacation mode is active.";
		defs.add( new ConstantDefinition( "vacation.cashDivider" , "Vacation mode" , cDesc , 3.0 , 1.1 , 10.0 ) );

		// Map names
		cDesc = "Minimal delay between map object renaming.";
		defs.add( new ConstantDefinition( "map.names.minDelay" , "Map names" , cDesc , 15.0 , 1.0 , true ) );
		cDesc = "Units for the minimal delay between map object renaming (seconds).";
		defs.add( new ConstantDefinition( "map.names.minDelay.units" , "Map names" , cDesc , 86400.0 , 1.0 , true ) );

		// Log clean-up
		cDesc = "Maximal age of administration log entries, in seconds.";
		defs.add( new ConstantDefinition( "log.maxAge.admin" , "Logs" , cDesc , oneWeek , 1.0 , true ) );
		cDesc = "Maximal age of account log entries, in seconds.";
		defs.add( new ConstantDefinition( "log.maxAge.users" , "Logs" , cDesc , oneWeek , 1.0 , true ) );
		cDesc = "Maximal age of system log entries, in seconds.";
		defs.add( new ConstantDefinition( "log.maxAge.sys" , "Logs" , cDesc , oneWeek , 1.0 , true ) );

		// Battles
		cDesc = "Amount of ticks before a battle reaches full intensity.";
		defs.add( new ConstantDefinition( "game.battle.timeToFullIntensity" , "Battles" , cDesc , 30.0 , 0.0 , true ) );
		cDesc = "Initial intensity of a battle.";
		defs.add( new ConstantDefinition( "game.battle.initialIntensity" , "Battles" , cDesc , 0.25 , 0.0 , 1.0 ) );
		cDesc = "Defence bonus.";
		defs.add( new ConstantDefinition( "game.battle.defenceBonus" , "Battles" , cDesc , 0.1 , 0.0 , true ) );
		cDesc = "Damage / power unit / minute";
		defs.add( new ConstantDefinition( "game.battle.damage" , "Battles" , cDesc , 0.01 , 0.001 , true ) );
		cDesc = "Proportion of random damage variation.";
		defs.add( new ConstantDefinition( "game.battle.randomDamage" , "Battles" , cDesc , 0.05 , 0.0 , 0.9 ) );

		// Ticker
		cDesc = "Interval between ticks with the highest frequency, in milliseconds.";
		defs.add( new ConstantDefinition( "ticker.interval" , "Ticker" , cDesc , 5000.0 , 1000.0 , true ) );

		// Accounts
		cDesc = "Minimal interval between address change requests (seconds)";
		defs.add( new ConstantDefinition( "accounts.acrDelay" , "Accounts" , cDesc , 14400.0 , 1.0 , true ) );
		cDesc = "Minimal interval between password recovery requests (seconds)";
		defs.add( new ConstantDefinition( "accounts.prrDelay" , "Accounts" , cDesc , 8800.0 , 1.0 , true ) );
		cDesc = "Time before a new or reactivated account is deleted / disabled (seconds)";
		defs.add( new ConstantDefinition( "accounts.cacDelay" , "Accounts" , cDesc , 86400.0 , 3600.0 , true ) );
		cDesc = "Delay between clicking the 'Quit' button and account de-activation (seconds)";
		defs.add( new ConstantDefinition( "accounts.quitDelay" , "Accounts" , cDesc , 86400.0 , 3600.0 , true ) );
		cDesc = "Delay between a ban and the deletion of the player's empire (seconds)";
		defs.add( new ConstantDefinition( "accounts.banDelay" , "Accounts" , cDesc , 178000.0 , 3600.0 , true ) );
		cDesc = "Delay before a ban request expires (seconds)";
		defs.add( new ConstantDefinition( "accounts.banExpiration" , "Accounts" , cDesc , oneWeek , 3600.0 , true ) );

		// Accounts - warnings
		cDesc = "Amount of warnings that triggers an automatic ban request.";
		defs.add( new ConstantDefinition( "accounts.warnings.autoBan" , "Accounts - Warnings" , cDesc , 3.0 , 1.0 ,
				true ) );
		cDesc = "Period after a warning is received during which additional warnings will be ignored (seconds).";
		defs.add( new ConstantDefinition( "accounts.warnings.grace" , "Accounts - Warnings" , cDesc , 7200.0 , 60.0 ,
				true ) );
		cDesc = "Time after which warnings are decreased (expressed in units as defined by a.w.expiration.units).";
		defs.add( new ConstantDefinition( "accounts.warnings.expiration" , "Accounts - Warnings" , cDesc , 60.0 , 1.0 ,
				true ) );
		cDesc = "Units used to express warning expiration time (seconds).";
		defs.add( new ConstantDefinition( "accounts.warnings.expiration.units" , "Accounts - Warnings" , cDesc ,
				86400.0 , 1.0 , true ) );

		// Account inactivity
		cDesc = "Time units (seconds)";
		defs.add( new ConstantDefinition( "accounts.inactivity.units" , "Accounts - Inactivity" , cDesc , oneWeek ,
				3600.0 , true ) );
		cDesc = "Time after which the inactivity warning e-mail is to be sent, expressed using units defined by a.i.units.";
		defs.add( new ConstantDefinition( "accounts.inactivity.warningMail" , "Accounts - Inactivity" , cDesc , 3.0 ,
				1.0 , true ) );
		cDesc = "Time between the inactivity warning e-mail and actual account deletion, expressed using units defined by a.i.units.";
		defs.add( new ConstantDefinition( "accounts.inactivity.deletion" , "Accounts - Inactivity" , cDesc , 1.0 , 1.0 ,
				true ) );

		// Bug reports
		cDesc = "Amount of credits granted for low priority bug reports.";
		defs.add( new ConstantDefinition( "bugtracker.lowCredits" , "Bug tracking system" , cDesc , 1.0 , 1.0 , true ) );
		cDesc = "Amount of credits granted for normal bug reports.";
		defs.add( new ConstantDefinition( "bugtracker.mediumCredits" , "Bug tracking system" , cDesc , 2.0 , 1.0 , true ) );
		cDesc = "Amount of credits granted for critical bug reports.";
		defs.add( new ConstantDefinition( "bugtracker.highCredits" , "Bug tracking system" , cDesc , 3.0 , 1.0 , true ) );

		cm.registerConstants( defs );
	}

}
