package com.deepclone.lw.beans.updates;


import com.deepclone.lw.interfaces.game.updates.UpdateBatchProcessor;



/**
 * Server-side update processor registry
 * 
 * <p>
 * This interface allows server-side game update batch processors to be looked up depending on the
 * type of the updates to process.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
interface ServerProcessorRegistry
{

	/**
	 * Get a batch processor for some type of update
	 * 
	 * @param type
	 *            the type of updates to process, as stored in the <code>sys.update_types</code>
	 *            database table
	 * 
	 * @return <code>null</code> if no processor for the specified type has been registered, or the
	 *         processor to use.
	 */
	public UpdateBatchProcessor getProcessorFor( String type );

}
