package com.deepclone.lw.beans.updates;


import java.sql.Types;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.game.updates.UpdatesDAO;
import com.deepclone.lw.sqld.sys.GameUpdateResult;
import com.deepclone.lw.utils.StoredProc;



/**
 * Updates data access component
 * 
 * <p>
 * This component accesses the database procedures and views which constitute the game update
 * system.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class UpdatesDAOBean
		implements UpdatesDAO
{

	/** Game update processor stored procedure */
	private StoredProc process;


	/**
	 * Dependency injector for the data source
	 * 
	 * <p>
	 * When Spring injects the data source component, initialise the stored procedures used by the
	 * component.
	 * 
	 * @param dataSource
	 *            the data source component
	 */
	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.process = new StoredProc( dataSource , "sys" , "process_updates" ).addParameter( "tick_id" , Types.BIGINT )
				.addOutput( "_has_more" , Types.BOOLEAN ).addOutput( "_process_externally" , Types.VARCHAR );
	}


	/**
	 * Execute the processor's stored procedure
	 * 
	 * <p>
	 * This implementation simply runs the database's <code>sys.process_updates()</code> stored
	 * procedure, and converts the two values it returns into a {@link GameUpdateResult} instance.
	 */
	@Override
	public GameUpdateResult processUpdates( long tickId )
	{
		Map< String , Object > m = this.process.execute( tickId );
		if ( !(Boolean) m.get( "_has_more" ) ) {
			return new GameUpdateResult( );
		}
		return new GameUpdateResult( (String) m.get( "_process_externally" ) );
	}

}
