package com.deepclone.lw.beans.user.admin.main.i18n;


import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.i18n.GetLanguageCommand;
import com.deepclone.lw.cmd.admin.i18n.GetLanguageResponse;
import com.deepclone.lw.cmd.admin.i18n.I18NString;
import com.deepclone.lw.cmd.admin.i18n.Language;
import com.deepclone.lw.interfaces.i18n.I18NAdministration;
import com.deepclone.lw.interfaces.i18n.I18NManager;
import com.deepclone.lw.interfaces.i18n.UnknownLanguageException;
import com.deepclone.lw.interfaces.i18n.UnknownStringException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class GetLanguageCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{

	private I18NManager i18n;


	@Autowired( required = true )
	public void setI18n( I18NManager i18n )
	{
		this.i18n = i18n;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return GetLanguageCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.I18N ) ) {
			return new GetLanguageResponse( admin );
		}

		String lId = ( (GetLanguageCommand) cParam ).getLanguage( );
		I18NAdministration i18n = this.i18n.getAdminSession( admin.getId( ) );

		Language language = new Language( );
		List< I18NString > strings = new LinkedList< I18NString >( );
		language.setId( lId );
		try {
			language.setName( i18n.getLanguageName( lId ) );
			language.setCompletion( (int) Math.floor( i18n.getLanguageSupport( lId ) * 100 ) );
			for ( String sId : i18n.getStrings( ) ) {
				strings.add( new I18NString( sId , i18n.getTranslation( lId , sId ) ) );
			}
		} catch ( UnknownLanguageException e ) {
			return new GetLanguageResponse( admin , null , null );
		} catch ( UnknownStringException e ) {
			throw new RuntimeException( e );
		}

		return new GetLanguageResponse( admin , language , strings );
	}
}
