package com.deepclone.lw.beans.user.admin.main.i18n;


import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.i18n.Language;
import com.deepclone.lw.cmd.admin.i18n.ViewLanguagesCommand;
import com.deepclone.lw.cmd.admin.i18n.ViewLanguagesResponse;
import com.deepclone.lw.interfaces.i18n.I18NAdministration;
import com.deepclone.lw.interfaces.i18n.I18NManager;
import com.deepclone.lw.interfaces.i18n.UnknownLanguageException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class ViewLanguagesCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{

	private I18NManager i18n;


	@Autowired( required = true )
	public void setI18n( I18NManager i18n )
	{
		this.i18n = i18n;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return ViewLanguagesCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.I18N ) ) {
			return new ViewLanguagesResponse( admin );
		}

		I18NAdministration i18n = this.i18n.getAdminSession( admin.getId( ) );
		List< Language > languages = new LinkedList< Language >( );
		for ( String lang : i18n.getLanguages( ) ) {
			Language language = new Language( );
			language.setId( lang );
			try {
				language.setName( i18n.getLanguageName( lang ) );
				language.setCompletion( (int) Math.floor( i18n.getLanguageSupport( lang ) * 100 ) );
			} catch ( UnknownLanguageException e ) {
				throw new RuntimeException( e );
			}
			languages.add( language );
		}

		return new ViewLanguagesResponse( admin , languages );
	}

}
