package com.deepclone.lw.beans.user.admin.main.mntm;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.mntm.EnableMaintenanceCommand;
import com.deepclone.lw.cmd.admin.mntm.MaintenanceChangeResponse;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.sys.MaintenanceData;
import com.deepclone.lw.interfaces.sys.MaintenanceStatusException;
import com.deepclone.lw.interfaces.sys.SystemStatus;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class EnableMaintenanceCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{

	private SystemStatus systemStatus;


	@Autowired( required = true )
	public void setSystemStatus( SystemStatus systemStatus )
	{
		this.systemStatus = systemStatus;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return EnableMaintenanceCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.MNTM ) ) {
			return new MaintenanceChangeResponse( admin , true );
		}

		String reason = ( (EnableMaintenanceCommand) command ).getReason( );
		ObjectNameError one;
		if ( "".equals( reason ) ) {
			one = ObjectNameError.EMPTY;
		} else if ( reason.length( ) < 10 ) {
			one = ObjectNameError.INVALID;
		} else {
			one = null;
		}

		int duration = ( (EnableMaintenanceCommand) command ).getDuration( );
		if ( duration < 1 || one != null ) {
			MaintenanceData data = this.systemStatus.checkMaintenance( );
			if ( data == null ) {
				data = new MaintenanceData( null , null , null );
			}
			return new MaintenanceChangeResponse( admin , data.reason , data.start , data.end , reason , one , duration );
		}

		try {
			this.systemStatus.startMaintenance( admin.getId( ) , reason , duration );
		} catch ( MaintenanceStatusException e ) {
			// Do nothing
		}
		return new MaintenanceChangeResponse( admin , false );
	}

}
