package com.deepclone.lw.beans.user.admin.main.names;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.naming.NameAction;
import com.deepclone.lw.cmd.admin.naming.NameType;
import com.deepclone.lw.cmd.admin.naming.NamesActionCommand;
import com.deepclone.lw.interfaces.naming.NamesManager;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.NullResponse;



public class NamesActionCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{
	private NamesManager namesManager;


	@Autowired( required = true )
	public void setNamesManager( NamesManager namesManager )
	{
		this.namesManager = namesManager;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return NamesActionCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.NAME ) ) {
			return new NullResponse( );
		}

		NamesActionCommand command = (NamesActionCommand) cParam;
		NameType type = command.getType( );
		NameAction action = command.getAction( );
		int[] ids = command.getIdentifiers( );
		if ( ids.length == 0 ) {
			return new NullResponse( );
		}

		switch ( action ) {
			case VALIDATE:
				if ( type != NameType.ALLIANCE && type != NameType.EMPIRE ) {
					this.namesManager.validateMapNames( admin , ids );
				}
				break;
			case REJECT:
			case REJECT_BAN:
				if ( type != NameType.ALLIANCE && type != NameType.EMPIRE ) {
					this.namesManager.rejectMapNames( admin , ids , action == NameAction.REJECT_BAN );
				} else if ( type == NameType.ALLIANCE ) {
					this.namesManager.rejectAllianceNames( admin , ids );
				} else if ( type == NameType.EMPIRE ) {
					this.namesManager.rejectEmpireNames( admin , ids , action == NameAction.REJECT_BAN );
				}
				break;
			case RESET:
				if ( type == NameType.MAP_CHANGED || type == NameType.MAP_VALIDATED ) {
					this.namesManager.allowMapNameChanges( admin , ids );
				}
				break;
		}

		return new NullResponse( );
	}
}
