package com.deepclone.lw.beans.user.admin.main.su;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.su.AddAdministratorCommand;
import com.deepclone.lw.cmd.admin.su.AddAdministratorResponse;
import com.deepclone.lw.cmd.admin.su.AddAdministratorResponse.AddressError;
import com.deepclone.lw.interfaces.admin.Administration;
import com.deepclone.lw.interfaces.naming.ObjectNameValidator;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;



public class AddAdministratorCommandDelegateBean
		extends SuperUserOperation
{

	private Administration administration;
	private ObjectNameValidator validator;


	@Autowired( required = true )
	public void setAdministration( Administration administration )
	{
		this.administration = administration;
	}


	@Autowired( required = true )
	public void setValidator( ObjectNameValidator validator )
	{
		this.validator = validator;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return AddAdministratorCommand.class;
	}


	@Override
	protected CommandResponse handleCommand( Administrator admin , Command cParam )
	{
		AddAdministratorCommand command = (AddAdministratorCommand) cParam;

		// Check name
		String name = command.getAppearAs( );
		ObjectNameError nameError;
		if ( "".equals( name ) ) {
			nameError = ObjectNameError.EMPTY;
		} else {
			nameError = this.validator.validate( name , 2 , 64 );
		}

		// Check address
		String aStr = command.getAddress( );
		AddressError addrError;
		if ( "".equals( aStr ) ) {
			addrError = AddressError.EMPTY;
		} else {
			EmailAddress address = new EmailAddress( aStr );
			if ( address.isValid( ) ) {
				addrError = null;
			} else {
				addrError = AddressError.INVALID;
			}
			aStr = address.getAddress( );
		}
		
		// Check privileges
		int privileges = 0;
		for ( Privileges priv : command.getPrivileges( )) {
			privileges = priv.grant( privileges );
		}
		boolean privError = ( privileges == 0 );
		
		// If we already have errors, return
		if ( addrError != null || nameError != null || privError ) {
			return new AddAdministratorResponse( admin , addrError , aStr , nameError , name , privError , privileges );
		}

		return this.administration.createAdmin( admin , aStr , name , privileges );
	}


	@Override
	protected CommandResponse insufficientPrivileges( Administrator admin )
	{
		return new AddAdministratorResponse( admin , true );
	}

}
