package com.deepclone.lw.beans.user.ext;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.cmd.ext.ConfirmPasswordRecoveryCommand;
import com.deepclone.lw.cmd.ext.ConfirmPasswordRecoveryResponse;
import com.deepclone.lw.cmd.ext.RequestPasswordRecoveryResponse;
import com.deepclone.lw.cmd.ext.ConfirmPasswordRecoveryResponse.PasswordRecoveryStatus;
import com.deepclone.lw.cmd.ext.RequestPasswordRecoveryResponse.PasswordRecoveryRequestStatus;
import com.deepclone.lw.interfaces.acm.AccountMailException;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.acm.PasswordProhibitedException;
import com.deepclone.lw.interfaces.acm.PasswordRecoveryException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;



public class ConfPwdRecoveryCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement accountManagement;
	private int minPasswordStrength = 10;


	@Autowired( required = true )
	public void setAccountManagement( AccountManagement accountManagement )
	{
		this.accountManagement = accountManagement;
	}


	public void setMinPasswordStrength( int strength )
	{
		this.minPasswordStrength = strength;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return ExternalSessionDefinerBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return ConfirmPasswordRecoveryCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		ConfirmPasswordRecoveryCommand command = (ConfirmPasswordRecoveryCommand) cParam;
		EmailAddress address = new EmailAddress( command.getMailAddress( ) );
		if ( !address.isValid( ) ) {
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.INVALID_MAIL );
		}
		if ( !command.getPassword( ).equals( command.getPasswordConfirm( ) ) ) {
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.MISMATCH_PASSWORD );
		}

		Password pwd = new Password( command.getPassword( ) );
		if ( pwd.getStrength( ) < this.minPasswordStrength ) {
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.WEAK_PASSWORD );
		}

		String token = command.getToken( );
		try {
			this.accountManagement.recoverPassword( address , token , pwd );
		} catch ( AccountMailException e ) {
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.NOT_FOUND );
		} catch ( PasswordRecoveryException e ) {
			if ( e.statusProblem ) {
				return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.ACCOUNT_STATUS );
			}
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.NOT_FOUND );
		} catch ( PasswordProhibitedException e ) {
			return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.PROHIBITED );
		}

		return new ConfirmPasswordRecoveryResponse( PasswordRecoveryStatus.OK );
	}

}
