package com.deepclone.lw.beans.user.player;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.account.AccountValidationCommand;
import com.deepclone.lw.cmd.player.account.AccountValidationResponse;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.naming.ObjectNameValidator;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.sqld.accounts.ValidationResult;
import com.deepclone.lw.utils.EmailAddress;



public class ValidationCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement manager;
	private ObjectNameValidator validator;


	@Autowired( required = true )
	public void setAccountManager( AccountManagement manager )
	{
		this.manager = manager;
	}


	@Autowired( required = true )
	public void setObjectNameValidator( ObjectNameValidator validator )
	{
		this.validator = validator;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return ValidationSubTypeBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return AccountValidationCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		AccountValidationCommand command = (AccountValidationCommand) cParam;
		EmailAddress address = new EmailAddress( session.get( "authenticationToken" , String.class ) );

		if ( command.isInitialisation( ) ) {
			return new AccountValidationResponse( this.manager.getEmpireNames( address ) );
		}

		AccountValidationResponse vResponse;

		// Validate empire / planet names
		vResponse = this.validateNames( command , address );
		if ( vResponse != null ) {
			return vResponse;
		}

		// Actual validation / game registration
		ValidationResult result;
		result = this.manager.validateAccount( address , command.getToken( ) , command.getEmpire( ) , command
				.getPlanet( ) );

		if ( result.isError( ) ) {
			return this.makeError( command , address , result );
		}

		return new AccountValidationResponse( );
	}


	private AccountValidationResponse validateNames( AccountValidationCommand command , EmailAddress address )
	{
		ObjectNameError eOne = this.validator.validate( command.getEmpire( ) );
		ObjectNameError pOne = this.validator.validate( command.getPlanet( ) );
		if ( eOne == null && pOne == null ) {
			return null;
		}
		return this.makeError( command , address , false , eOne , pOne );
	}


	private CommandResponse makeError( AccountValidationCommand command , EmailAddress address , ValidationResult result )
	{
		ObjectNameError eOne = null , pOne = null;

		switch ( result.getEmpireError( ) ) {
			case 1:
				eOne = ObjectNameError.BANNED;
				break;
			case 2:
				eOne = ObjectNameError.UNAVAILABLE;
				break;
		}

		switch ( result.getPlanetError( ) ) {
			case 1:
				pOne = ObjectNameError.BANNED;
				break;
			case 2:
				pOne = ObjectNameError.UNAVAILABLE;
				break;
		}

		return this.makeError( command , address , ( result.getAccountError( ) != 0 ) , eOne , pOne );
	}


	private AccountValidationResponse makeError( AccountValidationCommand command , EmailAddress address ,
			boolean wrongToken , ObjectNameError empireError , ObjectNameError planetError )
	{
		return new AccountValidationResponse( this.manager.getEmpireNames( address ) , command.getToken( ) , command
				.getEmpire( ) , command.getPlanet( ) , wrongToken , empireError , planetError );
	}

}
