package com.deepclone.lw.beans.user.player.game;


import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.ViewMapCommand;
import com.deepclone.lw.cmd.player.ViewMapResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.MapSize;
import com.deepclone.lw.cmd.player.gdata.map.MapSystemData;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.game.MapViewParameters;
import com.deepclone.lw.interfaces.game.MapViewer;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;



public class ViewMapCommandDelegateBean
		implements AutowiredCommandDelegate

{

	private AccountManagement accountManagement;
	private EmpireManagement empireManagement;
	private MapViewer mapViewer;
	private Translator translator;


	@Autowired( required = true )
	public void setAccountManagement( AccountManagement accountManagement )
	{
		this.accountManagement = accountManagement;
	}


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement manager )
	{
		this.empireManagement = manager;
	}


	@Autowired( required = true )
	public void setMapViewer( MapViewer mapViewer )
	{
		this.mapViewer = mapViewer;
	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return ViewMapCommand.class;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		ViewMapCommand command = (ViewMapCommand) cParam;
		EmailAddress address = new EmailAddress( session.get( "authenticationToken" , String.class ) );
		int empireId = session.get( "empireId" , Integer.class );

		boolean useDefaults = command.isDefaults( );
		if ( !useDefaults ) {
			useDefaults = ( command.getX( ) == null || command.getY( ) == null || command.getSize( ) == null );
		}

		MapViewParameters params;
		if ( useDefaults ) {
			params = this.mapViewer.getDefaults( address );
		} else {
			params = new MapViewParameters( command.getX( ) , command.getY( ) , command.getSize( ) );
		}

		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		List< String > sizeText = this.getSizeText( address );
		MapSystemData[][] systems = this.mapViewer.getMapView( empireId , params );
		return new ViewMapResponse( page , params.x , params.y , params.size , sizeText , systems );
	}


	private List< String > getSizeText( EmailAddress address )
	{
		String language = this.accountManagement.getAccount( address ).getLanguage( );
		Map< MapSize , String > sizes = MapSize.getValues( );

		List< String > rv = new ArrayList< String >( sizes.size( ) );
		try {
			for ( MapSize sz : MapSize.values( ) ) {
				rv.add( sz.ordinal( ) , this.translator.translate( language , sizes.get( sz ) ) );
			}
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return rv;
	}
}
