package com.deepclone.lw.beans.user.player.game.planets;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.planets.ToggleMiningSettingsCommand;
import com.deepclone.lw.cmd.player.planets.ViewPlanetCommand;
import com.deepclone.lw.interfaces.game.resources.ResourcesController;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.NullResponse;



/**
 * Command handler for {@link ToggleMiningSettingsCommand}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class ToggleMiningSettingsCommandDelegateBean
		implements AutowiredCommandDelegate
{

	/** The resources controller */
	private ResourcesController resourcesController;


	/**
	 * Dependency injector that sets the resources controller
	 * 
	 * @param resourcesController
	 *            the resources controller
	 */
	@Autowired( required = true )
	public void setResourcesController( ResourcesController resourcesController )
	{
		this.resourcesController = resourcesController;
	}


	/** This class handles {@link ToggleMiningSettingsCommand} instances */
	@Override
	public Class< ? extends Command > getType( )
	{
		return ToggleMiningSettingsCommand.class;
	}


	/** This class is enabled for the {@link GameSubTypeBean} session type */
	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	/** If the account is not in vacation mode, try to toggle the planet's settings */
	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		if ( !session.get( "vacation" , Boolean.class ) ) {
			int empireId = session.get( "empireId" , Integer.class );
			ViewPlanetCommand planet = (ViewPlanetCommand) command;
			this.resourcesController.togglePlanet( empireId , planet.getId( ) );
		}
		return new NullResponse( );
	}

}
