package com.deepclone.lw.beans.user.player.game.tech;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.tech.GetResearchCommand;
import com.deepclone.lw.interfaces.game.technologies.ResearchController;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



/**
 * Command delegate for {@link GetResearchCommand}
 * 
 * <p>
 * This command delegate uses the {@link ResearchController} to obtain all information pertaining to
 * an empire's research.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class GetResearchCommandDelegateBean
		implements AutowiredCommandDelegate
{
	/** The research controller */
	private ResearchController researchController;


	/**
	 * Dependency injector that sets the research controller
	 * 
	 * @param researchController
	 *            the research controller
	 */
	@Autowired
	public void setResearchController( ResearchController researchController )
	{
		this.researchController = researchController;
	}


	/** This delegate handles {@link GetResearchCommand} instances */
	@Override
	public Class< ? extends Command > getType( )
	{
		return GetResearchCommand.class;
	}


	/** This delegate is part of the game session sub-type */
	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	/** When the command is executed, fetch relevant data from the research controller */
	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		int empire = session.get( "empireId" , Integer.class );
		return this.researchController.getResearchData( empire );
	}

}
