package com.deepclone.lw.beans.user.player.game.tech;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.tech.ImplementTechCommand;
import com.deepclone.lw.interfaces.game.technologies.ResearchController;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.NullResponse;



public class ImplementTechCommandDelegateBean
		implements AutowiredCommandDelegate

{
	/** The research controller */
	private ResearchController researchController;


	/**
	 * Dependency injector that sets the research controller
	 * 
	 * @param researchController
	 *            the research controller
	 */
	@Autowired( required = true )
	public void setResearchController( ResearchController researchController )
	{
		this.researchController = researchController;
	}


	/** This class handles {@link ImplementTechCommand} instances */
	@Override
	public Class< ? extends Command > getType( )
	{
		return ImplementTechCommand.class;
	}


	/** This class is enabled for the {@link GameSubTypeBean} session type */
	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	/**
	 * Implement a technology
	 * 
	 * <p>
	 * If the empire is not in vacation mode, try to implement the technology. Always return a
	 * {@link NullResponse}.
	 */
	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		ImplementTechCommand command = (ImplementTechCommand) cParam;
		int empire = session.get( "empireId" , Integer.class );
		if ( !session.get( "vacation" , Boolean.class ) ) {
			this.researchController.implement( empire , command.getTech( ) );
		}
		return new NullResponse( );
	}
}
