package com.deepclone.lw.beans.user.player.game.tech;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.tech.UpdateResearchPrioritiesCommand;
import com.deepclone.lw.interfaces.game.technologies.ResearchController;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.NullResponse;



/**
 * Command handler for research priority updates
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class UpdateResearchPrioritiesCommandDelegateBean
		implements AutowiredCommandDelegate
{
	/** The research controller */
	private ResearchController researchController;


	/**
	 * Dependency injector that sets the research controller
	 * 
	 * @param researchController
	 *            the research controller
	 */
	@Autowired( required = true )
	public void setResearchController( ResearchController researchController )
	{
		this.researchController = researchController;
	}


	/** This class handles {@link UpdateResearchPrioritiesCommand} instances */
	@Override
	public Class< ? extends Command > getType( )
	{
		return UpdateResearchPrioritiesCommand.class;
	}


	/** This class is enabled for the {@link GameSubTypeBean} session type */
	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	/**
	 * Update research priorities
	 * 
	 * <p>
	 * If the empire is not in vacation mode, access the research controller and update mining
	 * priorities using the specified values. Return a {@link NullResponse} in all cases.
	 */
	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		if ( !session.get( "vacation" , Boolean.class ) ) {
			int empireId = session.get( "empireId" , Integer.class );
			this.researchController.updatePriorities( empireId ,
					( (UpdateResearchPrioritiesCommand) command ).getPriorities( ) );
		}
		return new NullResponse( );
	}

}
