-- LegacyWorlds Beta 6
-- PostgreSQL database scripts
--
-- Type definitions
--
-- Copyright(C) 2004-2010, DeepClone Development
-- --------------------------------------------------------

-- Processing status
CREATE TYPE processing_status
	AS ENUM ( 'FUTURE', 'PROCESSING', 'PROCESSED' );
	
-- Building output types
CREATE TYPE building_output_type
	AS ENUM ( 'CASH', 'POP', 'DEF', 'WORK' , 'MINE' );

-- Fleet status
CREATE TYPE fleet_status
	AS ENUM ( 'AVAILABLE', 'DEPLOYING' , 'REDEPLOYING' , 'REDIRECTING' );

-- Log levels
CREATE TYPE log_level
	AS ENUM ( 'TRACE' , 'DEBUG' , 'INFO' , 'WARNING' , 'ERROR' );

-- Log types
CREATE TYPE log_type
	AS ENUM ( 'SYS' , 'USERS' , 'ADMIN' );

-- Update types
CREATE TYPE update_type	AS ENUM (

	/* Empires' money is being updated using the previous update's results
	 * 
	 * FIXME: this update type should disappear, as everything it does will
	 * be replaced by the resources update.
	 */
	'EMPIRE_MONEY' ,

	/* Empires' resources are being updated using the previous update's
	 * results.
	 */
	'EMPIRE_RESOURCES' ,

	/* Empire research points are being attributed to technologies */
	'EMPIRE_RESEARCH' ,
	
	/* The effects of empires' debts are being computed */ 
	'EMPIRE_DEBT' ,
	
	/* Fleets which were 1 update away are arriving at their destination */
	'PLANET_FLEET_ARRIVALS' ,
	
	/* Other fleets are moving */
	'PLANET_FLEET_MOVEMENTS' ,
	
	/* Fleet states (e.g. "deploying", "unavailable", etc...) are being
	 * updated.
	 */
	'PLANET_FLEET_STATUS' ,
	
	/* Start new battles where necessary */
	'PLANET_BATTLE_START' ,
	
	/* Process all in-progress battles */
	'PLANET_BATTLE_MAIN' ,
	
	/* Finalise battles that need to be ended */
	'PLANET_BATTLE_END' ,
	
	/* Abandon planets */
	'PLANET_ABANDON' ,
	
	/* Handle civilian build queues */
	'PLANET_CONSTRUCTION' ,
	
	/* Handle military build queues */
	'PLANET_MILITARY' ,
	
	/* Update planets' population */
	'PLANET_POPULATION' ,
	
	/* Regenerate resources in resource providers */
	'PLANET_RES_REGEN' ,

	/* Compute income and upkeep of planets */
	'PLANET_MONEY' ,
	
	/* Compute mining results for planets owned by empires */
	'PLANET_MINING'
);

-- Types of recapitulative e-mail messages
CREATE TYPE recap_type
	AS ENUM ( 'ADMIN' , 'MSG', 'ERROR' );

-- Empire relations
CREATE TYPE empire_relation_type
	AS ENUM ('OWN' , 'ALLIED' , 'ENEMY');
