/*
 * Unit tests for defs.technology_dependencies_view
 */
BEGIN;
	\i utils/strings.sql
	-- Make the columns we don't use in the technology definition table NULL-able
	ALTER TABLE defs.technologies
		ALTER technology_category_id DROP NOT NULL ,
		ALTER technology_discovery_id DROP NOT NULL ,
		ALTER technology_description_id DROP NOT NULL ,
		ALTER technology_price DROP NOT NULL ,
		ALTER technology_points DROP NOT NULL;

	-- Create strings to use as the technologies' names
	SELECT _create_test_strings( 4 , 'tech' );
	
	-- Insert the technologies
	INSERT INTO defs.technologies ( technology_name_id )
		VALUES ( _get_string( 'tech1' ) ) ,
				( _get_string( 'tech2' ) ) ,
				( _get_string( 'tech3' ) ) ,
				( _get_string( 'tech4' ) );

	-- Insert dependencies: tech3 -> {tech1,tech2} , tech4 -> {tech3}
	INSERT INTO defs.technology_dependencies ( technology_name_id , technology_name_id_depends )
		VALUES ( _get_string( 'tech3' ) , _get_string( 'tech1' ) ) ,
			( _get_string( 'tech3' ) , _get_string( 'tech2' ) ) ,
			( _get_string( 'tech4' ) , _get_string( 'tech3' ) );

	-- ***** TESTS BEGIN HERE *****
	SELECT plan( 3 );

	SELECT diag_test_name( 'defs.technology_dependencies_view - Technologies with no dependencies' );
	SELECT is( technology_dependencies , '' )
		FROM defs.technology_dependencies_view
		WHERE technology_name_id = _get_string( 'tech1' );

	SELECT diag_test_name( 'defs.technology_dependencies_view - Technologies with a single dependency' );
	SELECT is( technology_dependencies , 'tech3' )
		FROM defs.technology_dependencies_view
		WHERE technology_name_id = _get_string( 'tech4' );

	SELECT diag_test_name( 'defs.technology_dependencies_view - Technologies with multiple dependencies' );
	SELECT ok( technology_dependencies = 'tech1,tech2' OR technology_dependencies = 'tech2,tech1' )
		FROM defs.technology_dependencies_view
		WHERE technology_name_id = _get_string( 'tech3' );

	SELECT * FROM finish( );
ROLLBACK;