package com.deepclone.lw.interfaces.acm;


import java.net.InetAddress;
import java.util.List;
import java.util.Map;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.users.AccountListEntry;
import com.deepclone.lw.cmd.admin.users.AccountSessionEntry;
import com.deepclone.lw.cmd.admin.users.AccountStatus;
import com.deepclone.lw.cmd.admin.users.AccountViewEntry;
import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.cmd.player.account.BanDetailsResponse;
import com.deepclone.lw.cmd.player.gdata.account.AccountData;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.sqld.accounts.ValidationResult;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;



public interface AccountManagement
{
	public void createAccount( EmailAddress address , Password password , String language )
			throws AccountMailException , MailerException , TranslationException;


	public ValidationResult validateAccount( EmailAddress address , String token , String empire , String planet );


	public void reactivateAccount( EmailAddress address )
			throws AccountMailException , MailerException;


	public void requestPasswordRecovery( EmailAddress address )
			throws AccountMailException , MailerException , PasswordRecoveryException;


	public void recoverPassword( EmailAddress address , String token , Password password )
			throws AccountMailException , PasswordRecoveryException , PasswordProhibitedException;


	public AccountSession login( EmailAddress address , String challenge , String sha1Hash , String md5Hash ,
			InetAddress ipAddress , String clientType , String sessionName );


	public void logout( long session , SessionTerminationType reason );


	public Account restoreSession( long session );


	public Account getAccount( EmailAddress address );


	public List< String > getEmpireNames( EmailAddress address );


	public AccountData getAccountPage( EmailAddress address );


	public void setLanguage( EmailAddress address , String language );


	public boolean setPassword( EmailAddress address , String challenge , String sha1Hash , String md5Hash ,
			Password newPassword )
			throws PasswordProhibitedException;


	public boolean setAddress( EmailAddress address , String challenge , String sha1Auth , String md5Auth ,
			EmailAddress nAddress )
			throws AccountMailException , MailerException;


	public void cancelAddressChange( EmailAddress cAddress );


	public AccountData confirmAddressChange( EmailAddress cAddress , String code );


	public void resetPreferences( EmailAddress address );


	public void setPreferences( EmailAddress address , Map< String , String > values );


	public void setQuit( EmailAddress address , String reason );


	public void cancelQuit( EmailAddress address );


	public void toggleVacation( EmailAddress address );


	public List< AccountListEntry > listAccounts( AccountStatus status , boolean online );


	public AccountViewEntry getAccountView( int id );


	public AccountSessionEntry viewSessions( int id );


	public void giveCredits( Administrator admin , int id , int credits );


	public BanDetailsResponse getBanDetails( EmailAddress address );
}
