package com.deepclone.lw.interfaces.game.resources;


import java.util.Map;



/**
 * Resources control interface
 * 
 * <p>
 * This interface corresponds to the component which controls resources from the game's point of
 * view. It contains methods that update mining settings.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface ResourcesController
{

	/**
	 * Update an empire's mining settings
	 * 
	 * @param empireId
	 *            the empire's identifier
	 * @param settings
	 *            the new settings
	 */
	public void updateEmpireSettings( int empireId , Map< String , Integer > settings );


	/**
	 * Toggle the source of a planet's mining priorities
	 * 
	 * <p>
	 * If the planet is currently using specific priorities, it will revert to empire-wide settings,
	 * and vice-versa. The planet must belong to the specified empire.
	 * 
	 * @param empire
	 *            the empire's identifier
	 * @param planet
	 *            the planet's identifier
	 */
	public void togglePlanet( int empire , int planet );


	/**
	 * Update a planet's mining settings
	 * 
	 * <p>
	 * If the planet is using specific mining priorities and belongs to the specified empire, update
	 * its mining priorities.
	 * 
	 * @param empire
	 *            the empire's identifier
	 * @param planet
	 *            the planet's identifier
	 * @param settings
	 *            the new priorities
	 */
	public void updatePlanetSettings( int empire , int planet , Map< String , Integer > settings );

}
