package com.deepclone.lw.interfaces.game.resources;


import java.util.List;
import java.util.Map;

import com.deepclone.lw.cmd.player.gdata.PlanetListResourceRecord;
import com.deepclone.lw.cmd.player.gdata.empire.EmpireResourceRecord;
import com.deepclone.lw.cmd.player.gdata.planets.PlanetResourceRecord;



/**
 * Resources information access component interface
 * 
 * <p>
 * This interface defines the methods which execute the queries required to extract information
 * about resources from the database.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface ResourcesInformationDAO
{

	/**
	 * Obtain resources information for a planet
	 * 
	 * @param planet
	 *            the planet to obtain information about
	 * 
	 * @return the list of planetary resource records
	 */
	public List< PlanetResourceRecord > getPlanetInformation( int planet );


	/**
	 * Obtain resources information for an empire
	 * 
	 * @param empire
	 *            the empire to obtain information about
	 * 
	 * @return the list of empire economic records
	 */
	public List< EmpireResourceRecord > getEmpireInformation( int empire );


	/**
	 * Obtain resources information for an empire's planet list
	 * 
	 * @param empire
	 *            the empire whose planet list is being generated
	 * 
	 * @return a map of planet list resource records associating planet identifiers to lists of
	 *         resource records
	 */
	public Map< Integer , List< PlanetListResourceRecord > > getPlanetListData( int empire );
}
