package com.deepclone.lw.interfaces.game.technologies;


import java.util.Map;

import com.deepclone.lw.cmd.player.tech.GetResearchCommand;
import com.deepclone.lw.cmd.player.tech.GetResearchResponse;



/**
 * Research control interface
 * 
 * <p>
 * This interface is implemented by the component which allows empires (and therefore players) to
 * view the research state and control it.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface ResearchController
{

	/**
	 * Obtain the state of an empire's research
	 * 
	 * <p>
	 * This method must be overridden to generate the response to a {@link GetResearchCommand}.
	 * 
	 * @param empire
	 *            the empire's identifier
	 * 
	 * @return the appropriate {@link GetResearchResponse} instance
	 */
	public GetResearchResponse getResearchData( int empire );


	/**
	 * Update research priorities
	 * 
	 * <p>
	 * This method must be implemented so it updates research priorities for an empire.
	 * 
	 * @param empire
	 *            the empire to update
	 * @param priorities
	 *            the new research priorities, as a map associating technology identifiers to
	 *            priority values
	 * 
	 * @return <code>true</code> on success, <code>false</code> otherwise
	 */
	public boolean updatePriorities( int empire , Map< String , Integer > priorities );


	/**
	 * Implement a technology
	 * 
	 * <p>
	 * This method implements a technology which has been fully researched.
	 * 
	 * @param empire
	 *            the empire trying to implement a technology
	 * @param technology
	 *            the technology's identifier
	 * 
	 * @return <code>true</code> on success, <code>false</code> otherwise
	 */
	public boolean implement( int empire , String technology );

}
