package com.deepclone.lw.interfaces.session;


import java.net.InetAddress;
import java.util.Date;

import com.deepclone.lw.cmd.admin.users.SessionTerminationType;



/**
 * Server-side session interface.
 * 
 * <p>
 * This interface provides session type definers (as defined by {@link SessionTypeDefiner}) limited
 * access to server-side session data.
 * 
 * @author tseeker
 */
public interface ServerSession
{

	/**
	 * @return the session's identifier (may be null when the session is being initialised)
	 */
	public String getIdentifier( );


	/**
	 * @return the identifier of the client on behalf of which the session was created
	 */
	public String getClient( );


	/**
	 * @return the IP address of the client on behalf of which the session was created
	 */
	public InetAddress getAddress( );


	/**
	 * @return the session's current authentication challenge string
	 */
	public String getChallenge( );


	/**
	 * Sets the session's expiration date
	 * 
	 * @param date
	 *            new expiration date
	 */
	public void setExpirationDate( Date date );


	/**
	 * @return the type of session termination or <code>null</code> if the session is not being
	 *         terminated
	 */
	public SessionTerminationType getTerminationType( );


	/**
	 * Terminates the session.
	 * 
	 * <p>
	 * This method can be called by session type definers which need to destroy a session. It will
	 * set the termination type to {@link SessionTerminationType#MANUAL} and call back
	 * {@link SessionTypeDefiner#terminate(ServerSession)}.
	 */
	public void terminate( );


	/**
	 * Terminates the session.
	 * 
	 * <p>
	 * This method can be called by session type definers which need to destroy a session and give a
	 * specific reason.
	 */
	public void terminate( SessionTerminationType reason );


	/**
	 * Stores data into the session.
	 * 
	 * @param key
	 *            key with which the specified value is to be associated
	 * @param value
	 *            value to be associated with the specified key
	 */
	public void put( String key , Object value );


	/**
	 * Retrieves data from the session.
	 * 
	 * @param key
	 *            the key whose associated value is to be returned
	 * @param type
	 *            expected type of the value
	 */
	public < T > T get( String key , Class< T > type );
}
