package com.deepclone.lw.interfaces.sys;


import java.util.Collection;
import java.util.Set;



/**
 * The constants management service provides a way for system constants to be accessed from a
 * central location. All constants are loaded in main memory, and can be modified through an
 * administrative interface.
 * 
 * @author tseeker
 * 
 */
public interface ConstantsManager
{

	/**
	 * Registers a set of constants; if the constants already exist in the database, their
	 * constraints are updated and their value is only modified if it violates the constraints. If
	 * they do not exist yet, they are created.
	 * 
	 * If constants are modified or created, and if there are users registered for these constants,
	 * they will be notified.
	 * 
	 * @param definitions
	 *            the set of constant definitions to register
	 */
	public void registerConstants( Collection< ConstantDefinition > definitions );


	/**
	 * Registers a constants user, which will need to be informed of the constants' changes. If the
	 * required constants have not been registered yet, the user instance will not be notified.
	 * 
	 * @param user
	 *            the constants user component to register
	 * @param constants
	 *            set of constant names the user wants to be informed about
	 */
	public void registerUser( ConstantsUser user , Set< String > constants );


	/**
	 * Unregisters a constants user.
	 * 
	 * @param user
	 *            the constants user component to unregister
	 */
	public void unregisterUser( ConstantsUser user );


	/**
	 * Creates an administrative interface for a specific administrator.
	 * 
	 * @param admin
	 *            the administrator making changes to the constants.
	 * @return the administrative interface
	 */
	public ConstantsAdministration getAdminSession( int admin );
}
