package com.deepclone.lw.interfaces.sys;


public interface SystemStatus
{

	/**
	 * Checks whether the system is currently under maintenance.
	 * 
	 * @return <code>null</code> or maintenance record
	 */
	public MaintenanceData checkMaintenance( );


	/**
	 * Activates maintenance mode.
	 */
	public void startMaintenance( int adminId , String reason , int duration )
			throws MaintenanceStatusException;


	/**
	 * Updates maintenance mode duration.
	 */
	public void updateMaintenance( int adminId , int durationFromNow )
			throws MaintenanceStatusException;


	/**
	 * Exits maintenance mode.
	 */
	public void endMaintenance( int adminId )
			throws MaintenanceStatusException;


	/**
	 * Starts a new tick.
	 * 
	 * @return new tick's identifier
	 */
	public long startTick( )
			throws TickStatusException , MaintenanceStatusException;


	/**
	 * Checks if there is a "stuck" tick.
	 * 
	 * @return current tick's identifier or <code>null</code> if there is no stuck tick
	 */
	public Long checkStuckTick( )
			throws MaintenanceStatusException;

}
