package com.deepclone.lw;


import java.util.Arrays;

import com.deepclone.lw.cli.CLITool;
import com.deepclone.lw.cli.ToolBase;
import com.deepclone.lw.srv.Server;



public class Main
		implements Runnable
{
	private String dataSource = "data-source.xml";
	private int port = 9137;
	private int servPort = 9138;
	private String tool = null;
	private String toolOptions[] = { };
	private ToolBase toolInstance;


	public Main( String[] args )
	{
		this.parseArguments( args );

		if ( this.tool != null ) {
			this.toolInstance = CLITool.create( this.tool );
		} else {
			this.toolInstance = new Server( );
		}

		if ( this.toolInstance == null ) {
			System.err.println( "tool " + this.tool + " not found" );
			System.exit( -1 );
		}

		this.toolInstance.setMainSettings( this.dataSource , this.port , this.servPort );
		if ( !this.toolInstance.setOptions( this.toolOptions ) ) {
			System.err.println( "invalid tool options (" + Arrays.asList( this.toolOptions ) + ")" );
			System.exit( -1 );
		}
	}


	@Override
	public void run( )
	{
		this.toolInstance.run( );
		System.gc( );
		System.exit( 0 );
	}


	private void parseArguments( String[] args )
	{
		int state = 0;
		for ( int i = 0 ; i < args.length ; i++ ) {
			switch ( state ) {
				case 0:
					state = this.parseNextOption( args[ i ] );
					break;
				case 1:
					this.dataSource = args[ i ];
					state = 0;
					break;
				case 2:
					this.port = this.getPort( args[ i ] );
					state = 0;
					break;
				case 3:
					this.servPort = this.getPort( args[ i ] );
					state = 0;
					break;
				case 4:
					this.tool = args[ i ];
					state = 300;
					break;
				case 300:
					if ( args[ i ].startsWith( "--" ) ) {
						i--;
					} else if ( !args[ i ].equals( "" ) ) {
						this.toolOptions = args[ i ].split( "\\s+" );
					}
					state = 0;
					break;
			}
		}

		if ( state % 100 != 0 || this.port == this.servPort ) {
			this.printCommandLine( );
		}
	}


	private int parseNextOption( String arg )
	{
		if ( "--data-source".equals( arg ) ) {
			return 1;
		} else if ( "--server-port".equals( arg ) ) {
			return 2;
		} else if ( "--service-port".equals( arg ) ) {
			return 3;
		} else if ( "--run-tool".equals( arg ) ) {
			return 4;
		}
		this.printCommandLine( );
		return -1;
	}


	private int getPort( String arg )
	{
		int port;
		try {
			port = Integer.parseInt( arg );
		} catch ( NumberFormatException e ) {
			port = 0;
		}
		if ( port < 1 || port > 65535 ) {
			this.printCommandLine( );
			return -1;
		}
		return port;
	}


	private void printCommandLine( )
	{
		System.err.println( "Legacy Worlds beta 6 - Game server and command line tools" );
		System.err.println( );
		System.err.println( " Command line options:" );
		System.err.println( "    --data-source <path>        Data source configuration file (default: "
				+ this.dataSource + ")" );
		System.err.println( "    --run-tool <tool> <options> Command line tool to run (default: none)" );
		System.err.println( "    --server-port <port>        RMI registry port number (default: " + this.port + ")" );
		System.err.println( "    --service-port <port>       RMI service port number (default: " + this.servPort + ")" );
		System.err.println( );

		System.exit( -1 );
	}


	public static void main( String[] args )
	{
		new Main( args ).run( );
	}

}
