package com.deepclone.lw.cli;


import java.sql.Types;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.utils.StoredProc;



public class CreateSuperuser
		extends CLITool
{
	private final Logger logger = Logger.getLogger( CreateSuperuser.class );

	private String address;
	private String appearAs;
	private TransactionTemplate tTemplate;
	private StoredProc fCreateAdmin;


	private ClassPathXmlApplicationContext getContext( )
	{
		// Load data source and Hibernate properties
		String[] dataConfig = {
			this.getDataSource( ) ,
		};
		FileSystemXmlApplicationContext ctx = new FileSystemXmlApplicationContext( dataConfig );
		ctx.refresh( );

		// Load transaction manager bean
		String[] cfg = {
			"configuration/transactions.xml"
		};
		return new ClassPathXmlApplicationContext( cfg , true , ctx );
	}


	private void getBeans( ApplicationContext ctx )
	{
		DataSource dSource = ctx.getBean( DataSource.class );
		PlatformTransactionManager tManager = ctx.getBean( PlatformTransactionManager.class );

		this.tTemplate = new TransactionTemplate( tManager );

		this.fCreateAdmin = new StoredProc( dSource , "admin" , "create_admin" );
		this.fCreateAdmin.addParameter( "address" , Types.VARCHAR );
		this.fCreateAdmin.addParameter( "appear_as" , Types.VARCHAR );
		this.fCreateAdmin.addParameter( "privileges" , Types.INTEGER );
		this.fCreateAdmin.addOutput( "err_code" , Types.INTEGER );
		this.fCreateAdmin.addOutput( "admin_id" , Types.INTEGER );
	}


	@Override
	public void run( )
	{
		AbstractApplicationContext ctx = this.getContext( );
		this.getBeans( ctx );
		this.createAdmin( );
	}


	private void createAdmin( )
	{
		int id;
		try {
			id = this.tTemplate.execute( new TransactionCallback< Integer >( ) {
				@Override
				public Integer doInTransaction( TransactionStatus status )
				{
					return doCreateAdmin( );
				}
			} );
		} catch ( RuntimeException e ) {
			this.logger.error( "administrator creation failed: " + e.getMessage( ) );
			return;
		}
		this.logger.info( "Administrator created with identifier #" + id );
	}


	private int doCreateAdmin( )
	{
		Map< String , Object > result;
		result = this.fCreateAdmin.execute( this.address , this.appearAs , Privileges.SUPER.getBits( ) );

		int errCode = (Integer) result.get( "err_code" );
		switch ( errCode ) {
			case 0:
				break;
			case 1:
				throw new RuntimeException( "unknown e-mail address " + this.address );
			case 2:
				throw new RuntimeException( "user with e-mail address " + this.address + " has an invalid status" );
			case 3:
				throw new RuntimeException( "administrator name " + this.appearAs + " is already in use" );
			case 4:
				throw new RuntimeException( "user with e-mail address " + this.address + " is already an administrator" );
			default:
				throw new RuntimeException( "unknown error code " + errCode );
		}

		return (Integer) result.get( "admin_id" );
	}


	@Override
	public boolean setOptions( String... options )
	{
		if ( options.length == 2 ) {
			this.address = options[ 0 ];
			this.appearAs = options[ 1 ];
			return true;
		}
		return false;
	}

}
