package com.deepclone.lw.cli;


import java.sql.Types;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;
import com.deepclone.lw.utils.StoredProc;



public class CreateUser
		extends CLITool
{

	private boolean query;
	private String mailAddress;
	private String password;
	private String language;

	private String pSha1;
	private String pMd5;

	private TransactionTemplate tTemplate;
	private StoredProc fCreateUser;


	private ClassPathXmlApplicationContext getContext( )
	{
		// Load data source and Hibernate properties
		String[] dataConfig = {
			this.getDataSource( ) ,
		};
		FileSystemXmlApplicationContext ctx = new FileSystemXmlApplicationContext( dataConfig );
		ctx.refresh( );

		// Load transaction manager bean
		String[] cfg = {
			"configuration/transactions.xml"
		};
		return new ClassPathXmlApplicationContext( cfg , true , ctx );
	}


	private void getBeans( ApplicationContext ctx )
	{
		DataSource dSource = ctx.getBean( DataSource.class );
		PlatformTransactionManager tManager = ctx.getBean( PlatformTransactionManager.class );

		this.tTemplate = new TransactionTemplate( tManager );

		this.fCreateUser = new StoredProc( dSource , "users" , "create_inactive_account" );
		this.fCreateUser.addParameter( "address" , Types.VARCHAR );
		this.fCreateUser.addParameter( "lang_id" , Types.VARCHAR );
		this.fCreateUser.addParameter( "pass_md5" , Types.VARCHAR );
		this.fCreateUser.addParameter( "pass_sha1" , Types.VARCHAR );
		this.fCreateUser.addOutput( "err_code" , Types.INTEGER );
		this.fCreateUser.addOutput( "a_id" , Types.INTEGER );
	}


	private void queryUser( )
	{
		System.out.println( "User creation     - E-mail address: " );
		this.mailAddress = CLITool.getLine( );
		System.out.println( "User creation     - Password:       " );
		this.password = CLITool.getLine( );
		System.out.println( "User creation     - Language ID:    " );
		this.language = CLITool.getLine( );
	}


	private boolean validate( )
	{
		EmailAddress addr = new EmailAddress( this.mailAddress );
		if ( !addr.isValid( ) ) {
			System.err.println( "Invalid mail address" );
			return false;
		}
		this.mailAddress = addr.getAddress( );

		Password pwd = new Password( this.password );
		if ( pwd.getStrength( ) < 10 ) {
			System.err.println( "Weak password" );
			return false;
		}
		this.pSha1 = pwd.getSha1( );
		this.pMd5 = pwd.getMd5( );

		return true;
	}


	private Map< String , Object > createAccount( )
	{
		return this.tTemplate.execute( new TransactionCallback< Map< String , Object > >( ) {
			@Override
			public Map< String , Object > doInTransaction( TransactionStatus status )
			{
				return fCreateUser.execute( mailAddress , language , pMd5 , pSha1 );
			}
		} );
	}


	@Override
	public void run( )
	{
		if ( this.query ) {
			this.queryUser( );
		}
		if ( !this.validate( ) ) {
			return;
		}

		AbstractApplicationContext ctx = this.getContext( );
		this.getBeans( ctx );

		Map< String , Object > m = this.createAccount( );
		int errCode = (Integer) m.get( "err_code" );
		switch ( errCode ) {
			case 0:
				System.out.println( "Account #" + (Integer) m.get( "a_id" ) + " created." );
				break;
			case -1:
				System.err.println( "Address in use" );
				break;
			case -2:
				System.err.println( "Invalid language" );
				break;
		}

		ToolBase.destroyContext( ctx );
	}


	@Override
	public boolean setOptions( String... options )
	{
		if ( options.length == 0 ) {
			this.query = true;
			return true;
		}

		if ( options.length == 3 ) {
			this.query = false;
			this.mailAddress = options[ 0 ];
			this.password = options[ 1 ];
			this.language = options[ 2 ];
			return true;
		}

		return false;
	}

}
