package com.deepclone.lw.cli;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cli.dbexport.BugEventMapper;
import com.deepclone.lw.cli.dbexport.BugGroup;
import com.deepclone.lw.cli.dbexport.LegacyWorldsDB;
import com.deepclone.lw.cli.dbexport.UserMapper;
import com.thoughtworks.xstream.XStream;



public class ExportDB
		extends CLITool
{

	private final String sGetUsers = "SELECT ul.* , al.administrator_id , al.name AS administrator_name , al.privileges , al.pass_sha1 AS administrator_sha1 , al.pass_md5 AS administrator_md5 "
			+ "FROM admin.users_list ul LEFT OUTER JOIN admin.admins_view al on al.account_id = ul.id "
			+ "WHERE ul.status IN ('ACTIVE' , 'VACATION' , 'START_VACATION' ) OR ( ul.game_credits > 0 AND ul.status <> 'BANNED' ) OR ( al IS NOT NULL AND al.active )";

	private final String sListGroupEvents = "SELECT group_id , event_id FROM bugs.br_first_report INNER JOIN bugs.br_status USING( group_id ) INNER JOIN bugs.events USING( group_id ) "
			+ "WHERE status <> 'NOT_A_BUG' ORDER BY group_id";

	private final String sGetBugEvents = "SELECT be.* , asd.account_status FROM bugs.br_first_report INNER JOIN bugs.br_status s USING( group_id ) INNER JOIN bugs.br_events be USING( bug_report_id ) "
			+ "LEFT OUTER JOIN bugs.account_status_data asd USING (event_id) WHERE s.status <> 'NOT_A_BUG' ORDER BY event_id";

	private static class LongPair
	{
		public long id1;
		public long id2;
	}

	private static class LongPairMapper
			implements RowMapper< LongPair >
	{

		@Override
		public LongPair mapRow( ResultSet rs , int rowNum )
				throws SQLException
		{
			LongPair lp = new LongPair( );
			lp.id1 = rs.getLong( 1 );
			lp.id2 = rs.getLong( 2 );
			return lp;
		}

	}

	private File file;
	private TransactionTemplate tTemplate;
	private JdbcTemplate dTemplate;


	private ClassPathXmlApplicationContext createContext( )
	{
		// Load data source
		String[] dataConfig = {
			this.getDataSource( ) ,
		};
		FileSystemXmlApplicationContext ctx = new FileSystemXmlApplicationContext( dataConfig );
		ctx.refresh( );

		// Load transaction manager bean
		String[] cfg = {
			"configuration/transaction-bean.xml"
		};
		return new ClassPathXmlApplicationContext( cfg , true , ctx );
	}


	private void prepareDataAccess( AbstractApplicationContext ctx )
	{
		DataSource dSource = ctx.getBean( DataSource.class );
		PlatformTransactionManager tManager = ctx.getBean( PlatformTransactionManager.class );

		this.tTemplate = new TransactionTemplate( tManager );
		this.dTemplate = new JdbcTemplate( dSource );
	}


	@Override
	public void run( )
	{
		AbstractApplicationContext ctx = this.createContext( );
		this.prepareDataAccess( ctx );

		LegacyWorldsDB db = this.tTemplate.execute( new TransactionCallback< LegacyWorldsDB >( ) {
			@Override
			public LegacyWorldsDB doInTransaction( TransactionStatus status )
			{
				return readDB( );
			}
		} );

		this.writeFile( db );
	}


	private LegacyWorldsDB readDB( )
	{
		LegacyWorldsDB db = new LegacyWorldsDB( );
		db.users = this.dTemplate.query( sGetUsers , new UserMapper( ) );
		db.bugReports.groups = this.readBugGroups( );
		db.bugReports.events = this.dTemplate.query( sGetBugEvents , new BugEventMapper( ) );
		return db;
	}


	private List< BugGroup > readBugGroups( )
	{
		List< BugGroup > groups = new ArrayList< BugGroup >( );
		Long prevGroupId = null;
		BugGroup current = null;

		for ( LongPair lp : this.dTemplate.query( sListGroupEvents , new LongPairMapper( ) ) ) {
			if ( prevGroupId == null || lp.id1 != prevGroupId ) {
				prevGroupId = lp.id1;
				current = new BugGroup( );
				groups.add( current );
			}
			current.events.add( lp.id2 );
		}

		return groups;
	}


	private void writeFile( LegacyWorldsDB db )
	{
		XStream xStream = new XStream( );
		xStream.autodetectAnnotations( true );

		FileWriter file;
		try {
			file = new FileWriter( this.file );
		} catch ( IOException e ) {
			throw new RuntimeException( e );
		}
		xStream.toXML( db , file );
		try {
			file.close( );
		} catch ( IOException e ) {
			throw new RuntimeException( e );
		}
	}


	@Override
	public boolean setOptions( String... options )
	{
		if ( options.length != 1 ) {
			return false;
		}
		this.file = new File( options[ 0 ] );
		return true;
	}
}
