package com.deepclone.lw.cli.dbexport;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.bt.data.BugEventType;



public class BugEventMapper
		implements RowMapper< BugReportEvent >
{

	@Override
	public BugReportEvent mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		BugEventType bet = BugEventType.valueOf( rs.getString( "event_type" ) );
		BugReportEvent bre;

		switch ( bet ) {
			case COMMENT:
				bre = this.getComment( rs );
				break;
			case INIT:
				bre = this.getInitialReport( rs );
				break;
			case MERGE:
				bre = this.getMerger( rs );
				break;
			case STATUS:
				bre = this.getStatusChange( rs );
				break;
			case VISIBILITY:
				bre = this.getVisibility( rs );
				break;
			default:
				throw new IllegalArgumentException( bet.toString( ) );
		}

		bre.id = rs.getLong( "event_id" );
		bre.timestamp = rs.getTimestamp( "event_timestamp" );
		bre.submitter = this.getSubmitter( rs );

		return bre;
	}


	private BugSubmitter getSubmitter( ResultSet rs )
			throws SQLException
	{
		BugSubmitter submitter = new BugSubmitter( );
		submitter.isAdmin = rs.getBoolean( "submitter_admin" );
		submitter.name = rs.getString( "submitter_name" );
		return submitter;
	}


	private BugReportEvent getComment( ResultSet rs )
			throws SQLException
	{
		BREComment evt = new BREComment( );
		evt.text = rs.getString( "contents" );
		evt.visible = rs.getBoolean( "visible" );
		return evt;
	}


	private BugReportEvent getInitialReport( ResultSet rs )
			throws SQLException
	{
		BREInitialReport evt = new BREInitialReport( );
		evt.title = rs.getString( "title" );
		evt.contents = rs.getString( "contents" );
		evt.xmlData = rs.getString( "account_status" );
		return evt;
	}


	private BugReportEvent getMerger( ResultSet rs )
			throws SQLException
	{
		BREMerger evt = new BREMerger( );
		evt.merged = rs.getLong( "merged_report_id" );
		return evt;
	}


	private BugReportEvent getStatusChange( ResultSet rs )
			throws SQLException
	{
		BREStatusChange evt = new BREStatusChange( );
		evt.status = rs.getString( "status" );
		return evt;
	}


	private BugReportEvent getVisibility( ResultSet rs )
			throws SQLException
	{
		BREVisibilityChange evt = new BREVisibilityChange( );
		evt.visible = rs.getBoolean( "visible" );
		return evt;
	}

}
