package com.deepclone.lw.cli.xmlimport.data;


import java.io.File;
import java.io.Serializable;



/**
 * Base class for importable data
 * 
 * <p>
 * This abstract class serves as the base for all classes that represent data imported from XML
 * files during the database's initialisation.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
public abstract class ImportableData
		implements Serializable
{

	/** The file these data were read from */
	private transient File readFrom;


	/** @return the file these data were read from */
	public final File getReadFrom( )
	{
		return this.readFrom;
	}


	/**
	 * Validate the item
	 * 
	 * <p>
	 * This method may be overridden to implement some form of validation of the loaded data.
	 * 
	 * @throws DataImportException
	 *             if data validation failed
	 */
	public void verifyData( )
			throws DataImportException
	{
		// EMPTY
	}


	/**
	 * Set the source file
	 * 
	 * <p>
	 * This method sets the file instance returned by {@link #getReadFrom()}. For classes that
	 * represent collections of data, this method should be overridden in order to update all
	 * contained items.
	 * 
	 * @param readFrom
	 *            the file the data were read from
	 */
	public void setReadFrom( File readFrom )
	{
		this.readFrom = readFrom;
	}

}
