package com.deepclone.lw.cli.xmlimport.data.evdef;


import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamImplicit;



/**
 * Event type definitions
 * 
 * <p>
 * This class represents the list of event type definitions as it is found in the appropriate data
 * file.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "lw-event-definitions" )
public class EventDefinitions
		extends ImportableData
		implements Iterable< EventDefinition >
{

	/** The list of event definitions */
	@XStreamImplicit( itemFieldName = "evdef" )
	private List< EventDefinition > definitions = new LinkedList< EventDefinition >( );


	/** Iterate over event definitions */
	@Override
	public Iterator< EventDefinition > iterator( )
	{
		return Collections.unmodifiableList( this.definitions ).iterator( );
	}


	/**
	 * Verify the list of definitions
	 * 
	 * <p>
	 * Make sure the list of definitions is not empty, then call the definitions' data check method.
	 */
	@Override
	public void verifyData( )
			throws DataImportException
	{
		super.verifyData( );
		if ( this.definitions == null ) {
			throw new DataImportException( "No definitions in this file" );
		}
		for ( EventDefinition def : this.definitions ) {
			def.verifyData( );
		}
	}

}
