package com.deepclone.lw.cli.xmlimport.data.evdef;


import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Common event field definition
 * 
 * <p>
 * This abstract class is used as the base for event field definitions.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
public abstract class EventFieldBase
		extends ImportableData
{

	/** The field's identifier */
	@XStreamAsAttribute
	@XStreamAlias( "id" )
	private String identifier;

	/** Whether the field is required or optional */
	@XStreamAsAttribute
	private Boolean required;


	/**
	 * Obtain the type of the field
	 * 
	 * @return the field's type.
	 */
	public abstract EventFieldType getType( );


	/**
	 * Gets the field's identifier.
	 * 
	 * @return the field's identifier
	 */
	public String getIdentifier( )
	{
		return this.identifier;
	}


	/**
	 * Checks if the field is required or optional.
	 * 
	 * @return <code>true</code> if the field is required
	 */
	public boolean isRequired( )
	{
		return this.required == null ? true : this.required;
	}

}
