package com.deepclone.lw.cli.xmlimport.data.i18n;


import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.thoughtworks.xstream.annotations.XStreamAlias;



/**
 * An in-line string definition
 * 
 * <p>
 * This class corresponds to string definitions which are stored directly inside the XML data file.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "inline-string" )
public class InlineString
		extends StringDefinition
{

	/** The string's text */
	private String value;


	/**
	 * Verify the in-line string definition
	 * 
	 * <p>
	 * Make sure that the definition actually contains a string.
	 */
	@Override
	public void verifyData( )
			throws DataImportException
	{
		super.verifyData( );

		if ( this.value == null || "".equals( this.value.trim( ) ) ) {
			throw new DataImportException( "Missing or empty in-line string definition '" + this.getId( ) + "'" );
		}
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see StringDefinition#getString()
	 */
	@Override
	public String getString( )
	{
		return this.value;
	}
}