package com.deepclone.lw.cli.xmlimport.data.resources;


import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Resource property parameter
 * 
 * <p>
 * This class represents the average and deviation of various properties of a resource.
 * 
 * @author <a href="mailto:tim@mitheren.com">T. Rosser</a>
 */
@SuppressWarnings( "serial" )
public class ResourceParameter
		extends ImportableData
{
	/** The property's average */
	@XStreamAsAttribute
	private Double average;

	/** The property's deviation */
	@XStreamAsAttribute
	private Double deviation;


	/**
	 * Check the property's data
	 * 
	 * <p>
	 * Make sure that a average and deviation are present.
	 * 
	 * @param property
	 *            the name of the property
	 */
	public void verifyData( String property )
			throws DataImportException
	{
		if ( this.average == null ) {
			throw new DataImportException( "Missing average for " + property );
		}
		if ( this.deviation == null ) {
			throw new DataImportException( "Missing deviation for " + property );
		}
	}


	/** @return the property's average */
	public Double getAverage( )
	{
		return this.average;
	}


	/** @return the property's deviation */
	public Double getDeviation( )
	{
		return this.deviation;
	}

}
