package com.deepclone.lw.srv;


import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.context.support.GenericApplicationContext;

import com.deepclone.lw.cli.ToolBase;
import com.deepclone.lw.interfaces.session.SessionManager;



public class Server
		extends ToolBase
{
	private static final String[] beanConfigurations = {
		"lw-server.xml" ,
	};

	private Lock terminationLock = new ReentrantLock( );
	private Condition termination = this.terminationLock.newCondition( );

	private AbstractApplicationContext applicationContext;


	private void addRMIService( GenericApplicationContext ctx , String name , Object bean , Class< ? > iface , int sPort )
	{
		BeanDefinitionBuilder builder;
		builder = BeanDefinitionBuilder.rootBeanDefinition( "org.springframework.remoting.rmi.RmiServiceExporter" );
		builder.setScope( BeanDefinition.SCOPE_SINGLETON );
		builder.addPropertyValue( "serviceName" , name );
		builder.addPropertyValue( "service" , bean );
		builder.addPropertyValue( "ServiceInterface" , iface.getCanonicalName( ) );
		builder.addPropertyValue( "registryPort" , String.valueOf( this.getRmiPort( ) ) );
		builder.addPropertyValue( "servicePort" , String.valueOf( sPort ) );
		ctx.registerBeanDefinition( name , builder.getBeanDefinition( ) );
	}


	private AbstractApplicationContext makeRMIContext( ApplicationContext parent )
	{
		GenericApplicationContext context = new GenericApplicationContext( parent );
		this.addRMIService( context , "termSrv" , parent.getBean( "terminator" ) , ServerTerminator.class , this.getTerminationPort( ) );
		this.addRMIService( context , "sessionSrv" , parent.getBean( "sessionManager" ) , SessionManager.class , this.getServicePort( ) );
		context.refresh( );
		return context;
	}


	private ApplicationContext makeDataConfigContext( )
	{
		String[] dSource = {
				this.getDataSource( ) ,
		};
		FileSystemXmlApplicationContext ctx = new FileSystemXmlApplicationContext( dSource );
		ctx.refresh( );
		return ctx;
	}


	private void createBeans( )
	{
		ApplicationContext ctx = this.makeDataConfigContext( );
		ctx = new ClassPathXmlApplicationContext( Server.beanConfigurations , true , ctx );
		this.applicationContext = this.makeRMIContext( ctx );
	}


	private void initialiseBeans( )
	{
		this.createBeans( );
		this.applicationContext.getBean( "terminator" , ServerTerminatorBean.class ).setServer( this );
	}


	private void waitForTermination( )
	{
		this.terminationLock.lock( );
		this.termination.awaitUninterruptibly( );
		this.terminationLock.unlock( );
	}


	@Override
	public void run( )
	{
		this.initialiseBeans( );
		this.waitForTermination( );
		ToolBase.destroyContext( this.applicationContext );
	}


	public void terminate( )
	{
		this.terminationLock.lock( );
		this.termination.signal( );
		this.terminationLock.unlock( );
	}

}
