package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.beans.bt.es.data.EmpireResourceInformation;
import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests for the {@link EmpireResourceInformationMapper} class.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestEmpireResourceInformationMapper
{

	/** Resource identifier used in tests */
	private static final String[] TEST_ID = {
			"Test1" , "Test2"
	};

	/** "Owed" value used in tests */
	private static final Double[] TEST_OWED = {
			1.0 , 2.0
	};

	/** "Possessed" value used in tests */
	private static final Double[] TEST_POSSESSED = {
			3.0 , 4.0
	};

	/** Mining priority used in tests */
	private static final Integer TEST_PRIORITY = 5;

	/** The fake result set used in the tests */
	private ResultSet resultSet;

	/** The mapper used in the tests */
	private EmpireResourceInformationMapper mapper;


	/** Create the mapper and the contents of the fake result set */
	@Before
	@SuppressWarnings( "unchecked" )
	public void setUp( )
	{
		this.mapper = new EmpireResourceInformationMapper( );

		HashMap< String , Object > rows[] = new HashMap[ 2 ];
		for ( int i = 0 ; i < 2 ; i++ ) {
			HashMap< String , Object > row = new HashMap< String , Object >( );
			row.put( "resource_name" , TEST_ID[ i ] );
			row.put( "empres_possessed" , TEST_POSSESSED[ i ] );
			row.put( "empres_owed" , TEST_OWED[ i ] );
			if ( i == 1 ) {
				row.put( "mining_priority" , TEST_PRIORITY );
			}
			rows[ i ] = row;
		}

		this.resultSet = MockResultSet.create( rows );
	}


	/** Mapping a row without mining priority */
	@Test
	public void testMapRow( )
			throws SQLException
	{
		EmpireResourceInformation empRes;

		this.resultSet.absolute( 1 );
		empRes = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( TEST_ID[ 0 ] , empRes.getResource( ) );
		assertEquals( TEST_POSSESSED[ 0 ] , empRes.getPossessed( ) );
		assertEquals( TEST_OWED[ 0 ] , empRes.getOwed( ) );
		assertNull( empRes.getMiningPriority( ) );
	}


	/** Mapping a row with mining priority */
	@Test
	public void testMapRowWithPriority( )
			throws SQLException
	{
		EmpireResourceInformation empRes;

		this.resultSet.absolute( 2 );
		empRes = this.mapper.mapRow( this.resultSet , 2 );

		assertEquals( TEST_ID[ 1 ] , empRes.getResource( ) );
		assertEquals( TEST_POSSESSED[ 1 ] , empRes.getPossessed( ) );
		assertEquals( TEST_OWED[ 1 ] , empRes.getOwed( ) );
		assertEquals( TEST_PRIORITY , empRes.getMiningPriority( ) );
	}
}
