package com.deepclone.lw.beans.game.resources;


import static org.junit.Assert.*;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cmd.player.gdata.empire.EmpireResourceRecord;
import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests of the {@link EmpireResourceMapper} class.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestEmpireResourceMapper
{
	private static final String TEST_STRING = "Test";

	private static final long POSSESSED_VALUE = 2L;

	private static final long P_INCOME_VALUE = 3L;

	private static final long P_UPKEEP_VALUE = 4L;

	private static final long F_UPKEEP_VALUE = 5L;

	private static final Integer PRIORITY_VALUE = 6;

	/** The fake result set fed to the mapper */
	private ResultSet resultSet;

	/** The mapper being tested */
	private EmpireResourceMapper mapper;


	@Before
	public void setUp( )
	{
		this.mapper = new EmpireResourceMapper( );

		@SuppressWarnings( "unchecked" )
		HashMap< String , Object > rows[] = new HashMap[ 2 ];
		for ( int i = 0 ; i < 2 ; i++ ) {
			HashMap< String , Object > row = new HashMap< String , Object >( );

			row.put( "resource_identifier" , TEST_STRING );

			row.put( "empres_possessed" , POSSESSED_VALUE + i );
			row.put( "planets_income" , P_INCOME_VALUE + i );
			row.put( "planets_upkeep" , P_UPKEEP_VALUE + i );
			row.put( "fleets_upkeep" , F_UPKEEP_VALUE + i );
			if ( i == 1 ) {
				row.put( "empmset_weight" , PRIORITY_VALUE + i );
			}

			rows[ i ] = row;
		}
		this.resultSet = MockResultSet.create( rows );
	}


	/** Test mapping a row that does not include a mining priority */
	@Test
	public void testMapWithoutPriority( )
			throws SQLException
	{
		this.resultSet.absolute( 1 );
		EmpireResourceRecord record = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( TEST_STRING , record.getIdentifier( ) );
		assertEquals( POSSESSED_VALUE , record.getStockpiled( ) );
		assertEquals( P_INCOME_VALUE , record.getIncome( ) );
		assertEquals( P_UPKEEP_VALUE , record.getPlanetUpkeep( ) );
		assertEquals( F_UPKEEP_VALUE , record.getFleetUpkeep( ) );
		assertNull( record.getMiningPriority( ) );
	}


	/** Test mapping a row that includes a mining priority */
	@Test
	public void testMapWithPriority( )
			throws SQLException
	{
		this.resultSet.absolute( 2 );
		EmpireResourceRecord record = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( TEST_STRING , record.getIdentifier( ) );
		assertEquals( 1 + POSSESSED_VALUE , record.getStockpiled( ) );
		assertEquals( 1 + P_INCOME_VALUE , record.getIncome( ) );
		assertEquals( 1 + P_UPKEEP_VALUE , record.getPlanetUpkeep( ) );
		assertEquals( 1 + F_UPKEEP_VALUE , record.getFleetUpkeep( ) );
		assertNotNull( record.getMiningPriority( ) );
		assertEquals( 1 + PRIORITY_VALUE , (int) record.getMiningPriority( ) );
	}

}
