package com.deepclone.lw.beans.updates;


import java.util.Collection;
import java.util.HashSet;

import com.deepclone.lw.interfaces.game.updates.UpdateBatchProcessor;



/**
 * A dummy batch processor used in game update tests
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class DummyBatchProcessor
		implements UpdateBatchProcessor
{
	/** The name that will be returned by {@link #getUpdateType()} */
	private final String updateType;

	/** Ticks for which the {@link #processBatch(long)} was called */
	private final HashSet< Long > ticks = new HashSet< Long >( );


	/**
	 * Set the update type
	 * 
	 * @param updateType
	 *            the update type
	 */
	public DummyBatchProcessor( String updateType )
	{
		this.updateType = updateType;
	}


	/**
	 * @return whatever type name was set when the instance was created
	 */
	@Override
	public String getUpdateType( )
	{
		return this.updateType;
	}


	/** Adds the specified tick identifier to the set of ticks */
	@Override
	public void processBatch( long tickId )
	{
		this.ticks.add( tickId );
	}


	/** @return the set of ticks for which {@link #processBatch(long)} was called */
	Collection< Long > getTicks( )
	{
		return this.ticks;
	}

}
