package com.deepclone.lw.beans.updates;


import com.deepclone.lw.interfaces.game.updates.GameUpdateProcessor;



/**
 * A mock game update processor
 * 
 * <p>
 * This mock component keeps track of which of its methods have been called. It can also simulate
 * runtime errors during tick processing.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
public class MockGameUpdateProcessor
		implements GameUpdateProcessor
{
	private boolean tryLockCalled = false;
	private boolean unlockCalled = false;
	private boolean endCycleCalled = false;
	private boolean executeCalled = false;

	private boolean failExecute = false;


	public boolean wasTryLockCalled( )
	{
		return this.tryLockCalled;
	}


	public boolean wasUnlockCalled( )
	{
		return this.unlockCalled;
	}


	public boolean wasEndCycleCalled( )
	{
		return this.endCycleCalled;
	}


	public boolean wasExecuteCalled( )
	{
		return this.executeCalled;
	}


	public void setFailExecute( boolean failExecute )
	{
		this.failExecute = failExecute;
	}


	@Override
	public boolean tryLock( )
	{
		this.tryLockCalled = true;
		return true;
	}


	@Override
	public void unlock( )
	{
		this.unlockCalled = true;
	}


	@Override
	public boolean endPreviousCycle( )
	{
		this.endCycleCalled = true;
		return false;
	}


	@Override
	public void executeUpdateCycle( )
	{
		this.executeCalled = true;
		if ( this.failExecute ) {
			throw new RuntimeException( "fail" );
		}
	}

}
