package com.deepclone.lw.cli.xmlimport.data;


import java.io.File;

import org.junit.Test;

import static org.junit.Assert.*;



/**
 * Unit tests for the {@link ImportableData} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestImportableData
{

	/**
	 * Dummy importable data
	 * 
	 * <p>
	 * This class is used in the test to have access to an importable data instance that corresponds
	 * exactly to the base class.
	 * 
	 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
	 */
	@SuppressWarnings( "serial" )
	private static class DummyData
			extends ImportableData
	{
		// EMPTY
	}


	/** Make sure that {@link ImportableData#getReadFrom()} returns <code>null</code> by default */
	@Test
	public void testUninitialisedReadFrom( )
	{
		DummyData data = new DummyData( );
		assertNull( data.getReadFrom( ) );
	}


	/** Checks the default implementation of {@link ImportableData#setReadFrom(File)} */
	@Test
	public void testSetReadFrom( )
	{
		DummyData data = new DummyData( );
		File file = new File( "test" );

		data.setReadFrom( file );
		assertNotNull( data.getReadFrom( ) );
		assertEquals( "test" , data.getReadFrom( ).getPath( ) );
	}


	/**
	 * Make sure that the default implementation of {@link ImportableData#verifyData()} always
	 * succeeds
	 * 
	 * @throws DataImportException
	 *             if the default implementation fails somehow
	 */
	@Test
	public void testDefaultVerify( )
			throws DataImportException
	{
		DummyData data = new DummyData( );
		data.verifyData( );
	}
}
