package com.deepclone.lw.cmd.admin.users;


/**
 * Account status enumeration
 * 
 * This enumerated type defines the various possible statuses for an account. Each status is
 * associated with a boolean value indicating whether the status corresponds to an active or
 * inactive account.
 * 
 * @author tseeker
 */
public enum AccountStatus {

	/** Unconfirmed account, still needs initial validation */
	UNCONFIRMED( false , "unconfirmed" ) ,

	/** Active account */
	ACTIVE( true , "active") ,

	/** Account currently in vacation mode */
	VACATION( true , "on vacation" ) ,

	/** Account currently entering vacation mode */
	START_VACATION( true , "entering vacation" ) ,

	/** Account being disabled by its owner */
	QUITTING( true , "leaving" ) ,

	/** Disabled account, either because it was closed or because it became inactive */
	DISABLED( false , "disabled" ) ,

	/** Banned account */
	BANNED( false , "banned" ) ,

	/** Account being re-activated */
	REACTIVATING( false , "being reactivated" );

	/** Activity flag */
	private boolean active;

	/** Description used in the admin interface */
	private final String description;


	private AccountStatus( boolean active , String description )
	{
		this.active = active;
		this.description = description;
	}


	/** @return <code>true</code> if the account status corresponds to an active account */
	public boolean isActive( )
	{
		return this.active;
	}


	public String getDescription( )
	{
		return description;
	}

}
