package com.deepclone.lw.cmd.player.gdata;


import java.io.Serializable;



/**
 * A resource record for the planet list
 * 
 * <p>
 * This class is used by {@link PlanetListData} to carry information about resources for the planet
 * list page.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class PlanetListResourceRecord
		implements Serializable

{

	/**
	 * Serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M2 with ID 1
	 * </ul>
	 */
	private static final long serialVersionUID = 1L;

	/** The name of the resource */
	private String name;

	/** The income for that resource type */
	private long income;

	/** The upkeep for that resource type */
	private long upkeep;

	/** The amount of that resource invested into the civilian construction queue */
	private long civInvestment;

	/** The amount of that resource invested into the military construction queue */
	private long milInvestment;


	/**
	 * Gets the name of the resource.
	 * 
	 * @return the name of the resource
	 */
	public String getName( )
	{
		return this.name;
	}


	/**
	 * Sets the name of the resource.
	 * 
	 * @param name
	 *            the new name of the resource
	 */
	public void setName( String name )
	{
		this.name = name;
	}


	/**
	 * Gets the income for that resource type.
	 * 
	 * @return the income for that resource type
	 */
	public long getIncome( )
	{
		return this.income;
	}


	/**
	 * Sets the income for that resource type.
	 * 
	 * @param income
	 *            the new income for that resource type
	 */
	public void setIncome( long income )
	{
		this.income = income;
	}


	/**
	 * Gets the upkeep for that resource type.
	 * 
	 * @return the upkeep for that resource type
	 */
	public long getUpkeep( )
	{
		return this.upkeep;
	}


	/**
	 * Sets the upkeep for that resource type.
	 * 
	 * @param upkeep
	 *            the new upkeep for that resource type
	 */
	public void setUpkeep( long upkeep )
	{
		this.upkeep = upkeep;
	}


	/**
	 * Gets the amount of that resource invested into the civilian construction queue.
	 * 
	 * @return the amount of that resource invested into the civilian construction queue
	 */
	public long getCivInvestment( )
	{
		return this.civInvestment;
	}


	/**
	 * Sets the amount of that resource invested into the civilian construction queue.
	 * 
	 * @param civInvestment
	 *            the new amount of that resource invested into the civilian construction queue
	 */
	public void setCivInvestment( long civInvestment )
	{
		this.civInvestment = civInvestment;
	}


	/**
	 * Gets the amount of that resource invested into the military construction queue.
	 * 
	 * @return the amount of that resource invested into the military construction queue
	 */
	public long getMilInvestment( )
	{
		return this.milInvestment;
	}


	/**
	 * Sets the amount of that resource invested into the military construction queue.
	 * 
	 * @param milInvestment
	 *            the new amount of that resource invested into the military construction queue
	 */
	public void setMilInvestment( long milInvestment )
	{
		this.milInvestment = milInvestment;
	}

}
