package com.deepclone.lw.cmd.player.gdata.planets;


import java.io.Serializable;
import java.util.List;



/**
 * Planet view details available to planet owners
 * 
 * <p>
 * This class carries the part of the planet view which is only available to empires owning the
 * planet being viewed.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class PlanetOwnView
		implements Serializable
{

	/**
	 * Serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M1 with ID 1
	 * <li>Modified in B6M2, ID set to 2
	 * </ul>
	 */
	private static final long serialVersionUID = 2L;

	/** Happiness of the planet's population as a percentage */
	private int happiness;

	/**
	 * Happiness change indicator
	 * 
	 * <p>
	 * This field's value is an integer between -2 and 2 (inclusive). It indicates the direction of
	 * the happiness' change (positive meaning that the happiness is increasing, negative meaning it
	 * is decreasing), and the strength of the change (absolute value 1 meaning a relatively small
	 * change, 2 a bigger one)
	 */
	private int hChange;

	/** Planet monetary income for 24h of RL time */
	private long income;

	/** Planet monetary upkeep for 24h of RL time */
	private long upkeep;

	/**
	 * Various planet state indicators
	 * 
	 * <p>
	 * This field carries a few flags about whether it is possible to rename or abandon the planet,
	 * and similar information.
	 */
	private OwnPlanetStatusData status;

	/** Civilian construction queue */
	private QueueData civQueue;

	/** Military construction queue */
	private QueueData milQueue;

	/** Descriptions of ships that can be built */
	private List< BuildableShipData > bShips;

	/** Descriptions of buildings that can be constructed */
	private List< BuildableBuildingData > bBuildings;

	/** List of the planet's resources */
	private List< PlanetResourceRecord > resources;

	/** Is the planet using specific mining settings? */
	private boolean ownMiningSettings;


	/** @return the planet's happiness */
	public int getHappiness( )
	{
		return this.happiness;
	}


	/**
	 * Set the planet's happiness
	 * 
	 * @param happiness
	 *            the planet's happiness
	 */
	public void setHappiness( int happiness )
	{
		this.happiness = happiness;
	}


	/** @return the planet's happiness change indicator */
	public int gethChange( )
	{
		return this.hChange;
	}


	/**
	 * Set the planet's happiness change indicator
	 * 
	 * @param hChange
	 *            the planet's happiness change indicator
	 */
	public void sethChange( int hChange )
	{
		this.hChange = hChange;
	}


	/** @return the planet's monetary income */
	public long getIncome( )
	{
		return this.income;
	}


	/**
	 * Set the planet's monetary income
	 * 
	 * @param income
	 *            the planet's monetary income
	 */
	public void setIncome( long income )
	{
		this.income = income;
	}


	/** @return the planet's monetary upkeep */
	public long getUpkeep( )
	{
		return this.upkeep;
	}


	/**
	 * Set the planet's monetary upkeep
	 * 
	 * @param upkeep
	 *            the planet's monetary upkeep
	 */
	public void setUpkeep( long upkeep )
	{
		this.upkeep = upkeep;
	}


	/** @return the planet status object */
	public OwnPlanetStatusData getStatus( )
	{
		return this.status;
	}


	/**
	 * Set the planet status object
	 * 
	 * @param status
	 *            the planet status object
	 */
	public void setStatus( OwnPlanetStatusData status )
	{
		this.status = status;
	}


	/** @return the civilian construction queue */
	public QueueData getCivQueue( )
	{
		return this.civQueue;
	}


	/**
	 * Set the civilian construction queue
	 * 
	 * @param civQueue
	 *            the civilian construction queue
	 */
	public void setCivQueue( QueueData civQueue )
	{
		this.civQueue = civQueue;
	}


	/** @return the military construction queue */
	public QueueData getMilQueue( )
	{
		return this.milQueue;
	}


	/**
	 * Set the military construction queue
	 * 
	 * @param milQueue
	 *            the military construction queue
	 */
	public void setMilQueue( QueueData milQueue )
	{
		this.milQueue = milQueue;
	}


	/** @return the descriptions of all ships which can be constructed */
	public List< BuildableShipData > getbShips( )
	{
		return this.bShips;
	}


	/**
	 * Set the descriptions of ships which can be constructed
	 * 
	 * @param bShips
	 *            the descriptions of ships which can be constructed
	 */
	public void setbShips( List< BuildableShipData > bShips )
	{
		this.bShips = bShips;
	}


	/** @return the descriptions of all buildings which can be constructed */
	public List< BuildableBuildingData > getbBuildings( )
	{
		return this.bBuildings;
	}


	/**
	 * Set the descriptions of buildings which can be constructed
	 * 
	 * @param bBuildings
	 *            the descriptions of buildings which can be constructed
	 */
	public void setbBuildings( List< BuildableBuildingData > bBuildings )
	{
		this.bBuildings = bBuildings;
	}


	/** @return the list of resources */
	public List< PlanetResourceRecord > getResources( )
	{
		return this.resources;
	}


	/**
	 * Set the list of resources
	 * 
	 * @param resourceProviders
	 *            the list of resources
	 */
	public void setResources( List< PlanetResourceRecord > resources )
	{
		this.resources = resources;
	}


	/** @return whether the planet is using specific mining settings */
	public boolean getOwnMiningSettings( )
	{
		return this.ownMiningSettings;
	}


	/**
	 * Set whether the planet is using specific mining settings
	 * 
	 * @param ownMiningSettings
	 *            whether the planet is using specific mining settings
	 */
	public void setOwnMiningSettings( boolean ownMiningSettings )
	{
		this.ownMiningSettings = ownMiningSettings;
	}

}
