package com.deepclone.lw.cmd.player.tech;


import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.GameResponseBase;
import com.deepclone.lw.cmd.player.gdata.empire.ResearchData;



/**
 * Response that lists all research entries (sent by the server in response to
 * {@link GetResearchCommand})
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class GetResearchResponse
		extends GameResponseBase
{

	/**
	 * The serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M2 with ID 1
	 * </ul>
	 */
	private static final long serialVersionUID = 1L;

	/** Sorted list of research entries */
	private final List< ResearchData > researchData;


	/**
	 * Initialise the response
	 * 
	 * <p>
	 * Copy and sort the list of research entries
	 * 
	 * @param page
	 *            the common page information
	 * @param technologies
	 *            the list of research entries
	 */
	public GetResearchResponse( GamePageData page , List< ResearchData > technologies )
	{
		super( page );
		this.researchData = new LinkedList< ResearchData >( technologies );
		Collections.sort( this.researchData );
	}


	/**
	 * Get the list of research entries
	 * 
	 * @return the sorted list of research entries
	 */
	public List< ResearchData > getResearchData( )
	{
		return this.researchData;
	}

}
