package com.deepclone.lw.web.admin;


import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.msg.*;
import com.deepclone.lw.cmd.msgdata.*;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.msgs.MessageFormatter;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class MessagesPages
		extends PageControllerBase
{

	private final static int perPage = 30;

	private MessageFormatter formatter;


	@Autowired( required = true )
	public void setFormatter( MessageFormatter formatter )
	{
		this.formatter = formatter;
	}


	@RequestMapping( "/messages" )
	public String viewInbox( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		MessageBoxView view = this.viewMessageBox( true , this.getSession( AdminSession.class , request ) , 0 );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	@RequestMapping( "/inbox-{page}" )
	public String viewInbox( HttpServletRequest request , Model model , @PathVariable( "page" ) String sPage )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int page;
		try {
			page = Integer.parseInt( sPage );
		} catch ( NumberFormatException e ) {
			page = 0;
		}

		MessageBoxView view = this.viewMessageBox( true , this.getSession( AdminSession.class , request ) , page );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	@RequestMapping( "/inbox-from-{id}" )
	public String viewInboxFrom( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			id = 0;
		}

		MessageBoxView view = this.viewMessageBoxFrom( true , this.getSession( AdminSession.class , request ) , id );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	@RequestMapping( "/outbox" )
	public String viewOutbox( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		MessageBoxView view = this.viewMessageBox( false , this.getSession( AdminSession.class , request ) , 0 );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	@RequestMapping( "/outbox-{page}" )
	public String viewOutbox( HttpServletRequest request , Model model , @PathVariable( "page" ) String sPage )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int page;
		try {
			page = Integer.parseInt( sPage );
		} catch ( NumberFormatException e ) {
			page = 0;
		}

		MessageBoxView view = this.viewMessageBox( false , this.getSession( AdminSession.class , request ) , page );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	@RequestMapping( "/outbox-from-{id}" )
	public String viewOutboxFrom( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			id = 0;
		}

		MessageBoxView view = this.viewMessageBoxFrom( false , this.getSession( AdminSession.class , request ) , id );
		return this.render( model , "internal" , "en" , "messageBox" , view );
	}


	private MessageBoxView viewMessageBox( boolean inbox , AdminSession aSession , int page )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		GetMessagesResponse mResponse = aSession.getMessages( inbox );
		MessageBoxView view = new MessageBoxView( mResponse.getAdmin( ) , inbox );
		List< MessageListEntry > messages = mResponse.getMessages( );

		// Handle paging
		messages = this.setPage( view , messages , page );
		this.prepareMessages( view , messages );

		return view;
	}


	private MessageBoxView viewMessageBoxFrom( boolean inbox , AdminSession pSession , long fromId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		GetMessagesResponse mResponse = pSession.getMessages( inbox );
		MessageBoxView view = new MessageBoxView( mResponse.getAdmin( ) , inbox );
		List< MessageListEntry > messages = mResponse.getMessages( );
		int page = 0;
		int nSeen = -1;
		for ( MessageListEntry e : messages ) {
			nSeen++;
			if ( e.getId( ) != fromId ) {
				continue;
			}
			page = ( nSeen - nSeen % MessagesPages.perPage ) / MessagesPages.perPage;
			break;
		}

		// Handle paging
		messages = this.setPage( view , messages , page );
		this.prepareMessages( view , messages );

		return view;
	}


	private void prepareMessages( MessageBoxView view , List< MessageListEntry > messages )
	{
		for ( MessageListEntry message : messages ) {
			message.setTitle( this.formatter.cleanMessage( message.getTitle( ) ) );
			view.addMessage( message );
		}
	}


	private List< MessageListEntry > setPage( MessageBoxView view , List< MessageListEntry > messages , int page )
	{
		int nMessages = messages.size( );
		int mod = nMessages % MessagesPages.perPage;
		int nPages = ( nMessages - mod ) / MessagesPages.perPage + ( mod > 0 ? 1 : 0 );
		if ( page < 0 ) {
			page = 0;
		} else if ( page >= nPages ) {
			page = nPages - 1;
		}
		if ( !messages.isEmpty( ) ) {
			messages = messages.subList( page * MessagesPages.perPage , Math.min( ( page + 1 ) * MessagesPages.perPage ,
					nMessages ) );
		}

		view.setPages( nPages );
		view.setcPage( page );
		return messages;
	}


	@RequestMapping( "/inbox-message-{id}" )
	public String viewInboxMessage( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.viewMessage( request , model , sId , true );
	}


	@RequestMapping( "/outbox-message-{id}" )
	public String viewOutboxMessage( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.viewMessage( request , model , sId , false );
	}


	private String viewMessage( HttpServletRequest request , Model model , String sId , boolean inbox )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long id;
		try {
			id = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( inbox ? "messages" : "outbox" );
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		ReadMessageResponse response = aSession.readMessage( inbox , id );
		Message message = response.getMessage( );
		if ( message == null ) {
			return this.redirect( inbox ? "messages" : "outbox" );
		}

		boolean internal = ( message.getType( ) == MessageType.INTERNAL );
		message.setTitle( this.formatter.cleanMessage( message.getTitle( ) ) );
		message.setContents( this.formatter.formatMessage( message.getContents( ) , internal ) );

		return this.render( model , "internal" , "en" , "message" , response );
	}


	@RequestMapping( value = "/messages.action" , method = RequestMethod.POST )
	public String mbAction( HttpServletRequest request , Model model , @RequestParam( "inbox" ) String sInbox ,
			@RequestParam( "page" ) String sPage , @RequestParam( "target" ) String sTarget ,
			@RequestParam( "action" ) String action ,
			@RequestParam( value = "selection" , required = false ) String[] sSelection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		boolean inbox = "1".equals( sInbox );
		if ( !inbox ) {
			action = "d";
		}

		boolean useSelected = "0".equals( sTarget );
		long selection[];
		if ( useSelected ) {
			List< Long > rSel = new LinkedList< Long >( );
			if ( sSelection != null ) {
				for ( String sItem : sSelection ) {
					Long value;
					try {
						value = Long.parseLong( sItem );
					} catch ( NumberFormatException e ) {
						continue;
					}
					rSel.add( value );
				}

				selection = new long[ rSel.size( ) ];
				int i = 0;
				for ( Long value : rSel ) {
					selection[ i++ ] = value;
				}
			} else {
				selection = new long[ 0 ];
			}
		} else {
			selection = null;
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		if ( selection == null || selection.length > 0 ) {
			if ( "d".equals( action ) ) {
				aSession.deleteMessages( inbox , selection );
			} else if ( "r".equals( action ) ) {
				aSession.markRead( selection );
			} else if ( "u".equals( action ) ) {
				aSession.markUnread( selection );
			}
		}

		int page;
		try {
			page = Integer.parseInt( sPage );
		} catch ( NumberFormatException e ) {
			page = 0;
		}
		return this.redirect( ( inbox ? "inbox" : "outbox" ) + "-" + page );
	}


	@RequestMapping( value = "/message.action" , method = RequestMethod.POST )
	public String msgAction( HttpServletRequest request , Model model , @RequestParam( "inbox" ) String sInbox ,
			@RequestParam( "id" ) String sId , @RequestParam( "next" ) String sNext ,
			@RequestParam( value = "delete" , required = false ) String delete ,
			@RequestParam( value = "reply" , required = false ) String reply )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		boolean inbox = "1".equals( sInbox );

		long id;
		try {
			id = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( inbox ? "messages" : "outbox" );
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		if ( delete != null ) {
			aSession.deleteMessages( inbox , new long[] {
				id
			} );

			try {
				id = Long.parseLong( sNext );
			} catch ( NumberFormatException e ) {
				return this.redirect( inbox ? "messages" : "outbox" );
			}
			return this.redirect( ( inbox ? "inbox" : "outbox" ) + "-message-" + id );
		} else if ( reply != null ) {
			ComposeMessageResponse response;
			response = aSession.replyTo( inbox , id );
			return this.showWriter( model , response );
		}
		return this.redirect( inbox ? "messages" : "outbox" );
	}


	@RequestMapping( "/compose-message" )
	public String composeNew( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		AdminSession aSession = this.getSession( AdminSession.class , request );
		return this.render( model , "internal" , "en" , "messageWriter" , aSession.initNewMessage( ) );
	}


	@RequestMapping( "/msg-empire-{id}" )
	public String messageEmpire( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.newMessageTo( request , model , MessageType.EMPIRE , sId );
	}


	@RequestMapping( "/msg-admin-{id}" )
	public String messageAdmin( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.newMessageTo( request , model , MessageType.ADMINISTRATOR , sId );
	}


	@RequestMapping( value = "/send-message.action" , method = RequestMethod.POST )
	public String sendMessage( HttpServletRequest request , Model model , @RequestParam( "toType" ) String sToType ,
			@RequestParam( "toName" ) String toName , @RequestParam( "title" ) String title ,
			@RequestParam( "contents" ) String contents ,
			@RequestParam( value = "rtInbox" , required = false ) String sRtInbox ,
			@RequestParam( value = "rtId" , required = false ) String sRtId ,
			@RequestParam( value = "cancel" , required = false ) String sCancel )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		// Handle cancellation
		if ( sCancel != null ) {
			return this.cancelSendRedirect( sRtInbox , sRtId );
		}

		// Get message type
		MessageType type;
		try {
			type = MessageType.valueOf( sToType );
		} catch ( IllegalArgumentException e ) {
			type = MessageType.INTERNAL;
		}
		if ( type == MessageType.INTERNAL ) {
			type = MessageType.EMPIRE;
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		ComposeMessageResponse response;
		if ( sRtInbox == null || sRtId == null ) {
			response = aSession.sendMessage( type , toName , title , contents );
		} else {
			boolean inbox = "1".equals( sRtInbox );
			long rtId;
			try {
				rtId = Long.parseLong( sRtId );
			} catch ( NumberFormatException e ) {
				return this.redirect( inbox ? "messages" : "outbox" );
			}

			response = aSession.sendReply( inbox , rtId , type , toName , title , contents );
		}

		if ( !response.isError( ) ) {
			return this.cancelSendRedirect( sRtInbox , sRtId );
		}
		return this.showWriter( model , response );
	}


	private String newMessageTo( HttpServletRequest request , Model model , MessageType type , String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "compose-message" );
		}

		ComposeMessageResponse response;
		response = this.getSession( AdminSession.class , request ).messageTo( type , id );
		return this.showWriter( model , response );
	}


	private String showWriter( Model model , ComposeMessageResponse response )
	{
		if ( response.getReplyTo( ) != null ) {
			Message message = response.getReplyTo( );
			message.setTitle( this.formatter.cleanMessage( message.getTitle( ) ) );
			message.setContents( this.formatter.formatMessage( message.getContents( ) , false ) );
		}
		return this.render( model , "internal" , "en" , "messageWriter" , response );
	}


	private String cancelSendRedirect( String sRtInbox , String sRtId )
	{
		if ( sRtInbox == null || sRtId == null ) {
			return this.redirect( "messages" );
		}

		boolean inbox = "1".equals( sRtInbox );
		long rtId;
		try {
			rtId = Long.parseLong( sRtId );
		} catch ( NumberFormatException e ) {
			return this.redirect( inbox ? "messages" : "outbox" );
		}

		return this.redirect( ( inbox ? "inbox" : "outbox" ) + "-message-" + rtId );
	}
}
