package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.tick.TickerStatusResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class TickerPages
		extends PageControllerBase
{

	@RequestMapping( "/ticker" )
	public String viewStatus( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		TickerStatusResponse response = this.getSession( AdminSession.class , request ).getTickerStatus( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "ticker" , response );
	}


	@RequestMapping( "/toggle-ticker.action" )
	public String toggleTicker( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( AdminSession.class , request ).toggleTicker( );
		return this.redirect( "ticker" );
	}


	@RequestMapping( "/start-ticker-task-{id}.action" )
	public String startTask( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.setTickerTaskStatus( request , sId , true );
	}


	@RequestMapping( "/stop-ticker-task-{id}.action" )
	public String stopTask( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.setTickerTaskStatus( request , sId , false );
	}


	private String setTickerTaskStatus( HttpServletRequest request , String sId , boolean run )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "ticker" );
		}
		this.getSession( AdminSession.class , request ).setTickerTaskStatus( id , run );
		return this.redirect( "ticker" );
	}


	@RequestMapping( value = "/set-ticker-task-{id}.action" , method = RequestMethod.POST )
	public String setTaskDelay( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId ,
			@RequestParam( "delay" ) String sDelay , @RequestParam( "multiplier" ) String sMultiplier )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		long delay , multiplier;
		try {
			id = Integer.parseInt( sId );
			delay = Long.parseLong( sDelay );
			multiplier = Long.parseLong( sMultiplier );
		} catch ( NumberFormatException e ) {
			return this.redirect( "ticker" );
		}
		delay *= multiplier;
		if ( delay <= 0 ) {
			return this.redirect( "ticker" );
		}
		this.getSession( AdminSession.class , request ).setTickerTaskStatus( id , delay );
		return this.redirect( "ticker" );
	}
}
