package com.deepclone.lw.web.beans.msgs;


import java.util.regex.Pattern;



public class MessageFormatterBean
		implements MessageFormatter
{
	private final Pattern planet = Pattern.compile( "\\{\\{planet:(\\d+) ([^\\}]+)\\}\\}" );
	private final Pattern empire = Pattern.compile( "\\{\\{empire:(\\d+) ([^\\}]+)\\}\\}" );
	private final Pattern battle = Pattern.compile( "\\{\\{battle:(\\d+) ([^\\}]+)\\}\\}" );
	private final Pattern bug = Pattern.compile( "\\{\\{bug:(\\d+)\\}\\}" );


	public String cleanMessage( String message )
	{
		message = message.trim( );

		StringBuilder repLine = new StringBuilder( );
		for ( Character c : message.toCharArray( ) ) {
			int nVal = Character.codePointAt( new char[] {
				c
			} , 0 );
			if ( Character.isISOControl( nVal ) ) {
				continue;
			}

			switch ( c ) {
				case '<':
					repLine.append( "&lt;" );
					continue;
				case '>':
					repLine.append( "&gt;" );
					continue;
				case '&':
					repLine.append( "&amp;" );
					continue;
				case '\'':
					repLine.append( "&#39;" );
					continue;
				case '"':
					repLine.append( "&quot;" );
					continue;
			}
			if ( nVal < 128 ) {
				repLine.append( c );
			} else {
				repLine.append( "&#" ).append( nVal ).append( ";" );
			}
		}
		message = repLine.toString( );
		return message;
	}


	@Override
	public String formatMessage( String message , boolean internal )
	{
		StringBuilder result = new StringBuilder( );
		boolean inList = false;
		boolean prevEmpty = true;
		for ( String line : message.split( "\\n" ) ) {
			line = this.cleanMessage( line );

			if ( line.equals( "" ) ) {
				if ( prevEmpty ) {
					continue;
				}
				if ( inList ) {
					result.append( "</ul>" );
				} else {
					result.append( "</p>" );
				}
				result.append( "<p>&nbsp;</p>" );
				prevEmpty = true;
				inList = false;
				continue;
			}

			if ( line.charAt( 0 ) == '*' ) {
				if ( !inList ) {
					if ( !prevEmpty ) {
						result.append( "</p>" );
					}
					result.append( "<ul>" );
					inList = true;
				}
				result.append( "<li>" ).append( line.substring( 1 ) ).append( "</li>" );
			} else {
				if ( prevEmpty ) {
					result.append( "<p>" );
				} else if ( inList ) {
					result.append( "</ul><p>" );
					inList = false;
				} else {
					result.append( "<br/>" );
				}
				result.append( line );
			}
			prevEmpty = false;
		}

		if ( !prevEmpty ) {
			if ( inList ) {
				result.append( "</ul>" );
			} else {
				result.append( "</p>" );
			}
		}

		if ( !internal ) {
			return result.toString( );
		}
		return this.applyInternals( result.toString( ) );
	}


	private String applyInternals( String message )
	{
		message = planet.matcher( message ).replaceAll( "<a href=\"planet-$1\">$2</a>" );
		message = empire.matcher( message ).replaceAll( "<a href=\"msg-empire-$1\">$2</a>" );
		message = battle.matcher( message ).replaceAll( "<a href=\"battle-$1-latest\">$2</a>" );
		message = bug.matcher( message ).replaceAll( "<a href=\"bug-$1\">#$1</a>" );
		return message;
	}

}
