package com.deepclone.lw.web.beans.session;


import com.deepclone.lw.cmd.MaintenanceResponse;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionException;



public abstract class Session
{

	private ClientSessionReference reference;
	private SessionClient sClient;


	void initialise( SessionClient client , ClientSessionReference ref )
	{
		this.sClient = client;
		this.reference = ref;
	}


	public final boolean isActive( )
	{
		return !this.reference.isNull( );
	}


	public boolean authenticate( String identifier , String secret )
			throws SessionException , SessionServerException
	{
		try {
			this.sClient.authenticate( this.reference , identifier , secret );
		} catch ( SessionException e ) {
			this.handleSessionException( e );
		} catch ( SessionServerException e ) {
			this.reference.clear( );
			throw e;
		}

		if ( this.reference.isNull( ) ) {
			return false;
		}
		return this.reference.getReference( ).authenticated;
	}


	public void terminate( )
			throws SessionServerException , SessionException
	{
		try {
			this.sClient.terminate( this.reference );
		} catch ( SessionServerException e ) {
			throw e;
		} catch ( SessionException e ) {
			this.handleSessionException( e );
		}
	}


	public final String getSessionSubType( )
	{
		try {
			return this.reference.getReference( ).extra;
		} catch ( NullPointerException e ) {
			return null;
		}
	}


	protected CommandResponse execute( Command command )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		CommandResponse response = null;
		try {
			response = this.sClient.execute( this.reference , command );
		} catch ( SessionServerException e ) {
			this.reference.clear( );
			throw e;
		} catch ( SessionException e ) {
			this.handleSessionException( e );
		}

		if ( response instanceof MaintenanceResponse ) {
			throw new SessionMaintenanceException( (MaintenanceResponse) response );
		}
		return response;
	}


	protected void handleSessionException( SessionException e )
			throws SessionException
	{
		if ( !e.isSessionKept( ) ) {
			this.reference.clear( );
		}
		throw e;
	}

}
