package com.deepclone.lw.web.beans.view;


import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.Model;

import com.deepclone.lw.cmd.player.gdata.GameResponseBase;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.session.ClientSessionReference;
import com.deepclone.lw.web.beans.session.Session;
import com.deepclone.lw.web.beans.session.SessionClient;
import com.deepclone.lw.web.beans.session.SessionServerException;



public abstract class PageControllerBase
{

	protected SessionClient client;


	@Autowired( required = true )
	public void setSessionClient( SessionClient client )
	{
		this.client = client;
	}


	protected String render( Model model , String template , String language )
	{
		return this.render( model , template , new Page( null , language ) );
	}


	protected String render( Model model , String template , String language , String subPage )
	{
		return this.render( model , template , new Page( subPage , language ) );
	}


	protected String render( Model model , String template , String language , CommandResponse data )
	{
		return this.render( model , template , new Page( null , language , data ) );
	}


	protected String render( Model model , String template , String language , String subPage , CommandResponse data )
	{
		return this.render( model , template , new Page( subPage , language , data ) );
	}


	protected String renderMap( Model model , String template , String language , String subPage , Object... data )
	{
		HashMap< String , Serializable > map = new HashMap< String , Serializable >( );
		for ( int i = 0 ; i < data.length / 2 ; i++ ) {
			map.put( (String) data[ i * 2 ] , (Serializable) data[ i * 2 + 1 ] );
		}
		return this.render( model , template , new Page( subPage , language , map ) );
	}


	protected String renderStatic( Model model , String template , String language , String name )
	{
		return this.render( model , template , new Page( "static" , language , name ) );
	}


	protected String renderStatic( Model model , String template , String language , GameResponseBase gameData ,
			String name )
	{
		return this.renderMap( model , template , language , "static" , "page" , gameData.getPage( ) , "name" , name );
	}


	protected String render( Model model , String template , Page page )
	{
		model.addAttribute( "container" , template );
		model.addAttribute( "language" , page.getLanguage( ) );
		model.addAttribute( "type" , page.getType( ) );
		model.addAttribute( "data" , page.getData( ) );
		model.addAttribute( "dataType" , page.getDataType( ) );
		return "ROOT";
	}


	protected String render( Model model , String template , String language , String type , Map< String , Object > data )
	{
		model.addAttribute( "container" , template );
		model.addAttribute( "language" , language );
		model.addAttribute( "type" , type );
		model.addAttribute( "data" , data );
		model.addAttribute( "dataType" , "map" );
		return "ROOT";
	}


	protected String redirect( String to )
	{
		return "redirect:" + to;
	}


	protected final InetAddress getAddress( HttpServletRequest request )
	{
		InetAddress address;
		try {
			address = InetAddress.getByName( request.getRemoteAddr( ) );
		} catch ( UnknownHostException e ) {
			throw new RuntimeException( e );
		}
		return address;
	}


	protected final ClientSessionReference getSessionReference( HttpServletRequest request )
	{
		return (ClientSessionReference) request.getSession( ).getAttribute( "sReference" );
	}


	protected < T extends Session > T createTemporarySession( Class< T > type , HttpServletRequest request )
			throws SessionException , SessionServerException
	{
		return this.client.getSession( new ClientSessionReference( null ) , type , this.getAddress( request ) );
	}


	protected < T extends Session > T initSession( Class< T > type , HttpServletRequest request )
			throws SessionException , SessionServerException
	{
		return this.client.getSession( this.getSessionReference( request ) , type , this.getAddress( request ) );
	}


	protected < T extends Session > T getSession( Class< T > type , HttpServletRequest request )
	{
		return this.client.restoreSession( this.getSessionReference( request ) , type );
	}


	protected void clearSession( HttpServletRequest request )
	{
		this.getSessionReference( request ).clear( );
	}


	@SuppressWarnings( "unchecked" )
	protected Map< String , Object > getInput( HttpServletRequest request )
	{
		return (Map< String , Object >) request.getParameterMap( );
	}

}
