<#macro render_preference pref>
	<#switch pref.type>
		<#case 'BOOLEAN'>
			<@form_select label=pref.name name=pref.id id="pref-${pref.id}">
				<@form_option value="0" text="No" selected=( pref.value != "1" )/>
				<@form_option value="1" text="Yes" selected=( pref.value == "1" )/>
			</@form_select>
			<#break>
		<#case 'INTEGER'>
		<#case 'STRING'>
			<@form_text label=pref.name name=pref.id id="pref-${pref.id}" value=pref.value />
			<#break>
		<#case 'CHOICE'>
			<@form_select label=pref.name name=pref.id id="pref-${pref.id}">
				<#list pref.choices as choice>
					<@form_option value=choice.value text=choice.display selected=( pref.value == choice.value )/>
				</#list>
			</@form_select>
			<#break>
	</#switch>
	<#if pref.description?has_content>
		<@form_extra>${pref.description?xhtml}</@form_extra>
	</#if>
</#macro>
<#macro render_prefs_category category>
	<@form_part title=category.name />
	<#list category.preferences as p>
		<@render_preference pref=p />
	</#list>
</#macro>
<#macro render>
<@page title="Account">
	<@tabs>

		<@tab id="prefs" title="Preferences">
			<@form name="prefs-form" action="set-preferences" hash="prefs">
				<#list data.account.preferences as c>
					<@render_prefs_category category=c />
				</#list>
				<@form_extended_submit label="Change preferences">
					<@ff_submit label="Defaults" name="load-default-preferences" />
				</@form_extended_submit>
			</@form>
		</@tab>

		<@tab id="basics" title="Identity">
			<@form name="lang-form" action="set-language" hash="basics">
				<@form_part title="Account language" />
				<@form_select name="language" label="Language">
					<#list data.account.supportedLanguages.languages as lang>
						<@form_option text=lang.name value=lang.id selected=(lang.id == data.account.language) />
					</#list>
				</@form_select>
				<@form_submit label="Update" />
			</@form>

			<@form name="pwd-form" action="set-password" hash="basics">
				<@form_part title="Change password" />
				<#if data.authError!false>
					<@form_error>Incorrect password</@form_error>
				</#if>
				<@form_pwd label="Current password" name="current" />
				<#switch data.passwordError!"">
					<#case "EMPTY">
						<@form_error>Empty password.</@form_error>
						<#break>
					<#case "TOO_WEAK">
						<@form_error>This password is too weak.</@form_error>
						<#break>
					<#case "MISMATCH">
						<@form_error>Password field and confirmation field do not match.</@form_error>
						<#break>
					<#case "PROHIBITED">
						<@form_error>You are not allowed to use this password here.</@form_error>
						<#break>
					<#default>
						<@form_extra>At least 6 characters, containing both text and numbers. Bonus points for special characters, spaces, and extremely long passwords.</@form_extra>
				</#switch>
				<@form_pwd label="Password" name="password" />
				<@form_pwd label="Password (confirm)" name="passwordConfirm" />
				<@form_submit label="Update" />
			</@form>
			
			<#if data.account.mailChange?has_content>
				<#local mc = data.account.mailChange>
				<#if mc.used>
					<@form name="mail-form" action="set-address" hash="basics">
						<@form_part title="Change e-mail address" />
						<tr class="form-field">
							<th>Current address:</th>
							<td>${data.account.address?xhtml}</td> 
						</tr>
						<@form_extra>
							You will be able to request an e-mail address change at ${mc.until?string("HH:mm:ss (ZZZZZ)")}.
						</@form_extra>
					</@form>
				<#else>
					<@form name="mail-form" action="confirm-set-address" hash="basics">
						<@form_part title="Change e-mail address" />
						<tr class="form-field">
							<th>Current address:</th>
							<td>${data.account.address?xhtml}</td> 
						</tr>
						<@form_extra>
							You are changing your address to <em>${mc.newAddress?xhtml}</em>.<br/>
							Your request will expire at ${mc.until?string("HH:mm (ZZZZZ)")}.
						</@form_extra>
						<#if data.codeError!false>
							<@form_error>Incorrect confirmation code</@form_error>
						</#if>
						<@form_text name="code" id="mail-confirmation-code" value="${data.code!}" maxLength=64 label="Confirmation code" />
						<@form_extended_submit label="Change address">
							<@ff_submit label="Cancel" name="cancel-set-address" />
						</@form_extended_submit>
					</@form>
				</#if>
			<#else>
				<@form name="mail-form" action="set-address" hash="basics">
					<@form_part title="Change e-mail address" />
					<#if data.mailAuthError!false>
						<@form_error>Incorrect password</@form_error>
					</#if>
					<tr class="form-field">
						<th>Current address:</th>
						<td>${data.account.address?xhtml}</td> 
					</tr>
					<@form_pwd label="Current password" name="password" id="current-2" />
					<#switch data.mailError!"">
						<#case "EMPTY">
							<@form_error>Empty e-mail address.</@form_error>
							<#break>
						<#case "INVALID">
							<@form_error>Invalid e-mail address.</@form_error>
							<#break>
						<#case "IN_USE">
							<@form_error>This e-mail address is already associated with an account.</@form_error>
							<#break>
						<#case "SEND_FAIL">
							<@form_error>Unable to send confirmation code to this address.</@form_error>
							<#break>
						<#case "MISMATCH">
							<@form_error>E-mail address field and confirmation field do not match.</@form_error>
							<#break>
						<#default>
							<@form_extra>The e-mail address <em>MUST</em> exist.</@form_extra>
					</#switch>
					<@form_text label="E-mail address" maxLength=128 name="mail" value=data.mail! />
					<@form_text label="E-mail address (confirm)" maxLength=128 name="mailConfirm" value=data.mail! />
					<@form_submit label="Update" />
				</@form>
			</#if>
		</@tab>

		<@tab id="status" title="Status">
			<@form name="game-credits" action="no-action" hash="status">
				<@form_part title="Game credits" />
				<tr class="form-field">
					<th>Game credits:</th>
					<td>${data.account.gameCredits?string(',##0')}</td> 
				</tr>
				<@form_extra>
					Game credits cannot be used at this time. They will however be accumulated
					throughout the LWB6 development cycle. You obtain them by reporting bugs.
				</@form_extra>
			</@form>

			<@form name="vacation-form" action="toggle-vacation" hash="status">
				<@form_part title="Vacation mode" />
				<tr class="form-field">
					<th>Vacation credits:</th>
					<td>${data.account.vacCredits?string(',##0')}</td> 
				</tr>
				<tr class="form-field">
					<th>Maximal duration:</th>
					<td>
						<#if data.account.vacTime?has_content>
							approximately <@game_duration record=data.account.vacTime />
						<#else>
							less than an hour
						</#if>
					</td> 
				</tr>
				<#if data.account.vacStart?has_content>
					<tr class="form-field">
						<th>Vacation start:</th>
						<td>${data.account.vacStart?string("HH:mm (yyyy-MM-dd ZZZZZ)")}</td> 
					</tr>
					<#if data.page.special == 'v'>
						<@form_submit label="Exit vacation mode" />
					<#else>
						<@form_submit label="Cancel vacation mode" />
					</#if>
				<#elseif !data.page.special?has_content && data.account.vacTime?has_content>
					<@form_submit label="Enter vacation mode" />
				</#if>
			</@form>

			<#if !data.page.special?has_content>
				<@form name="quit-form" action="quit" hash="status">
					<@form_part title="Quit game" />
					<@form_extra>
						If you click the button below, your account will be disabled after 24 hours. You will be able to re-enable it for the next 6 months, after which it will be deleted.
					</@form_extra>
					<@form_text label="Reason for quitting (optional)" name="reason" value="" />
					<@form_submit label="Quit game" />
				</@form>
			<#elseif data.page.special == 'q'>
				<@form name="quit-form" action="cancel-quit" hash="status">
					<@form_part title="Quit game" />
					<@form_extra>
						Your account will be disabled at ${data.account.quitGame?string("HH:mm (yyyy-MM-dd ZZZZZ)")}.
					</@form_extra>
					<@form_submit label="Cancel" />
				</@form>
			</#if>
		</@tab>

	</@tabs>
</@page>
</#macro>