<#macro render_fleet fleet>
	<td class="selector"><input type="checkbox" name="selection" value="${fleet.id}" checked="checked" /></td>
	<td class="name">
		<#if fleet.name?has_content && fleet.name != "">
			${fleet.name?xhtml}
		<#else>
			<em>(unnamed fleet)</em>
		</#if>
	</td>	
	<td class="power">${fleet.power?string(',##0')}</td>
	<td class="fltime"><@duration rTime=fleet.flightTime gTime=fleet.gameFlightTime /></td>
	<td class="status">
		<#switch fleet.status>
			<#case 'DEPLOYING'><abbr title="This fleet is undergoing its initial deployment from the space docks.">DPL</abbr><#break>
			<#case 'REDEPLOYING'><abbr title="This fleet is re-deploying after arrival or mode change.">RDP</abbr><#break>
			<#case 'REDIRECTING'><abbr title="This fleet is changing course.">RDP</abbr><#break>
			<#case 'AVAILABLE'><abbr title="This fleet is available, it can be given any order.">AVL</abbr><#break>
		</#switch>
	</td>
	<td class="cloc">
		(${fleet.x?string("0.00")};${fleet.y?string("0.00")})<br/>
		<#if fleet.nearest?has_content>
			<#if fleet.atPlanet>At<#else>Near</#if>
			<a href="planet-${fleet.nearest.id}">${fleet.nearest.name?xhtml}</a>
		<#else>
			In outer space
		</#if>
	</td>
</#macro>
<#macro render_selected_fleets fleets>
	<h3>Selected fleets</h3>
	<table class="selected-fleets">
		<tr>
			<td class="selector">&nbsp;</td>
			<th class="name">Name</th>
			<th class="power"><abbr title="Fleet power">P</abbr></th>
			<th class="fltime"><abbr title="Orbit-to-orbit flight time">FT</abbr></th>
			<th class="status"><abbr title="Fleet status">S</abbr></th>
			<th class="cloc">Location</th>
		</tr>
		<#list fleets as fleet>
			<tr>
				<@render_fleet fleet=fleet />
			</tr>
		</#list>
	</table>
</#macro>
<#macro render_move_fleets>
	<h3>New orders</h3>
	<#if data.error>
		<@standalone_error>This planet does not exist.</@standalone_error>
	</#if>
	<p>
		Move selected fleets to
		<@ff_text name="destination" id="destination" value="${data.destination!?xhtml}" maxLength=20 size=21 />
		for
		<@ff_select name="mode" id="mode">
			<@form_option text="defence" selected=( !data.mode!false ) value="0" />
			<@form_option text="attack" selected=( data.mode!false ) value="1" />
		</@ff_select>
		&nbsp;
		<@ff_submit label="Ok" />
		<@ff_submit label="Cancel" name="cancel" />
	</p>
</#macro>
<#macro render_rename_fleets>
	<h3>Rename fleets</h3>
	<#if data.error>
		<@standalone_error>This name is invalid.</@standalone_error>
	</#if>
	<p>
		Rename selected fleets to
		<@ff_text name="name" id="name" value="${data.name!?xhtml}" maxLength=40 size=21 />
		&nbsp;
		<@ff_submit label="Ok" />
		<@ff_submit label="Cancel" name="cancel" />
	</p>
</#macro>
<#macro render_set_fleets_mode>
	<h3>Set fleets mode</h3>
	<p>
		Assign selected fleets to
		<#if data.attack>attack<#else>defence</#if>?
		&nbsp;
		<@ff_submit label="Yes" />
		<@ff_submit label="No" name="cancel" />
	</p>
	<input type="hidden" name="attack" value="<#if data.attack>1<#else>0</#if>" />
</#macro>
<#macro render_disband_fleets>
	<h3>Disband fleets</h3>
	<p>
		Are you sure?
		&nbsp;
		<@ff_submit label="Yes" />
		<@ff_submit label="No" name="cancel" />
	</p>
</#macro>
<#macro render>
<#switch dataType>
	<#case 'MoveFleetsResponse'>
		<#local title="Move fleets">
		<#local action="move-fleets.action">
		<#break>
	<#case 'RenameFleetsResponse'>
		<#local title="Rename fleets">
		<#local action="rename-fleets.action">
		<#break>
	<#case 'SetFleetsModeResponse'>
		<#local title="Set fleets mode">
		<#local action="set-fleets-mode.action">
		<#break>
	<#case 'DisbandFleetsResponse'>
		<#local title="Disband fleets">
		<#local action="disband-fleets.action">
		<#break>
</#switch>
<@page title=title>
	<form action="${action}" method="post">
		<@render_selected_fleets fleets=data.fleets />
		<#switch dataType>
			<#case 'MoveFleetsResponse'>
				<@render_move_fleets />
				<#break>
			<#case 'RenameFleetsResponse'>
				<@render_rename_fleets />
				<#break>
			<#case 'SetFleetsModeResponse'>
				<@render_set_fleets_mode />
				<#break>
			<#case 'DisbandFleetsResponse'>
				<@render_disband_fleets />
				<#break>
		</#switch>
	</form>
</@page>
</#macro>