<#macro render>
<@page title="Password recovery">
	<@tabs>

		<@tab title="Request password recovery" id="request-recovery">
			<#if data.sent?has_content>
				<p>An email has been sent to your address, <strong>${data.mail?xhtml}</strong></p>
				<p>It contains a one-time authentication code you can use to change your password.</p>
				<p>The authentication code will expire in one hour, and you will not be able to request another code until it expires.</p>
			<#else>

				<@form name="req-pwd-recovery" action="request-password-recovery">
					<#switch data.status!>
						<#case "INVALID_INPUT">
							<@form_error>Invalid e-mail address.</@form_error>
							<#break>
						<#case "ACCOUNT_NOT_FOUND">
							<@form_error>Unknown e-mail address.</@form_error>
							<#break>
						<#case "ACCOUNT_STATUS">
							<@form_error>The account's status prevents password recovery.</@form_error>
							<#break>
						<#case "RECOVERY_IN_PROGRESS">
							<@form_error>A password recovery request has been made within the last hour.</@form_error>
							<#break>
						<#case "MAIL_ERROR">
							<@form_error>Unable to send e-mail.</@form_error>
							<#break>
					</#switch>
					<@form_text label="E-mail address" name="mail" id="req-mail" value=data.mail! maxLength=128 />
					<@form_submit label="Request password recovery" extraClass="" />
				</@form>

			</#if>
		</@tab>

		<@tab id="confirm-recovery" title="Confirm password recovery">
			<@form name="conf-pwd-recovery" action="confirm-password-recovery" hash="confirm-recovery">
				<#switch data.cStatus!>
					<#case "INVALID_MAIL">
						<@form_error>Invalid e-mail address.</@form_error>
						<#break>
					<#case "NOT_FOUND">
						<@form_error>Unknown e-mail address or code.</@form_error>
						<#break>
					<#case "ACCOUNT_STATUS">
						<@form_error>The account's status prevents password recovery.</@form_error>
						<#break>
					<#case "WEAK_PASSWORD">
						<@form_error>New password too weak.</@form_error>
						<#break>
					<#case "MISMATCH_PASSWORD">
						<@form_error>Password and password confirmation did not match.</@form_error>
						<#break>
					<#case "PROHIBITED">
						<@form_error>You may not use this password.</@form_error>
						<#break>
				</#switch>
				<@form_text label="E-mail address" name="mail" id="conf-mail" value=data.cMail! maxLength=128 />
				<@form_text label="Confirmation code" name="code" id="conf-code" value=data.cCode! maxLength=64 />
				<@form_pwd label="New password" name="password" id="conf-pwd1" />
				<@form_pwd label="Confirm new password" name="passwordConfirm" id="conf-pwd2" />
				<@form_submit label="Set new password" />
			</@form>
		</@tab>

	</@tabs>
</@page>
</#macro>