<#macro render_preference pref>
	<#switch pref.type>
		<#case 'BOOLEAN'>
			<@form_select label=pref.name name=pref.id id="pref-${pref.id}">
				<@form_option value="0" text="Non" selected=( pref.value != "1" )/>
				<@form_option value="1" text="Oui" selected=( pref.value == "1" )/>
			</@form_select>
			<#break>
		<#case 'INTEGER'>
		<#case 'STRING'>
			<@form_text label=pref.name name=pref.id id="pref-${pref.id}" value=pref.value />
			<#break>
		<#case 'CHOICE'>
			<@form_select label=pref.name name=pref.id id="pref-${pref.id}">
				<#list pref.choices as choice>
					<@form_option value=choice.value text=choice.display selected=( pref.value == choice.value )/>
				</#list>
			</@form_select>
			<#break>
	</#switch>
	<#if pref.description?has_content>
		<@form_extra>${pref.description?xhtml}</@form_extra>
	</#if>
</#macro>
<#macro render_prefs_category category>
	<@form_part title=category.name />
	<#list category.preferences as p>
		<@render_preference pref=p />
	</#list>
</#macro>
<#macro render>
<@page title="Compte">
	<@tabs>

		<@tab id="prefs" title="Préférences">
			<@form name="prefs-form" action="set-preferences" hash="prefs">
				<#list data.account.preferences as c>
					<@render_prefs_category category=c />
				</#list>
				<@form_extended_submit label="Changer les préférences">
					<@ff_submit label="Défauts" name="load-default-preferences" />
				</@form_extended_submit>
			</@form>
		</@tab>

		<@tab id="basics" title="Identité">
			<@form name="lang-form" action="set-language" hash="basics">
				<@form_part title="Langue du compte" />
				<@form_select name="language" label="Langue ">
					<#list data.account.supportedLanguages.languages as lang>
						<@form_option text=lang.name value=lang.id selected=(lang.id == data.account.language) />
					</#list>
				</@form_select>
				<@form_submit label="Mettre à jour" />
			</@form>

			<@form name="pwd-form" action="set-password" hash="basics">
				<@form_part title="Changement de mot de passe" />
				<#if data.authError!false>
					<@form_error>Mot de passe incorrect</@form_error>
				</#if>
				<@form_pwd label="Mot de passe actuel " name="current" />
				<#switch data.passwordError!"">
					<#case "EMPTY">
						<@form_error>Mot de passe vide.</@form_error>
						<#break>
					<#case "TOO_WEAK">
						<@form_error>Ce mot de passe est trop faible.</@form_error>
						<#break>
					<#case "MISMATCH">
						<@form_error>Le mot de passe et sa confirmation ne sont pas identiques.</@form_error>
						<#break>
					<#case "PROHIBITED">
						<@form_error>Vous ne pouvez pas utiliser ce mot de passe ici.</@form_error>
						<#break>
					<#default>
						<@form_extra>Au moins 6 caractères, incluant à la fois des lettres et des chiffres. Bonus pour les caractères spéciaux, les espaces et les mots de passe extrèmement longs.</@form_extra>
				</#switch>
				<@form_pwd label="Mot de passe " name="password" />
				<@form_pwd label="Mot de passe (confirmation) " name="passwordConfirm" />
				<@form_submit label="Mettre à jour" />
			</@form>
			
			<#if data.account.mailChange?has_content>
				<#local mc = data.account.mailChange>
				<#if mc.used>
					<@form name="mail-form" action="set-address" hash="basics">
						<@form_part title="Changement d'adresse e-mail" />
						<tr class="form-field">
							<th>Adresse actuelle :</th>
							<td>${data.account.address?xhtml}</td> 
						</tr>
						<@form_extra>
							Vous pourrez demander un changement d'adresse e-mail à  ${mc.until?string("HH:mm:ss (ZZZZZ)")}.
						</@form_extra>
					</@form>
				<#else>
					<@form name="mail-form" action="confirm-set-address" hash="basics">
						<@form_part title="Changement d'adresse e-mail" />
						<tr class="form-field">
							<th>Adresse actuelle :</th>
							<td>${data.account.address?xhtml}</td> 
						</tr>
						<@form_extra>
							Vous êtes en train de demander un changement d'adresse e-mail pour <em>${mc.newAddress?xhtml}</em>.<br/>
							Votre requête va expirer à ${mc.until?string("HH:mm (ZZZZZ)")}.
						</@form_extra>
						<#if data.codeError!false>
							<@form_error>Code de confirmation erroné</@form_error>
						</#if>
						<@form_text name="code" id="mail-confirmation-code" value="${data.code!}" maxLength=64 label="Code de confirmation" />
						<@form_extended_submit label="Changer d'adresse">
							<@ff_submit label="Annuler" name="cancel-set-address" />
						</@form_extended_submit>
					</@form>
				</#if>
			<#else>
				<@form name="mail-form" action="set-address" hash="basics">
					<@form_part title="Changement d'adresse e-mail" />
					<#if data.mailAuthError!false>
						<@form_error>Mot de pase erroné</@form_error>
					</#if>
					<tr class="form-field">
						<th>Current address:</th>
						<td>${data.account.address?xhtml}</td> 
					</tr>
					<@form_pwd label="Mot de passe actuel " name="password" id="current-2" />
					<#switch data.mailError!"">
						<#case "EMPTY">
							<@form_error>Adresse e-mail vide.</@form_error>
							<#break>
						<#case "INVALID">
							<@form_error>Adresse e-mail invalide.</@form_error>
							<#break>
						<#case "IN_USE">
							<@form_error>Cette adresse e-mail est déjà associée à un compte.</@form_error>
							<#break>
						<#case "SEND_FAIL">
							<@form_error>Impossible d'envoyer un code de confirmation à cette adresse.</@form_error>
							<#break>
						<#case "MISMATCH">
							<@form_error>L'adresse e-mail et sa confirmation ne sont pas identiques.</@form_error>
							<#break>
						<#default>
							<@form_extra>L'adresse e-mail <em>DOIT</em> exister.</@form_extra>
					</#switch>
					<@form_text label="Adresse e-mail " maxLength=128 name="mail" value=data.mail! />
					<@form_text label="Adresse e-mail (confirmation) " maxLength=128 name="mailConfirm" value=data.mail! />
					<@form_submit label="Mettre à jour" />
				</@form>
			</#if>
		</@tab>

		<@tab id="status" title="Statut">
			<@form name="game-credits" action="no-action" hash="status">
				<@form_part title="Crédits de jeu" />
				<tr class="form-field">
					<th>Crédits de jeu:</th>
					<td>${data.account.gameCredits?string(',##0')}</td> 
				</tr>
				<@form_extra>
					Les crédits de jeu ne peuvent pas être utilisés actuellement. Par contre, ils seront accumulés au cours du cycle de développement de LWB6. 
					Vous les obtenez en rapportant des bugs.
				</@form_extra>
			</@form>

			<@form name="vacation-form" action="toggle-vacation" hash="status">
				<@form_part title="Mode vacances" />
				<tr class="form-field">
					<th>Crédits de vacances :</th>
					<td>${data.account.vacCredits?string(',##0')}</td> 
				</tr>
				<tr class="form-field">
					<th>Durée maximale :</th>
					<td>
						<#if data.account.vacTime?has_content>
							approximativement <@game_duration record=data.account.vacTime />
						<#else>
							moins d'une heure
						</#if>
					</td> 
				</tr>
				<#if data.account.vacStart?has_content>
					<tr class="form-field">
						<th>Début de vacances :</th>
						<td>${data.account.vacStart?string("HH:mm (yyyy-MM-dd ZZZZZ)")}</td> 
					</tr>
					<#if data.page.special == 'v'>
						<@form_submit label="Quitter le mode vacances" />
					<#else>
						<@form_submit label="Annuler le mode vacances" />
					</#if>
				<#elseif !data.page.special?has_content && data.account.vacTime?has_content>
					<@form_submit label="Entrer en mode vacances" />
				</#if>
			</@form>

			<#if !data.page.special?has_content>
				<@form name="quit-form" action="quit" hash="status">
					<@form_part title="Quitter le jeu" />
					<@form_extra>
						Si vous cliquez le bouton ci-dessous, votre compte sera désactivé après un délai de 24h. Vous pourrez le réactiver pendant les 6 prochains mois, délai après lequel il sera supprimé.
					</@form_extra>
					<@form_text label="Raison de quitter (optionel) " name="reason" value="" />
					<@form_submit label="Quitter le jeu" />
				</@form>
			<#elseif data.page.special == 'q'>
				<@form name="quit-form" action="cancel-quit" hash="status">
					<@form_part title="Quitter le jeu" />
					<@form_extra>
						Votre compte sera désactivé à ${data.account.quitGame?string("HH:mm (yyyy-MM-dd ZZZZZ)")}.
					</@form_extra>
					<@form_submit label="Annuler" />
				</@form>
			</#if>
		</@tab>

	</@tabs>
</@page>
</#macro>