package com.deepclone.lw.web.main.game;


import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.account.GetAccountResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class AccountPage
		extends PageControllerBase
{
	@RequestMapping( "/account" )
	public String view( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		GetAccountResponse data = pSession.getAccount( );
		return this.render( model , "game" , data.getAccount( ).getLanguage( ) , "account" , data );
	}


	@RequestMapping( "/set-language.action" )
	public String setLanguage( HttpServletRequest request , Model model , @RequestParam( "language" ) String newLanguage )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( PlayerSession.class , request ).setLanguage( newLanguage );
		return this.redirect( "account" );
	}


	@RequestMapping( "/set-password.action" )
	public String setPassword( HttpServletRequest request , Model model ,
			@RequestParam( "current" ) String currentPassword , @RequestParam( "password" ) String newPass1 ,
			@RequestParam( "passwordConfirm" ) String newPass2 )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		String challenge = pSession.createAuthenticationChallenge( );

		GetAccountResponse data = pSession.setPassword( currentPassword , challenge , newPass1 , newPass2 );
		if ( data.getAccount( ) == null ) {
			return this.redirect( "account" );
		}
		return this.render( model , "game" , data.getAccount( ).getLanguage( ) , "account" , data );
	}


	@RequestMapping( "/set-address.action" )
	public String setAddress( HttpServletRequest request , Model model , @RequestParam( "password" ) String password ,
			@RequestParam( "mail" ) String newAddr1 , @RequestParam( "mailConfirm" ) String newAddr2 )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		String challenge = pSession.createAuthenticationChallenge( );

		GetAccountResponse data = pSession.setAddress( password , challenge , newAddr1 , newAddr2 );
		if ( data.getAccount( ) == null ) {
			return this.redirect( "account" );
		}
		return this.render( model , "game" , data.getAccount( ).getLanguage( ) , "account" , data );
	}


	@RequestMapping( "/confirm-set-address.action" )
	public String confirmSetAddress( HttpServletRequest request , Model model ,
			@RequestParam( value = "cancel-set-address" , required = false ) String cancel ,
			@RequestParam( "code" ) String code )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		if ( cancel != null ) {
			pSession.cancelAddressChange( );
			return this.redirect( "account" );
		}

		GetAccountResponse data = pSession.confirmAddressChange( code );
		if ( data.getAccount( ) == null ) {
			return this.redirect( "account" );
		}
		return this.render( model , "game" , data.getAccount( ).getLanguage( ) , "account" , data );
	}


	@RequestMapping( "/set-preferences.action" )
	public String confirmSetAddress( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		Map< String , Object > input = this.getInput( request );

		if ( input.containsKey( "load-default-preferences" ) ) {
			pSession.loadDefaultPreferences( );
		} else {
			Map< String , String > prefs = new HashMap< String , String >( );
			for ( Map.Entry< String , Object > entry : input.entrySet( ) ) {
				if ( ! ( entry.getValue( ) instanceof String[] ) ) {
					continue;
				}

				String[] values = (String[]) entry.getValue( );
				if ( values.length < 1 ) {
					continue;
				}
				prefs.put( entry.getKey( ) , values[ 0 ] );
			}
			pSession.setPreferences( prefs );
		}

		return this.redirect( "account" );
	}


	@RequestMapping( "/quit.action" )
	public String quit( HttpServletRequest request , Model model , @RequestParam( "reason" ) String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( PlayerSession.class , request ).setQuit( reason );
		return this.redirect( "account" );
	}


	@RequestMapping( "/cancel-quit.action" )
	public String cancelQuit( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( PlayerSession.class , request ).cancelQuit( );
		return this.redirect( "account" );
	}


	@RequestMapping( "/toggle-vacation.action" )
	public String toggleVacation( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( PlayerSession.class , request ).toggleVacation( );
		return this.redirect( "account" );
	}
}
