package com.deepclone.lw.web.main.game;


import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.battles.GetBattleResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class BattlePages
		extends PageControllerBase
{

	@RequestMapping( "/battle-{battleId}-latest" )
	public String getBattle( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable( "battleId" ) String sBattleId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long battle;
		try {
			battle = Long.parseLong( sBattleId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		GetBattleResponse data = this.getSession( PlayerSession.class , request ).getBattle( battle );
		if ( data.getBattle( ) == null ) {
			return this.redirect( "overview" );
		}
		return this.render( model , "game" , language , "battle" , data );
	}


	@RequestMapping( "/battle-{battleId}-at-{tickId}" )
	public String getBattle( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable( "battleId" ) String sBattleId , @PathVariable( "tickId" ) String sTickId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long battle;
		try {
			battle = Long.parseLong( sBattleId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		long tick;
		try {
			tick = Long.parseLong( sTickId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "battle-" + battle + "-latest" );
		}

		GetBattleResponse data = this.getSession( PlayerSession.class , request ).getBattle( battle , tick );
		if ( data.getBattle( ) == null ) {
			return this.redirect( "overview" );
		}
		return this.render( model , "game" , language , "battle" , data );
	}


	@RequestMapping( value = "/battle-{battleId}-at.action" , method = RequestMethod.POST )
	public String getBattlePost( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @PathVariable( "battleId" ) String sBattleId , @RequestParam( "tick" ) String sTickId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long battle;
		try {
			battle = Long.parseLong( sBattleId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		long tick;
		try {
			tick = Long.parseLong( sTickId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "battle-" + battle + "-latest" );
		}

		return this.redirect( "battle-" + battle + "-at-" + tick );
	}


	@RequestMapping( value = "/battles" )
	public String getBattles( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.getBattles( request , language , model , 0 );
	}


	@RequestMapping( value = "/battles-{page}" )
	public String getBattles( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable( "page" ) String sPage )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int page;
		try {
			page = Integer.parseInt( sPage );
		} catch ( NumberFormatException e ) {
			page = 0;
		}

		return this.getBattles( request , language , model , page );
	}


	private String getBattles( HttpServletRequest request , String language , Model model , int page )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "battles" , pSession.getBattles( page ) );
	}
}
