package com.deepclone.lw.web.main.game;


import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.bt.data.BugEvent;
import com.deepclone.lw.cmd.player.bt.*;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.msgs.MessageFormatter;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.BugTrackerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class BugTrackerPages
		extends BugTrackerBase
{

	private static final int perPage = 10;

	private MessageFormatter formatter;


	@Autowired( required = true )
	public void setFormatter( MessageFormatter formatter )
	{
		this.formatter = formatter;
	}


	@RequestMapping( "/bugtrack" )
	public String listBugs( HttpServletRequest request , Model model , @ModelAttribute( "language" ) String language ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		ListBugsResponse response = pSession.listBugs( query.status , query.ownOnly , query.first , perPage );
		return this.render( model , "game" , language , "bugsList" , response );
	}


	@RequestMapping( "/report-bug" )
	public String showReportForm( HttpServletRequest request , Model model ,
			@ModelAttribute( "language" ) String language ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		GamePageData page = pSession.listPlanets( ).getPage( );
		return this.renderMap( model , "game" , language , "bugsReport" , "page" , page , "query" , query );
	}


	@RequestMapping( value = "/report-bug.action" , method = RequestMethod.POST )
	public String postReport( HttpServletRequest request , Model model , @ModelAttribute( "language" ) String language ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @RequestParam( "title" ) String title ,
			@RequestParam( "description" ) String description )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		ReportBugResponse response = pSession.reportBug( title , description );

		if ( response.getTitle( ) == null ) {
			// Successful post
			String rTo = "bug-" + response.getBugId( ) + this.makeGetParams( query );
			return this.redirect( rTo );
		}

		response = new ReportBugResponse( response , query );
		return this.render( model , "game" , language , "bugsReport" , response );
	}


	@RequestMapping( value = "/bug-{id}" )
	public String viewBug( HttpServletRequest request , Model model , @ModelAttribute( "language" ) String language ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		ViewBugResponse response = this.getSession( PlayerSession.class , request ).getBugReport( bugId );
		if ( response.getReport( ) == null ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		response = new ViewBugResponse( response , query );
		return this.displayReport( model , language , response );
	}


	@RequestMapping( value = "/bug-{id}-comment.action" , method = RequestMethod.POST )
	public String commentBug( HttpServletRequest request , Model model , @ModelAttribute( "language" ) String language ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "comment" ) String comment )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		PostCommentResponse response = this.getSession( PlayerSession.class , request )
				.postBugComment( bugId , comment );
		if ( response.isPosted( ) ) {
			String rTo = "bug-" + bugId + this.makeGetParams( query );
			return this.redirect( rTo );
		} else if ( response.getReport( ) == null ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		response = new PostCommentResponse( response , query );
		return this.displayReport( model , language , response );
	}


	private String displayReport( Model model , String language , ViewBugResponse response )
	{
		for ( BugEvent event : response.getEvents( ) ) {
			if ( event.getTitle( ) != null ) {
				event.setTitle( this.formatter.cleanMessage( event.getTitle( ) ) );
			}
			if ( event.getContents( ) != null ) {
				event.setContents( this.formatter.formatMessage( event.getContents( ) , false ) );
			}
		}
		return this.render( model , "game" , language , "bugsView" , response );
	}

}
